// $Id: dcel.C,v 1.3 1995/07/13 11:28:12 geert Exp $

#include <assert.h>
#include <dcel.h>

void dcel_vertex::make_empty()
{
    an_incident_edge = No_dcel_edge_id;
    edge_count = 0;
    component = No_dcel_comp_id;
    info = 0;
    ident = 0;
}

void dcel_edge::make_empty()
{
    sym = No_dcel_edge_id;
    next = No_dcel_edge_id;
    prev = No_dcel_edge_id;
    origin = No_dcel_vertex_id;
    incident_cell = No_dcel_cell_id;
    info = 0;
    ident = 0;
}

void dcel_cell::make_empty()
{
    outer_component = No_dcel_comp_id;
    next_cell = No_dcel_cell_id;
    an_incident_edge = No_dcel_edge_id;
    incident_edge_count = 0;
    an_edge_around = No_dcel_edge_id;
    inner_components = No_dcel_comp_id;
    info = 0;
    ident = 0;
}

void dcel_comp::make_empty()
{
    next_component = No_dcel_comp_id;
    outer_cell = No_dcel_cell_id;
    a_vertex = No_dcel_vertex_id;
    inner_cells = No_dcel_cell_id;
    an_outside_edge = No_dcel_edge_id;
    member_vertex_count = 0;
    info = 0;
    ident = 0;
}

dcel_vt_reray::dcel_vt_reray()
{
    free_element = No_dcel_vertex_id;
    in_use = 0;
}

dcel_vertex_id dcel_vt_reray::new_element()
{
    dcel_vertex_id result;
    in_use++;
    result = free_element;
    if (result == No_dcel_vertex_id) {
	grow(1);
	result.index = size()-1;
    } else {
    //unchain from free vertices
	free_element = (*this)[result].next;
	(*this)[result].make_empty();
    }
    return result;
}

int dcel_vt_reray::is_free(dcel_vertex_id id) const
{
    return (*this)[id].is_free();
}

void dcel_vt_reray::delete_element(dcel_vertex_id id)
{
    if (is_free(id))
	return;
    (*this)[id].next = free_element;
    free_element = id;
    (*this)[id].mark_free();
    in_use--;
}

dcel_edge_reray::dcel_edge_reray()
{
    free_element = No_dcel_edge_id;
    in_use = 0;
}

dcel_edge_id dcel_edge_reray::new_element()
{
    dcel_edge_id result;
    in_use++;
    result = free_element;
    if (result == No_dcel_edge_id) {
	grow(1);
	result.index = size()-1;
    } else {
    //unchain from free vertices
	free_element = (*this)[result].next;
	(*this)[result].make_empty();
    }
    return result;
}

int dcel_edge_reray::is_free(dcel_edge_id id) const
{
    return (*this)[id].is_free();
}

void dcel_edge_reray::delete_element(dcel_edge_id id)
{
    if (is_free(id))
	return;
    (*this)[id].next = free_element;
    free_element = id;
    (*this)[id].mark_free();
    in_use--;
}

dcel_cell_reray::dcel_cell_reray()
{
    free_element = No_dcel_cell_id;
    in_use = 0;
}

dcel_cell_id dcel_cell_reray::new_element()
{
    dcel_cell_id result;
    in_use++;
    result = free_element;
    if (result == No_dcel_cell_id) {
	grow(1);
	result.index = size()-1;
    } else {
    //unchain from free vertices
	free_element = (*this)[result].next_cell;
	(*this)[result].make_empty();
    }
    return result;
}

int dcel_cell_reray::is_free(dcel_cell_id id) const
{
    return (*this)[id].is_free();
}

void dcel_cell_reray::delete_element(dcel_cell_id id)
{
    if (is_free(id))
	return;
    (*this)[id].next_cell = free_element;
    free_element = id;
    (*this)[id].mark_free();
    in_use--;
}

dcel_comp_reray::dcel_comp_reray()
{
    free_element = No_dcel_comp_id;
    in_use = 0;
}

dcel_comp_id dcel_comp_reray::new_element()
{
    dcel_comp_id result;
    in_use++;
    result = free_element;
    if (result == No_dcel_comp_id) {
	grow(1);
	result.index = size()-1;
    } else {
    //unchain from free vertices
	free_element = (*this)[result].next_component;
	(*this)[result].make_empty();
    }
    return result;
}

int dcel_comp_reray::is_free(dcel_comp_id id) const
{
    return (*this)[id].is_free();
}

void dcel_comp_reray::delete_element(dcel_comp_id id)
{
    if (is_free(id))
	return;
    (*this)[id].next_component = free_element;
    free_element = id;
    (*this)[id].mark_free();
    in_use--;
}

dcedgelist::dcedgelist()
{
    _outer_cell = new_cell();
}

dcedgelist::dcedgelist(int vtn, int edgen, int celln, int compn)
{
    _vertices.newsize(vtn); _vertices.newsize(0);
    _edges.newsize(edgen); _edges.newsize(0);
    _cells.newsize(celln); _cells.newsize(0);
    _components.newsize(compn); _components.newsize(0);
    _outer_cell = new_cell();
}

void dcedgelist::clear()
{
// ??????? inefficient implmentation
    dcel_vertex_id vt_to_destroy, tmp_vertex;
    dcel_edge_id edge_to_destroy, tmp_edge;
    dcel_cell_id cell_to_destroy, tmp_cell;
    dcel_comp_id comp_to_destroy, tmp_comp;
    vt_to_destroy = first_vertex();
    while (vt_to_destroy != No_dcel_vertex_id) {
	tmp_vertex = next_vertex(vt_to_destroy);
	delete_vertex(vt_to_destroy);
	vt_to_destroy = tmp_vertex;
    }
    edge_to_destroy = first_edge();
    while (edge_to_destroy != No_dcel_edge_id) {
	tmp_edge = next_edge(edge_to_destroy);
	delete_edge(edge_to_destroy);
	edge_to_destroy = tmp_edge;
    }
    cell_to_destroy = first_cell();
    while (cell_to_destroy != No_dcel_cell_id) {
	tmp_cell = next_cell(cell_to_destroy);
	delete_cell(cell_to_destroy);
	cell_to_destroy = tmp_cell;
    }

    comp_to_destroy = first_component();
    while (comp_to_destroy != No_dcel_comp_id) {
	tmp_comp = next_component(comp_to_destroy);
	delete_component(comp_to_destroy);
	comp_to_destroy = tmp_comp;
    }
    _outer_cell = new_cell();
}

dcel_vertex_id dcedgelist::new_vertex()
{
    _idflags.set(flags8::flag0);
    return _vertices.new_element();
}

void dcedgelist::delete_vertex(dcel_vertex_id id)
{
    assert(is_valid_vertex(id));
    dcel_edge_id edge;
    _idflags.set(flags8::flag0);
    set_component(id, No_dcel_comp_id);
    while ((edge = an_edge_from(id)) != No_dcel_edge_id)
	set_origin(edge, No_dcel_vertex_id);
    _vertices.delete_element(id);
}

bool dcedgelist::is_valid_vertex(dcel_vertex_id id) const
{
    if (id.index<0 || id.index >= _vertices.size())
	return false;
    return !(_vertices[id].is_free());
}

dcel_edge_id dcedgelist::new_edge()
{
    _idflags.set(flags8::flag1);
    return _edges.new_element();
}

void dcedgelist::delete_edge(dcel_edge_id id)
{
    assert(is_valid_edge(id));
    dcel_cell_id cell;
    dcel_comp_id comp;
    _idflags.set(flags8::flag1);
    set_origin(id, No_dcel_vertex_id);
    make_twins(id, No_dcel_edge_id);
    make_consecutive(id, No_dcel_edge_id);
    make_consecutive(No_dcel_edge_id, id);
    cell = incident_cell(id);
    if (cell != No_dcel_cell_id && id == _cells[cell].an_edge_around)
	_cells[cell].an_edge_around = No_dcel_edge_id;
    set_incident_cell(id, No_dcel_cell_id);
    comp = component(id);
    if (comp != No_dcel_comp_id && id == _components[comp].an_outside_edge)
	_components[comp].an_outside_edge = No_dcel_edge_id;
    _edges.delete_element(id);
}

bool dcedgelist::is_valid_edge(dcel_edge_id id) const
{
    if (id.index<0 || id.index >= _edges.size())
	return false;
    return !(_edges[id].is_free());
}

dcel_cell_id dcedgelist::new_cell()
{
    _idflags.set(flags8::flag2);
    return _cells.new_element();
}

void dcedgelist::delete_cell(dcel_cell_id id)
{
    assert(is_valid_cell(id));
    dcel_comp_id comp;
    dcel_edge_id edge, cur;
    _idflags.set(flags8::flag2);
    if (outer_cell() == id)
	set_outer_cell(No_dcel_cell_id);
// remove possible incidence relations from edges on outer boundary.
    if (_cells[id].incident_edge_count > 0) {
	edge = _cells[id].an_edge_around;
	if (edge != No_dcel_edge_id) {
	    cur = edge;
	    do {
		if (incident_cell(cur) == id)
		    set_incident_cell(cur, No_dcel_cell_id);
		cur = following(cur);
	    } while (cur != edge && cur != No_dcel_edge_id);
	}
    }
    set_surrounding_component(id, No_dcel_comp_id);
    while ((comp = first_inner_component(id)) != No_dcel_comp_id) {
	// remove possible incidence relations from edges on inner boundary.
	if (_cells[id].incident_edge_count > 0) {
	    edge = _components[comp].an_outside_edge;
	    if (edge != No_dcel_edge_id) {
		cur = edge;
		do {
		    if (incident_cell(cur) == id)
			set_incident_cell(cur, No_dcel_cell_id);
		    cur = following(cur);
		} while (cur != edge && cur != No_dcel_edge_id);
	    }
	}
	set_surrounding_cell(comp, No_dcel_cell_id);
    }
    if (_cells[id].incident_edge_count > 0) {
	for (edge = first_edge(); edge != No_dcel_edge_id;
				    edge = next_edge(edge)) {
	    if (incident_cell(edge) == id)
		set_incident_cell(edge, No_dcel_cell_id);
	}
    }
    _cells.delete_element(id);
}

bool dcedgelist::is_valid_cell(dcel_cell_id id) const
{
    if (id.index<0 || id.index >= _cells.size())
	return false;
    return !(_cells[id].is_free());
}

dcel_comp_id dcedgelist::new_component()
{
    _idflags.set(flags8::flag3);
    return _components.new_element();
}

void dcedgelist::delete_component(dcel_comp_id id)
{
    assert(is_valid_component(id));
    dcel_cell_id cell;
    dcel_vertex_id vertex;
    _idflags.set(flags8::flag3);
    set_surrounding_cell(id, No_dcel_cell_id);
    while ((cell = first_inner_cell(id)) != No_dcel_cell_id)
	set_surrounding_component(cell, No_dcel_comp_id);
    while((vertex = a_vertex_of(id)) != No_dcel_vertex_id)
	set_component(vertex, No_dcel_comp_id);
    _components.delete_element(id);
}

bool dcedgelist::is_valid_component(dcel_comp_id id) const
{
    if (id.index<0 || id.index >= _components.size())
	return false;
    return !(_components[id].is_free());
}

const pl_vec & dcedgelist::coordinates(dcel_vertex_id id) const
{
    assert(is_valid_vertex(id));
    return _vertices[id].coords;
}

void dcedgelist::set_coordinates(dcel_vertex_id id, const pl_fixedvec &coords)
{
    assert(is_valid_vertex(id));
    _vertices[id].coords = coords;
}

dcel_vertex_id dcedgelist::first_vertex() const
{
    dcel_vertex_id result;
    result.index = 0;
    while (result.index<_vertices.size() && _vertices.is_free(result))
	result.index = result.index+1;
    return (result.index<_vertices.size()) ? result : No_dcel_vertex_id;
}

dcel_vertex_id dcedgelist::next_vertex(dcel_vertex_id prev) const
{
    assert(is_valid_vertex(prev));
    dcel_vertex_id result = prev;
    do
	result.index = result.index+1;
    while (result.index<_vertices.size() && _vertices.is_free(result));
    return (result.index<_vertices.size()) ? result : No_dcel_vertex_id;
}

int dcedgelist::vertex_count() const
{
    return _vertices.in_use;
}


void dcedgelist::make_twins(dcel_edge_id one, dcel_edge_id other)
{
    assert(is_valid_or_no_edge(one));
    assert(is_valid_or_no_edge(other));
    if (one != No_dcel_edge_id && twin(one) != No_dcel_edge_id)
	set_sym(twin(one), No_dcel_edge_id);
    if (other != No_dcel_edge_id && twin(other) != No_dcel_edge_id)
	set_sym(twin(other), No_dcel_edge_id);
    if (one != No_dcel_edge_id)
	set_sym(one, other);
    if (other != No_dcel_edge_id)
	set_sym(other, one);
}

void dcedgelist::make_consecutive(dcel_edge_id former,
					    dcel_edge_id latter)
{
    assert(is_valid_or_no_edge(former));
    assert(is_valid_or_no_edge(latter));
    dcel_edge_id edge;
    if (latter != No_dcel_edge_id) {
	edge = _edges[latter].prev;
	if (edge != No_dcel_edge_id)
	    _edges[edge].next = No_dcel_edge_id;
    }
    if (former != No_dcel_edge_id) {
	edge = _edges[former].next;
	if (edge != No_dcel_edge_id)
	    _edges[edge].prev = No_dcel_edge_id;
    }
    if (latter != No_dcel_edge_id)
	_edges[latter].prev = former;
    if (former != No_dcel_edge_id)
	_edges[former].next = latter;
}

void dcedgelist::set_incident_cell(dcel_edge_id id, dcel_cell_id cell)
{
    assert(is_valid_edge(id));
    assert(is_valid_or_no_cell(cell));
    dcel_cell_id prevcell = _edges[id].incident_cell;
    if (prevcell == cell)
	return;
    if (prevcell != No_dcel_cell_id) {
	_cells[prevcell].incident_edge_count--;
	if (_cells[prevcell].an_incident_edge == id) {
	    _cells[prevcell].an_incident_edge = No_dcel_edge_id;
	}
    }
    _edges[id].incident_cell = cell;
    if (cell != No_dcel_cell_id) {
	_cells[cell].an_incident_edge = id;
	_cells[cell].incident_edge_count++;
    }
}

dcel_vertex_id dcedgelist::destination(dcel_edge_id id) const
{
    assert(is_valid_edge(id));
    dcel_edge_id edge;
    edge = twin(id);
    if (edge == No_dcel_edge_id)
	return No_dcel_vertex_id;
    return origin(edge);
}

dcel_comp_id dcedgelist::component(dcel_edge_id id) const
{
    assert(is_valid_edge(id));
    dcel_vertex_id vertex;
    vertex = origin(id);
    if (vertex == No_dcel_vertex_id)
	return No_dcel_comp_id;
    return component(vertex);
}

dcel_edge_id dcedgelist::first_edge() const
{
    dcel_edge_id result;
    result.index = 0;
    while (result.index<_edges.size() && _edges.is_free(result))
	result.index = result.index+1;
    return (result.index<_edges.size()) ? result : No_dcel_edge_id;
}

dcel_edge_id dcedgelist::next_edge(dcel_edge_id prev) const
{
    assert(is_valid_edge(prev));
    dcel_edge_id result = prev;
    do
	result.index = result.index+1;
    while (result.index<_edges.size() && _edges.is_free(result));
    return (result.index<_edges.size()) ? result : No_dcel_edge_id;
}

int dcedgelist::edge_count() const
{
    return _edges.in_use;
}

dcel_edge_id dcedgelist::next_cw_from_same_vertex(dcel_edge_id id) const
{
    assert(is_valid_edge(id));
    return following(twin(id));
}

dcel_edge_id dcedgelist::next_ccw_from_same_vertex(dcel_edge_id id) const
{
    assert(is_valid_edge(id));
    return twin(preceding(id));
}

dcel_edge_id dcedgelist::next_cw_to_same_vertex(dcel_edge_id id) const
{
    assert(is_valid_edge(id));
    return twin(following(id));
}

dcel_edge_id dcedgelist::next_ccw_to_same_vertex(dcel_edge_id id) const
{
    assert(is_valid_edge(id));
    return preceding(twin(id));
}

dcel_edge_id dcedgelist::an_edge_from(dcel_vertex_id id) const
{
    assert(is_valid_vertex(id));
    if (_vertices[id].an_incident_edge != No_dcel_edge_id)
        return _vertices[id].an_incident_edge;
    if (_vertices[id].edge_count == 0)
	return No_dcel_edge_id;
    // no edge is recorded, but there must be one so we'll search for it.
    dcel_edge_id edge = first_edge();
    while (edge != No_dcel_edge_id && origin(edge) != id)
	edge = next_edge(edge);
    _vertices[id].an_incident_edge = edge; // (dcel_edge_id &) (...)
    if (edge == No_dcel_edge_id)	// should not happen.
	_vertices[id].edge_count = 0;	// (int &) (...)
    return edge;
}

static dcel_edge_id search_next_edge_from(dcedgelist &dcel, dcel_edge_id id)
{
    dcel_edge_id new_edge;
    dcel_vertex_id origin = dcel.origin(id);
    new_edge = dcel.twin(id);
    if (new_edge != No_dcel_edge_id) {
	new_edge = dcel.following(new_edge);
	if (new_edge != No_dcel_edge_id && dcel.origin(new_edge) == origin)
	    return new_edge;
    }
    new_edge = dcel.preceding(id);
    if (new_edge != No_dcel_edge_id) {
	new_edge = dcel.twin(new_edge);
	if (new_edge != No_dcel_edge_id && dcel.origin(new_edge) == origin)
	    return new_edge;
    }
    // the normal procedure failed.
    return No_dcel_edge_id;
}

void dcedgelist::set_origin(dcel_edge_id id, dcel_vertex_id origin)
{
    assert(is_valid_edge(id));
    assert(is_valid_or_no_vertex(origin));
    dcel_vertex_id prevvt = _edges[id].origin;
    if (prevvt == origin)
	return;
    if (prevvt != No_dcel_vertex_id) {
	_vertices[prevvt].edge_count--;
	if (_vertices[prevvt].an_incident_edge == id) {
	    if (_vertices[prevvt].edge_count == 0) {
		_vertices[prevvt].an_incident_edge = No_dcel_edge_id;
	    } else {
		// search a new edge from this vertex
		_vertices[prevvt].an_incident_edge =
			    search_next_edge_from(*this, id);
	    }
	}
    }
    _edges[id].origin = origin;
    if (origin != No_dcel_vertex_id) {
	_vertices[origin].an_incident_edge = id;
	_vertices[origin].edge_count++;
    }
 }



// ************** CELLS ***************


bool dcedgelist::is_inner_cell_of(dcel_cell_id cell, dcel_comp_id comp) const
{
    assert(is_valid_or_no_cell(cell));
    assert(is_valid_component(comp));
    dcel_cell_id curcell;
    for (curcell = first_inner_cell(comp); curcell != No_dcel_cell_id;
			    curcell = next_inner_cell(comp, curcell)) {
	if (curcell == cell)
	    return 1;
    }
    return 0;
}

void dcedgelist::add_inner_component(dcel_cell_id id, dcel_comp_id comp)
{
    _components[comp].next_component = _cells[id].inner_components;
    _cells[id].inner_components = comp;
}

void dcedgelist::remove_inner_component(dcel_cell_id id, dcel_comp_id comp)
{
    dcel_comp_id *compp;
    compp = &(_cells[id].inner_components);
    while (*compp != No_dcel_comp_id && *compp != comp)
	compp = &(_components[*compp].next_component);
    if (*compp == No_dcel_comp_id)
	return;	// warning: not an inner component.
    *compp = _components[comp].next_component;
    _components[comp].next_component = No_dcel_comp_id;
}

void dcedgelist::set_surrounding_component(dcel_cell_id id,
						    dcel_comp_id comp)
{
    assert(is_valid_cell(id));
    assert(is_valid_or_no_component(comp));
    dcel_comp_id prevcomp;
    prevcomp = _cells[id].outer_component;
    if (prevcomp != No_dcel_comp_id && prevcomp != comp) {
	remove_inner_cell(prevcomp, id);
    }
    _cells[id].outer_component = comp;
    if (comp != No_dcel_comp_id && prevcomp != comp) {
	add_inner_cell(comp, id);
    }
}

dcel_edge_id dcedgelist::an_incident_edge(dcel_cell_id id) const
{
    assert(is_valid_cell(id));
    if (_cells[id].an_incident_edge != No_dcel_edge_id)
        return _cells[id].an_incident_edge;
    if (_cells[id].incident_edge_count == 0)
	return No_dcel_edge_id;
    // no edge is recorded, but there must be one so we'll search for it.
    dcel_edge_id edge = first_edge();
    while (edge != No_dcel_edge_id && incident_cell(edge) != id)
	edge = next_edge(edge);
    _cells[id].an_incident_edge = edge;
    if (edge == No_dcel_edge_id)	// should not happen.
	_cells[id].incident_edge_count = 0;
    return edge;
}

dcel_edge_id dcedgelist::outside_edge(dcel_cell_id id) const
{
    assert(is_valid_cell(id));
    dcel_edge_id edge;
    edge = _cells[id].an_edge_around;
    if (edge != No_dcel_edge_id)
	return edge;
// no edge is registered, so we'll try to find one.
    dcel_comp_id comp = surrounding_component(id);
    if (comp == No_dcel_comp_id)
	return No_dcel_edge_id;
    for (edge = first_edge();
	    edge != No_dcel_edge_id;
	    edge = next_edge(edge)) {
	if (incident_cell(edge) == id && component(edge) == comp) {
	    _cells[id].an_edge_around = edge;
	    return edge;
	}
    }
    return No_dcel_edge_id;
}

dcel_cell_id dcedgelist::first_cell() const
{
    dcel_cell_id result;
    result.index = 0;
    while (result.index<_cells.size() && _cells.is_free(result))
	result.index = result.index+1;
    return (result.index<_cells.size()) ? result : No_dcel_cell_id;
}

dcel_cell_id dcedgelist::next_cell(dcel_cell_id prev) const
{
    assert(is_valid_cell(prev));
    dcel_cell_id result = prev;
    do
	result.index = result.index+1;
    while (result.index<_cells.size() && _cells.is_free(result));
    return (result.index<_cells.size()) ? result : No_dcel_cell_id;
}

int dcedgelist::cell_count() const
{
    return _cells.in_use;
}

// ************** COMPONENTS ***************


bool dcedgelist::is_inner_component_of(dcel_comp_id comp,
					dcel_cell_id cell) const
{
    assert(is_valid_or_no_component(comp));
    assert(is_valid_cell(cell));
    dcel_comp_id curcomp;
    for (curcomp = first_inner_component(cell); curcomp != No_dcel_comp_id;
			    curcomp = next_inner_component(cell, curcomp)) {
	if (curcomp == comp)
	    return 1;
    }
    return 0;
}

void dcedgelist::add_inner_cell(dcel_comp_id id, dcel_cell_id cell)
{
    _cells[cell].next_cell = _components[id].inner_cells;
    _components[id].inner_cells = cell;
}

void dcedgelist::remove_inner_cell(dcel_comp_id id, dcel_cell_id cell)
{
    dcel_cell_id *cellp;
    cellp = &(_components[id].inner_cells);
    while (*cellp != No_dcel_cell_id && *cellp != cell)
	cellp = &(_cells[*cellp].next_cell);
    if (*cellp == No_dcel_cell_id)
	return;	// warning: not an inner cell.
    *cellp = _cells[cell].next_cell;
    _cells[cell].next_cell = No_dcel_cell_id;
}

dcel_vertex_id dcedgelist::a_vertex_of(dcel_comp_id id) const
{
    assert(is_valid_component(id));
    if (_components[id].a_vertex != No_dcel_vertex_id)
        return _components[id].a_vertex;
    if (_components[id].member_vertex_count == 0)
	return No_dcel_vertex_id;
    // no vertex is recorded, but there must be one so we'll search for it.
    dcel_vertex_id vertex = first_vertex();
    while (vertex != No_dcel_vertex_id && component(vertex) != id)
	vertex = next_vertex(vertex);
    _components[id].a_vertex = vertex; // (dcel_vertex_id &) (...)
    if (vertex == No_dcel_vertex_id)	// should not happen.
	_components[id].member_vertex_count = 0;	// (int &) (...)
    return vertex;
}

void dcedgelist::set_surrounding_cell(dcel_comp_id id, dcel_cell_id cell)
{
    assert(is_valid_component(id));
    assert(is_valid_or_no_cell(cell));
    dcel_cell_id prevcell;
    prevcell = _components[id].outer_cell;
    if (prevcell != No_dcel_cell_id && prevcell != cell) {
	remove_inner_component(prevcell, id);
    }
    _components[id].outer_cell = cell;
    if (cell != No_dcel_cell_id && prevcell != cell) {
	add_inner_component(cell, id);
    }
}

dcel_edge_id dcedgelist::outside_edge(dcel_comp_id id) const
{
    assert(is_valid_component(id));
    if (_components[id].member_vertex_count <= 1)
	return No_dcel_edge_id;
    dcel_edge_id edge;
    edge = _components[id].an_outside_edge;
    if (edge != No_dcel_edge_id)
	return edge;
// no edge is registered, so we'll try to find one.
    dcel_cell_id cell = surrounding_cell(id);
    if (cell == No_dcel_cell_id)
	return No_dcel_edge_id;
    for (edge = first_edge();
	    edge != No_dcel_edge_id;
	    edge = next_edge(edge)) {
	if (component(edge) == id && incident_cell(edge) == cell) {
	    _components[id].an_outside_edge = edge;
	    return edge;
	}
    }
    return No_dcel_edge_id;
}

void dcedgelist::set_outside_edge(dcel_comp_id id, dcel_edge_id edge)
{
    assert(is_valid_component(id));
    assert(is_valid_or_no_edge(edge));
    _components[id].an_outside_edge = edge;
}

void dcedgelist::set_component(dcel_vertex_id id, dcel_comp_id comp)
{
    assert(is_valid_vertex(id));
    assert(is_valid_or_no_component(comp));
    dcel_comp_id prevcomp = _vertices[id].component;
    if (prevcomp == comp)
	return;
    if (prevcomp != No_dcel_comp_id) {
	_components[prevcomp].member_vertex_count--;
	if (_components[prevcomp].a_vertex == id) {
	    _components[prevcomp].a_vertex = No_dcel_vertex_id;
	}
    }
    _vertices[id].component = comp;
    if (comp != No_dcel_comp_id) {
	_components[comp].a_vertex = id;
	_components[comp].member_vertex_count++;
    }
}

dcel_comp_id dcedgelist::first_component() const
{
    dcel_comp_id result;
    result.index = 0;
    while (result.index<_components.size() && _components.is_free(result))
	result.index = result.index+1;
    return (result.index<_components.size()) ? result : No_dcel_comp_id;
}

dcel_comp_id dcedgelist::next_component(dcel_comp_id prev) const
{
    assert(is_valid_component(prev));
    dcel_comp_id result = prev;
    do
	result.index = result.index+1;
    while (result.index<_components.size() && _components.is_free(result));
    return (result.index<_components.size()) ? result : No_dcel_comp_id;
}

int dcedgelist::component_count() const
{
    return _components.in_use;
}

int dcedgelist::int_ident(dcel_vertex_id id) const
{
    assert(is_valid_or_no_vertex(id));
    assert (!_idflags.is_set(flags8::flag0));
    if (id == No_dcel_vertex_id)
	return -1;
    return _vertices[id].ident;
}

int dcedgelist::int_ident(dcel_edge_id id) const
{
    assert(is_valid_or_no_edge(id));
    assert (!_idflags.is_set(flags8::flag1));
    if (id == No_dcel_edge_id)
	return -1;
    return _edges[id].ident;
}

int dcedgelist::int_ident(dcel_cell_id id) const
{
    assert(is_valid_or_no_cell(id));
    assert (!_idflags.is_set(flags8::flag2));
    if (id == No_dcel_cell_id)
	return -1;
    return _cells[id].ident;
}

int dcedgelist::int_ident(dcel_comp_id id) const
{
    assert(is_valid_or_no_component(id));
    assert (!_idflags.is_set(flags8::flag3));
    if (id == No_dcel_comp_id)
	return -1;
    return _components[id].ident;
}

dcel_vertex_id dcedgelist::vertex_id(int id) const
{
    assert (id >=-1 && id < vertex_count());
    assert (!_idflags.is_set(flags8::flag0));
    int i;
    dcel_vertex_id curvt;
    if (id<0)
	return No_dcel_vertex_id;
    if (_vertex_corr.size() != vertex_count()) {
	((intreray&)_vertex_corr).newsize(vertex_count());
	curvt = first_vertex();
	for (i=0; i<vertex_count(); i++) {
	    _vertex_corr[int_ident(curvt)] = curvt.index;
	    curvt = next_vertex(curvt);
	}
    }
    dcel_vertex_id result;
    result.index = _vertex_corr[id];
    return result;
}

dcel_edge_id dcedgelist::edge_id(int id) const
{
    assert (id >=-1 && id < edge_count());
    assert (!_idflags.is_set(flags8::flag1));
    int i;
    dcel_edge_id curvt;
    if (id<0)
	return No_dcel_edge_id;
    if (_edge_corr.size() != edge_count()) {
	((intreray&)_edge_corr).newsize(edge_count());
	curvt = first_edge();
	for (i=0; i<edge_count(); i++) {
	    _edge_corr[int_ident(curvt)] = curvt.index;
	    curvt = next_edge(curvt);
	}
    }
    dcel_edge_id result;
    result.index = _edge_corr[id];
    return result;
}

dcel_cell_id dcedgelist::cell_id(int id) const
{
    assert (id >=-1 && id < cell_count());
    assert (!_idflags.is_set(flags8::flag2));
    int i;
    dcel_cell_id curvt;
    if (id<0)
	return No_dcel_cell_id;
    if (_cell_corr.size() != cell_count()) {
	((intreray&)_cell_corr).newsize(cell_count());
	curvt = first_cell();
	for (i=0; i<cell_count(); i++) {
	    _cell_corr[int_ident(curvt)] = curvt.index;
	    curvt = next_cell(curvt);
	}
    }
    dcel_cell_id result;
    result.index = _cell_corr[id];
    return result;
}

dcel_comp_id dcedgelist::comp_id(int id) const
{
    assert (id >=-1 && id < component_count());
    assert (!_idflags.is_set(flags8::flag3));
    int i;
    dcel_comp_id curvt;
    if (id<0)
	return No_dcel_comp_id;
    if (_comp_corr.size() != component_count()) {
	((intreray&)_comp_corr).newsize(component_count());
	curvt = first_component();
	for (i=0; i<component_count(); i++) {
	    _comp_corr[int_ident(curvt)] = curvt.index;
	    curvt = next_component(curvt);
	}
    }
    dcel_comp_id result;
    result.index = _comp_corr[id];
    return result;
}


void dcedgelist::update_vertex_idents() const
{
    ((dcedgelist*)this)->upd_vertex_idents();
}

void dcedgelist::upd_vertex_idents()
{
    int id;
    dcel_vertex_id vtid;
    if (_idflags.is_set(flags8::flag0)) {
	_idflags.unset(flags8::flag0);
	_vertex_corr.newsize(0);
	id = 0;
	for (vtid.index=0; vtid.index<_vertices.size(); vtid.index++) {
	    if (!_vertices[vtid].is_free())
		_vertices[vtid].ident = id++;
	}
    }
}

void dcedgelist::update_edge_idents() const
{
    ((dcedgelist*)this)->upd_edge_idents();
}

void dcedgelist::upd_edge_idents()
{
    int id;
    dcel_edge_id edgeid;
    if (_idflags.is_set(flags8::flag1)) {
	_idflags.unset(flags8::flag1);
	_edge_corr.newsize(0);
	id = 0;
	for (edgeid.index=0; edgeid.index<_edges.size(); edgeid.index++) {
	    if (!_edges[edgeid].is_free())
		_edges[edgeid].ident = id++;
	}
    }
}

void dcedgelist::update_cell_idents() const
{
    ((dcedgelist*)this)->upd_cell_idents();
}

void dcedgelist::upd_cell_idents()
{
    int id;
    dcel_cell_id cellid;
    if (_idflags.is_set(flags8::flag2)) {
	_idflags.unset(flags8::flag2);
	_cell_corr.newsize(0);
	id = 0;
	for (cellid.index=0; cellid.index<_cells.size(); cellid.index++) {
	    if (!_cells[cellid].is_free())
		_cells[cellid].ident = id++;
	}
    }
}

void dcedgelist::update_component_idents() const
{
    ((dcedgelist*)this)->upd_component_idents();
}

void dcedgelist::upd_component_idents()
{
    int id;
    dcel_comp_id compid;
    if (_idflags.is_set(flags8::flag3)) {
	_idflags.unset(flags8::flag3);
	_comp_corr.newsize(0);
	id = 0;
	for (compid.index=0; compid.index<_components.size(); compid.index++) {
	    if (!_components[compid].is_free())
		_components[compid].ident = id++;
	}
    }
}

void dcedgelist::update_int_idents() const
{
    dcedgelist *nc_this = (dcedgelist*) this;	// non const this pointer
    nc_this->upd_vertex_idents();
    nc_this->upd_edge_idents();
    nc_this->upd_cell_idents();
    nc_this->upd_component_idents();
}


// ************* Consistency checking ***************




const char * dcedgelist::consistency_message() const
{
    return _msg;
}

bool dcedgelist::is_consistent() const
{
    dcel_vertex_id vertexid;
    dcel_edge_id edgeid;
    dcel_cell_id cellid;
    dcel_comp_id compid;
//    static intreray vt_in_use, edge_in_use, cell_in_use, comp_in_use;
    (char *&)_msg = "Everything is OK.";
    
    if (!is_valid_cell(outer_cell())) {
	(char *&)_msg = "No outer cell specified.";
	return 0;
    }
    for (compid = first_component(); compid != No_dcel_comp_id;
				compid = next_component(compid)) {
	const dcel_comp &this_comp = _components[compid];
	if (!is_valid_cell(this_comp.outer_cell)) {
	    (char *&)_msg = "Invalid surrounding cell of a component.";
	    return 0;
	}
	if (!is_inner_component_of(compid, this_comp.outer_cell)){
	    (char *&)_msg = "A component is not an inner component of its surrounding cell.";
	    return 0;
	}
	if (!is_valid_or_no_component(this_comp.next_component)) {
	    (char *&)_msg = "Invalid component in inner_components list of a cell.";
	    return 0;
	}
	dcel_cell_id inner_cell = this_comp.inner_cells;
	while (inner_cell != No_dcel_cell_id) {
	    if (!is_valid_cell(inner_cell)) {
		(char *&)_msg = "Inconsistent inner_cells list of a component.";
		return 0;
	    }
	    const dcel_cell &curcell = _cells[inner_cell];
	    if (curcell.outer_component != compid) {
		(char *&)_msg = "Inconsistent inner_cells list of a component.";
		return 0;
	    }
	    inner_cell = curcell.next_cell;
	}
	if (!is_valid_or_no_vertex(this_comp.a_vertex)) {
	    (char *&)_msg = "Invalid vertex as a_vertex of a component.";
	    return 0;
	}
	if (this_comp.a_vertex != No_dcel_vertex_id
	&& _vertices[this_comp.a_vertex].component != compid) {
	    (char *&)_msg = "Inconsistent member vertex of a component.";
	    return 0;
	}
	if (!is_valid_or_no_edge(this_comp.an_outside_edge)) {
	    (char *&)_msg = "Invalid outside edge of a component.";
	    return 0;
	}
	if (this_comp.an_outside_edge != No_dcel_edge_id
	&& incident_cell(this_comp.an_outside_edge) != this_comp.outer_cell) {
	    (char *&)_msg = "outside edge is not incident on surrounding cell.";
	    return 0;
	}
    }

    for (cellid = first_cell(); cellid != No_dcel_cell_id;
				cellid = next_cell(cellid)) {
	const dcel_cell &this_cell = _cells[cellid];
	if (this_cell.outer_component == No_dcel_comp_id) {
	    if (outer_cell() != cellid) {
		(char *&)_msg = "Cell without a surrounding component.";
		return 0;
	    }
	} else {
	    if (!is_valid_component(this_cell.outer_component)) {
		(char *&)_msg = "Invalid surrounding component for a cell.";
		return 0;
	    }
	    if (!is_inner_cell_of(cellid, this_cell.outer_component)) {
		(char *&)_msg = "Inconsistent surrounding component of a cell.";
		return 0;
	    }
	}
	dcel_comp_id inner_comp = this_cell.inner_components;
	while (inner_comp != No_dcel_comp_id) {
	    if (!is_valid_component(inner_comp)) {
		(char *&)_msg = "Inconsistent inner_components list of a cell.";
		return 0;
	    }
	    const dcel_comp &curcomp = _components[inner_comp];
	    if (curcomp.outer_cell != cellid) {
		(char *&)_msg = "Inconsistent inner_componentss list of a cell.";
		return 0;
	    }
	    inner_comp = curcomp.next_component;
	}
	if (this_cell.an_edge_around != No_dcel_edge_id) {
	    if (!is_valid_edge(this_cell.an_edge_around)) {
		(char *&)_msg = "Invalid edge_around a cell.";
		return 0;
	    }
	    if (_edges[this_cell.an_edge_around].incident_cell != cellid) {
		(char *&)_msg = "Inconsistent edge around a cell.";
		return 0;
	    }
	}
    }

    for (vertexid = first_vertex(); vertexid != No_dcel_vertex_id;
		    vertexid = next_vertex(vertexid)) {	
	const dcel_vertex &this_vt = _vertices[vertexid];
	if (!is_valid_or_no_edge(this_vt.an_incident_edge)) {
	    (char *&)_msg = "Invalid incident edge of a vertex.";
	    return 0;
	}
	if (this_vt.an_incident_edge != No_dcel_edge_id
	    && _edges[this_vt.an_incident_edge].origin != vertexid) {
	    (char *&)_msg = "Inconsistent incident edge of a vertex.";
	    return 0;
	}
	if (!is_valid_component(this_vt.component)) {
	    (char *&)_msg = "Invalid component of a vertex.";
	    return 0;
	}
    }

    for (edgeid = first_edge(); edgeid != No_dcel_edge_id;
		    edgeid = next_edge(edgeid)) {	
	const dcel_edge &this_edge = _edges[edgeid];
	if (!is_valid_edge(this_edge.sym)) {
	    (char *&)_msg = "Invalid twin edge of an edge.";
	    return 0;
	}
	if (_edges[this_edge.sym].sym != edgeid) {
	    (char *&)_msg = "Inconsistent twin edge of an edge.";
	    return 0;
	}
	if (!is_valid_edge(this_edge.next)) {
	    (char *&)_msg = "Invalid next edge of an edge.";
	    return 0;
	}
	if (_edges[this_edge.next].prev != edgeid) {
	    (char *&)_msg = "Inconsistent next-prev edge pair.";
	    return 0;
	}
	if (!is_valid_edge(this_edge.prev)) {
	    (char *&)_msg = "Invalid prev edge of an edge.";
	    return 0;
	}
	if (_edges[this_edge.prev].next != edgeid) {
	    (char *&)_msg = "Inconsistent prev-next edge pair.";
	    return 0;
	}
	if (!is_valid_vertex(this_edge.origin)) {
	    (char *&)_msg = "Invalid origin of edge.";
	    return 0;
	}
	if (!is_valid_cell(this_edge.incident_cell)) {
	    (char *&)_msg = "Invalid incident_cell of an edge.";
	    return 0;
	}
	if (_edges[this_edge.next].incident_cell != this_edge.incident_cell) {
	    (char *&)_msg = "Subsequent edges not incident on same cell.";
	    return 0;
	}
	if (_vertices[_edges[this_edge.next].origin].component !=
		_vertices[this_edge.origin].component) {
	    (char *&)_msg = "Connected vertices not part of same component.";
	    return 0;
	}
	if (_vertices[_edges[this_edge.sym].origin].component !=
		_vertices[this_edge.origin].component) {
	    (char *&)_msg = "Connected vertices not part of same component.";
	    return 0;
	}
	if (_edges[_edges[this_edge.sym].next].origin != this_edge.origin) {
	    (char *&)_msg = "Inconsistent origins of edges.";
	    return 0;
	}
    }
    return 1;
}


// ******************** input and output **********************

void write_without_brackets(ostream &out, const dcedgelist &dcel)
{
    dcel_vertex_id vertexid;
    dcel_edge_id edgeid;
    dcel_cell_id cellid;
    dcel_comp_id compid;
    dcel.update_int_idents();
    out<<dcel.vertex_count();
    out<<'\n'<<dcel.edge_count();
    out<<'\n'<<dcel.cell_count();
    cellid = dcel.first_cell();
    out<<'\n'<<dcel.component_count();

    forall_vertices(dcel, vertexid) {
	const pl_vec &coords = dcel.coordinates(vertexid);
	out<<'\n'<<dcel.int_ident(vertexid)
	    <<' '<<dcel.int_ident(dcel.component(vertexid))
	    <<' '<<coords.x()<<' '<<coords.y();
    }
    forall_edges(dcel, edgeid) {
	out<<'\n'<<dcel.int_ident(edgeid)
	    <<' '<<dcel.int_ident(dcel.twin(edgeid))
	    <<' '<<dcel.int_ident(dcel.following(edgeid))
	    <<' '<<dcel.int_ident(dcel.origin(edgeid))
	    <<' '<<dcel.int_ident(dcel.incident_cell(edgeid));
    }
    forall_cells(dcel, cellid) {
	out<<'\n'<<dcel.int_ident(cellid)
	    <<' '<<dcel.int_ident(dcel.surrounding_component(cellid))
	    <<' '<<dcel.int_ident(dcel.outside_edge(cellid));
    }
    forall_components(dcel, compid) {
	out<<'\n'<<dcel.int_ident(compid)
	    <<' '<<dcel.int_ident(dcel.surrounding_cell(compid))
	    <<' '<<dcel.int_ident(dcel.outside_edge(compid));
    }
}

ostream & operator<<(ostream &out, const dcedgelist &dcel)
{
    write_without_brackets(out, dcel);
    out<<"\n{}";
    return out;
}

void skip_to_matching_bracket(istream &in)
{
    char c;
    int depth;
    depth = 1;
    while (1) {
	in>>c;
	if (!in) {
	    return;
	}
	switch (c) {
	    case '{': depth++; break;
	    case '}': depth--; break;
	}
	if (depth == 0)
	    break;
    }
}

int read_index(istream &in, int max)
{
    int no;
    in>>no;
    if (!in)
	return -1;
    if (no == -1) {
	return -1;
    }
    if (no < -1 || no >= max) {
	in.clear(ios::badbit|in.rdstate());
	return -1;
    }
    return no;
}


void read_without_brackets(istream &in, dcedgelist &dcel)
{
    int i;
    int vertexcount, edgecount, cellcount, componentcount;
//    static intreray vertexcor, edgecor, cellcor, componentcor, elems;
    dcel.clear();    // clear possible contents.
    dcel.delete_cell(dcel.outer_cell());
// Read and check the number of vertices, edges, cells and components.
    in >> vertexcount >> edgecount >> cellcount >>componentcount;
    if (!in)
	return;
    if (vertexcount < 0 || edgecount < 0
	    || cellcount < 0 || componentcount < 0) {
	in.clear(ios::badbit|in.rdstate());
	return;
    }
// Make all subdivision elements and set up a correspondence between
// integer identifiers in file and dcel identifiers.
    for (i=0; i<vertexcount; i++)
	dcel.new_vertex();
    for (i=0; i<edgecount; i++)
	dcel.new_edge();
    for (i=0; i<cellcount; i++)
	dcel.new_cell();
    for (i=0; i<componentcount; i++)
	dcel.new_component();
    dcel.update_int_idents();

// Now read the info of all subdivision elements.
    for (i=0; i<vertexcount; i++) {
	int ident;
	int compid;
	s_coord x, y;
	pl_vec coords;
	ident = read_index(in, vertexcount);
	compid = read_index(in, componentcount);
	if (ident == -1)
	    in.clear(ios::badbit|in.rdstate());
	in>>x>>y;
	if (!in)
	    return;
	dcel.set_component(dcel.vertex_id(ident), dcel.comp_id(compid));
	dcel.set_coordinates(dcel.vertex_id(ident), pl_vec(x, y));
    }

    for (i=0; i<edgecount; i++) {
	int edgeid1, edgeid2;
	int vertexid;
	int cellid;
	edgeid1 = read_index(in, edgecount);
	edgeid2 = read_index(in, edgecount);
	if (edgeid1 == -1)
	    in.clear(ios::badbit|in.rdstate());
	if (!in)
	    return;
	dcel.make_twins(dcel.edge_id(edgeid1), dcel.edge_id(edgeid2));
	edgeid2 = read_index(in, edgecount);
	vertexid = read_index(in, vertexcount);
	cellid = read_index(in, cellcount);
	if (!in)
	    return;
	dcel.make_consecutive(dcel.edge_id(edgeid1), dcel.edge_id(edgeid2));
	dcel.set_origin(dcel.edge_id(edgeid1), dcel.vertex_id(vertexid));
	dcel.set_incident_cell(dcel.edge_id(edgeid1), dcel.cell_id(cellid));
    }
    for (i=0; i<cellcount; i++) {
	int cellid;
	int compid;
	int edgeid;
	cellid = read_index(in, cellcount);
	compid = read_index(in, componentcount);
	edgeid = read_index(in, edgecount);
	if (cellid == -1)
	    in.clear(ios::badbit|in.rdstate());
	if (!in)
	    return;
	if (compid == -1)
	    dcel.set_outer_cell(dcel.cell_id(cellid));
	else
	    dcel.set_surrounding_component(dcel.cell_id(cellid), dcel.comp_id(compid));
	dcel.set_outside_edge(dcel.cell_id(cellid), dcel.edge_id(edgeid));
    }
    for (i=0; i<componentcount; i++) {
	int compid;
	int cellid;
	int edgeid;
	compid = read_index(in, componentcount);
	cellid = read_index(in, cellcount);
	edgeid = read_index(in, edgecount);
	if (compid == -1)
	    in.clear(ios::badbit|in.rdstate());
	if (!in)
	    return;
	dcel.set_surrounding_cell(dcel.comp_id(compid), dcel.cell_id(cellid));
	dcel.set_outside_edge(dcel.comp_id(compid), dcel.edge_id(edgeid));
    }
}

istream & operator>>(istream &in, dcedgelist &dcel)
{
    read_without_brackets(in, dcel);
    if (!in)
	return in;
// now discard info part.    
    char c;
    in>>c;
    if (c!='{') {
	in.clear(ios::badbit|in.rdstate());
	return in;
    }
    skip_to_matching_bracket(in);
    return in;
}

static void special_cases(dcedgelist &dcel,
		    dcel_cell_id outside, const pl_pgn_ifc &pgn)
{
    const int vertex_count = pgn.size();
    if (vertex_count == 0)
	return;
    dcel_comp_id comp;
    dcel_vertex_id vt1, vt2;
    dcel_edge_id edge1, edge2;
    comp = dcel.new_component();
    dcel.set_surrounding_cell(comp, outside);
    vt1 = dcel.new_vertex();
    dcel.set_coordinates(vt1, pgn.vertex(0));
    dcel.set_component(vt1, comp);
    if (vertex_count == 1)
	return;
// polygon with two vertices.
    vt2 = dcel.new_vertex();
    dcel.set_coordinates(vt2, pgn.vertex(1));
    dcel.set_component(vt2, comp);
    edge1 = dcel.new_edge();
    edge2 = dcel.new_edge();
    dcel.make_consecutive(edge1, edge2);
    dcel.make_consecutive(edge2, edge1);
    dcel.make_twins(edge1, edge2);
    dcel.set_origin(edge1, vt1);
    dcel.set_origin(edge2, vt2);
}

dcel_cell_id insert_pgn_in_cell(dcedgelist &dcel,
		    dcel_cell_id outside, const pl_pgn_ifc &pgn)
{
    dcel_cell_id inside;
    const int vertex_count = pgn.size();
    const int edge_count = 2*pgn.size();
    dcel_comp_id comp;
    int i;
    dcel_vertex_id vt;
    dcel_edge_id first_edge, cur1, cur2;

// speciale gevallen: pgn.size() <3
    if (vertex_count < 3) {
	special_cases(dcel, outside, pgn);
	return No_dcel_cell_id;
    }
    inside = dcel.new_cell();
    comp = dcel.new_component();
    dcel.set_surrounding_cell(comp, outside);
    dcel.set_surrounding_component(inside, comp);

    first_edge = dcel.new_edge();
    cur1 = first_edge;
    for (i=1; i<vertex_count; i++) {
	cur2 = dcel.new_edge();
	dcel.make_consecutive(cur1, cur2);
	cur1 = cur2;
    }
    dcel.make_consecutive(cur1, first_edge);
    cur1 = dcel.new_edge();
    dcel.make_twins(cur1, first_edge);
    for (i=1; i<vertex_count; i++) {
	cur2 = dcel.new_edge();
	dcel.make_consecutive(cur1, cur2);
	cur1 = cur2;
    }
    cur2 = dcel.twin(first_edge);
    dcel.make_consecutive(cur1, cur2);
    cur1 = dcel.following(first_edge);
    cur2 = dcel.preceding(cur2);
    while (cur1 != first_edge) {
	dcel.make_twins(cur1, cur2);
	cur1 = dcel.following(cur1);
	cur2 = dcel.preceding(cur2);
    }
//    cur1 == first_edge;
//    cur2 == dcel.twin(first_edge);
    cur2 = dcel.following(cur2);
    for (i=0; i<vertex_count; i++) {
	vt = dcel.new_vertex();
	dcel.set_component(vt, comp);
	dcel.set_coordinates(vt, pgn.vertex(i));
	dcel.set_origin(cur1, vt);
	dcel.set_origin(cur2, vt);
	cur1 = dcel.following(cur1);
	cur2 = dcel.preceding(cur2);
    }
//    cur1 == first_edge;
//    cur2 is edge of opposite chain.
    if (pgn.is_clockwise()) {
	cur1 = cur2;
	cur2 = first_edge;
    }
    for (i=0; i<vertex_count; i++) {
	dcel.set_incident_cell(cur1, inside);
	dcel.set_incident_cell(cur2, outside);
	cur1 = dcel.following(cur1);
	cur2 = dcel.preceding(cur2);
    }
    dcel.set_outside_edge(inside, cur1);
    return inside;
}

void pgn_to_dcel(dcedgelist &dcel, const pl_pgn_ifc &pgn)
{
    dcel.clear();
    insert_pgn_in_cell(dcel, dcel.outer_cell(), pgn);
}


