// $Id: dcel.h,v 1.5 2000/03/21 12:39:13 geert Exp $

#ifndef DCELH
#define DCELH

#include <reray.h>
#include <flags.h>
#include <PLAGEO/base.h>
#include <PLAGEO/pgn_ifc.h>

#define private_index_def(classname) \
struct classname { \
    int index; \
    int operator==(classname other) const {return index == other.index;} \
    int operator!=(classname other) const {return index != other.index;} \
    int operator<(classname other)  const {return index <  other.index;} \
    int operator<=(classname other) const {return index <= other.index;} \
    int operator>(classname other)  const {return index >  other.index;} \
    int operator>=(classname other) const {return index >= other.index;} \
}; \
const classname No_ ## classname = {-1}; \
const classname Invalid_ ## classname = {-2};

#define private_index_reray(rerayname, elementclass, indexclass) \
struct rerayname: public reray<elementclass> { \
		    rerayname() ; \
    elementclass &  operator[](indexclass id) const \
		    {return (*(reray<elementclass> *)this)[id.index];} \
    indexclass	    free_element; \
    int		    in_use; \
    indexclass	    new_element(); \
    void	    delete_element(indexclass el); \
    int		    is_free(indexclass el) const; \
};

private_index_def(dcel_vertex_id)
private_index_def(dcel_edge_id)
private_index_def(dcel_cell_id)
private_index_def(dcel_comp_id)


struct dcel_edge {
		    dcel_edge() {make_empty();}
    dcel_edge_id    sym;
	// the edge opposite to this edge.
    dcel_edge_id    next;
	// the edge following this edge counterclockwise on same cell.
	// also used, internally, as a pointer in the unused list.
    dcel_edge_id    prev;
	// the edge preceding this edge.
    dcel_vertex_id  origin;
	// the vertex from which this edge starts.
    dcel_cell_id    incident_cell;
	// the cell bounding this edge.
    int		    ident;
	// -1 if this vertex is part of the free list.
    void	    make_empty()    ;
    bool	    is_free() const {return ident == -1;}
    void	    mark_free() {ident = -1;}
    void	    *info;
};

struct dcel_vertex {
		    dcel_vertex() {make_empty();}
    pl_vec	    coords;
    union {
    dcel_edge_id    an_incident_edge;
	// an arbitrary edge with this vertex as origin.
    dcel_vertex_id  next;
	// internally also used for chaining unused vertices
    };
    dcel_comp_id    component;
	// the component to which this vertex belongs.
    int		    ident;
	// -1 if this vertex is part of the free list.
    int		    edge_count;	    // number of edges from this vertex.
    void	    make_empty()    ;
    int		    is_free() const {return ident == -1;}
    void	    mark_free() {ident = -1;}
    void	    *info;
};

struct dcel_cell {
		    dcel_cell() {make_empty();}
    dcel_comp_id    outer_component;
	// the component that encloses this cell (No_dcel_comp_id for unbounded cell).
    dcel_cell_id    next_cell;
	// next element of inner_cells list of outer_component.
	// also used, internally, for chaining unused cells.
    dcel_comp_id    inner_components;
	// head of list of components contained inside this cell.
    dcel_edge_id    an_incident_edge;
    int		    incident_edge_count;
    dcel_edge_id    an_edge_around;
	// an edge on the boundary (if one exists)
    int		    ident;
	// -1 if this vertex is part of the free list.
    void	    make_empty()    ;
    bool	    is_free() const {return ident == -1;}
    void	    mark_free() {ident = -1;}
    void	    *info;
};

struct dcel_comp {
		    dcel_comp() {make_empty();}
    dcel_cell_id    outer_cell;
	// the cell surrounding this component.
	// also used internally as a free marker (Invalid_dcel_cell_id if unused)
    dcel_comp_id    next_component;
	// next element of inner_components list of outer_cell.
	// also used, internally, for chaining unused components.
    dcel_cell_id    inner_cells;
	// head of list of cells that are enclosed in this component.
    dcel_vertex_id  a_vertex;
	// an arbitrary vertex belonging to this component.
    int		    member_vertex_count;
    dcel_edge_id    an_outside_edge;
	// an arbitrary edge on the outside of this component.
    int		    ident;
	// -1 if this vertex is part of the free list.
    void	    make_empty()    ;
    bool	    is_free() const {return ident == -1;}
    void	    mark_free() {ident = -1;}
    void	    *info;
};

private_index_reray(dcel_vt_reray, dcel_vertex, dcel_vertex_id)
private_index_reray(dcel_edge_reray, dcel_edge, dcel_edge_id)
private_index_reray(dcel_cell_reray, dcel_cell, dcel_cell_id)
private_index_reray(dcel_comp_reray, dcel_comp, dcel_comp_id)



struct dcedgelist {
		    dcedgelist() ;
		    dcedgelist(int vtn, int edgen, int celln, int compn);
    bool	    is_consistent() const;
const char *	    consistency_message() const;
    void	    update_int_idents() const;
    void	    clear() ;
    dcel_vertex_id  new_vertex();
    void	    delete_vertex(dcel_vertex_id id);
    bool	    is_valid_vertex(dcel_vertex_id id) const;
    bool	    is_valid_or_no_vertex(dcel_vertex_id id) const
			{return id == No_dcel_vertex_id || is_valid_vertex(id);}
    dcel_comp_id    component(dcel_vertex_id id) const
			{return _vertices[id].component;}
    void	    set_component(dcel_vertex_id id, dcel_comp_id component);
    const pl_vec &  coordinates(dcel_vertex_id id) const;
    void	    set_coordinates(dcel_vertex_id id, const pl_fixedvec &coords);
    dcel_edge_id    an_edge_from(dcel_vertex_id id) const;
    void*	    info(dcel_vertex_id id) const
			{return _vertices[id].info;}
    void	    set_info(dcel_vertex_id id, void *info)
			{_vertices[id].info = info; }
    dcel_vertex_id  first_vertex() const;
    dcel_vertex_id  next_vertex(dcel_vertex_id prev) const;
    int		    vertex_count() const;
    void	    update_vertex_idents() const;
    int		    int_ident(dcel_vertex_id id) const;
    dcel_vertex_id  vertex_id(int) const;


    dcel_edge_id    new_edge();
    void	    delete_edge(dcel_edge_id id);
    bool	    is_valid_edge(dcel_edge_id id) const;
    bool	    is_valid_or_no_edge(dcel_edge_id id) const
			{return id == No_dcel_edge_id || is_valid_edge(id);}
    dcel_edge_id    twin(dcel_edge_id id) const
			{return _edges[id].sym;}
    void	    make_twins(dcel_edge_id one, dcel_edge_id other);
    dcel_edge_id    following(dcel_edge_id id) const
			{return _edges[id].next;}
    dcel_edge_id    preceding(dcel_edge_id id) const
			{return _edges[id].prev;}
    dcel_edge_id    next_cw_from_same_vertex(dcel_edge_id id) const ;
    dcel_edge_id    next_ccw_from_same_vertex(dcel_edge_id id) const ;
    dcel_edge_id    next_cw_to_same_vertex(dcel_edge_id id) const ;
    dcel_edge_id    next_ccw_to_same_vertex(dcel_edge_id id) const ;
    void	    make_consecutive(dcel_edge_id former, dcel_edge_id latter);
    dcel_vertex_id  origin(dcel_edge_id id) const
			{return _edges[id].origin;}
    void	    set_origin(dcel_edge_id id, dcel_vertex_id origin) ;
    dcel_vertex_id  destination(dcel_edge_id id) const;
    dcel_cell_id    incident_cell(dcel_edge_id id) const
			{return _edges[id].incident_cell;}
    void	    set_incident_cell(dcel_edge_id id, dcel_cell_id cell);
    dcel_comp_id    component(dcel_edge_id id) const;
    void*	    info(dcel_edge_id id) const
			{return _edges[id].info;}
    void	    set_info(dcel_edge_id id, void *info)
			{_edges[id].info = info; }
    dcel_edge_id    first_edge() const;
    dcel_edge_id    next_edge(dcel_edge_id prev) const;
    int		    edge_count() const;
    void	    update_edge_idents() const;
    int		    int_ident(dcel_edge_id id) const;
    dcel_edge_id    edge_id(int) const;

    dcel_cell_id    new_cell();
    void	    delete_cell(dcel_cell_id id);
    bool	    is_valid_cell(dcel_cell_id id) const;
    bool	    is_valid_or_no_cell(dcel_cell_id id) const
			{return id == No_dcel_cell_id || is_valid_cell(id);}
    dcel_comp_id    surrounding_component(dcel_cell_id id) const
			{return _cells[id].outer_component;}
    void	    set_surrounding_component(dcel_cell_id id, dcel_comp_id );
    dcel_edge_id    outside_edge(dcel_cell_id id) const;
    dcel_edge_id    an_incident_edge(dcel_cell_id id) const;
    dcel_edge_id    an_edge_around(dcel_cell_id id) const
			{return outside_edge(id);}
    void	    set_outside_edge(dcel_cell_id id, dcel_edge_id edge)
			{ _cells[id].an_edge_around = edge; }
    void	    set_an_edge_around(dcel_cell_id id, dcel_edge_id edge)
			{ set_outside_edge(id, edge); }
    dcel_comp_id    first_inner_component(dcel_cell_id id) const
			{return _cells[id].inner_components;}
    dcel_comp_id    next_inner_component(dcel_cell_id , dcel_comp_id prev) const
			{return _components[prev].next_component;}
    bool	    is_inner_cell_of(dcel_cell_id cell, dcel_comp_id comp) const;
    void*	    info(dcel_cell_id id) const
			{return _cells[id].info;}
    void	    set_info(dcel_cell_id id, void *info)
			{_cells[id].info = info; }
    dcel_cell_id    first_cell() const;
    dcel_cell_id    next_cell(dcel_cell_id prev) const;
    int		    cell_count() const;
    void	    update_cell_idents() const;
    int		    int_ident(dcel_cell_id id) const;
    dcel_cell_id    cell_id(int) const;
			

    dcel_comp_id    new_component();
    void	    delete_component(dcel_comp_id id);
    bool	    is_valid_component(dcel_comp_id id) const;
    bool	    is_valid_or_no_component(dcel_comp_id id) const
			{return id == No_dcel_comp_id || is_valid_component(id);}
    dcel_cell_id    surrounding_cell(dcel_comp_id id) const
			{return _components[id].outer_cell;}
    void	    set_surrounding_cell(dcel_comp_id id, dcel_cell_id cell);
    dcel_vertex_id  a_vertex_of(dcel_comp_id id) const;
    dcel_edge_id    outside_edge(dcel_comp_id id) const;
    void	    set_outside_edge(dcel_comp_id id, dcel_edge_id edge);
    dcel_edge_id    an_edge_on_the_outside(dcel_comp_id id) const
			{ return outside_edge(id); }
    dcel_cell_id    first_inner_cell(dcel_comp_id id) const
			{return _components[id].inner_cells;}
    dcel_cell_id    next_inner_cell(dcel_comp_id , dcel_cell_id prev) const
			{return _cells[prev].next_cell;}
    bool	    is_inner_component_of(dcel_comp_id comp, dcel_cell_id cell) const;
    void*	    info(dcel_comp_id id) const
			{return _components[id].info;}
    void	    set_info(dcel_comp_id id, void *info)
			{_components[id].info = info; }
    dcel_comp_id    first_component() const;
    dcel_comp_id    next_component(dcel_comp_id prev) const;
    int		    component_count() const;
    void	    update_component_idents() const;
    int		    int_ident(dcel_comp_id id) const;
    dcel_comp_id    comp_id(int) const;

    dcel_cell_id    outer_cell() const {return _outer_cell;}
    void	    set_outer_cell(dcel_cell_id id) {_outer_cell = id; }
protected:
    dcel_vt_reray    _vertices;
    dcel_edge_reray  _edges;
    dcel_cell_reray  _cells;
    dcel_comp_reray  _components;
    dcel_cell_id    _outer_cell;
    mutable intreray	    _vertex_corr;
    mutable intreray	    _edge_corr;
    mutable intreray	    _cell_corr;
    mutable intreray	    _comp_corr;
    flags8	    _idflags;
    char *	    _msg;
    void	    upd_int_idents();
    void	    upd_vertex_idents();
    void	    upd_edge_idents();
    void	    upd_cell_idents();
    void	    upd_component_idents();
    void	    set_sym(dcel_edge_id id, dcel_edge_id sym)
			{ _edges[id].sym = sym; }
    void	    set_next(dcel_edge_id id, dcel_edge_id next)
			{ _edges[id].next = next; }
    void	    set_prev(dcel_edge_id id, dcel_edge_id prev)
			{ _edges[id].prev = prev; }
    void	    add_inner_cell(dcel_comp_id id, dcel_cell_id cell);
    void	    remove_inner_cell(dcel_comp_id id, dcel_cell_id cell);
    void	    add_inner_component(dcel_cell_id id, dcel_comp_id comp);
    void	    remove_inner_component(dcel_cell_id id, dcel_comp_id comp);
};

#define forall_vertices(dcel, x) \
    for (x = dcel.first_vertex(); \
	x != No_dcel_vertex_id; \
	x = dcel.next_vertex(x))

#define forall_edges(dcel, x) \
    for (x = dcel.first_edge(); \
	x != No_dcel_edge_id; \
	x = dcel.next_edge(x))

#define forall_cells(dcel, x) \
    for (x = dcel.first_cell(); \
	x != No_dcel_cell_id; \
	x = dcel.next_cell(x))

#define forall_components(dcel, x) \
    for (x = dcel.first_component(); \
	x != No_dcel_comp_id; \
	x = dcel.next_component(x))

extern void pgn_to_dcel(dcedgelist &dcel, const pl_pgn_ifc &pgn);

extern dcel_cell_id insert_pgn_in_cell(dcedgelist &dcel,
		    dcel_cell_id outside, const pl_pgn_ifc &pgn);

// file invoer en uitvoer 

extern ostream & operator<<(ostream &out, const dcedgelist &dcel) ;

extern istream & operator>>(istream &in, dcedgelist &) ;

extern void write_without_brackets(ostream &out, const dcedgelist &dcel) ;

extern void read_without_brackets(istream &in, dcedgelist &dcel) ;


#endif
