// $Id: convpieces.C,v 1.3 1995/10/19 10:51:03 geert Exp $

#include <PLAGEO/pgn.h>
#include <geolib.h>





//////////////////////////////////////////////////////////////
//							    //
//  SPLITTING POLYGONS IN CONVEX PIECES			    //
//							    //
//////////////////////////////////////////////////////////////

static int colinear(const pl_vec &vec1, const pl_vec &vec2, const pl_vec &vec3)
{
    return cross(vec3-vec1, vec2-vec1) == 0;
}

static int operator==(const pl_vec &vec1, const pl_vec &vec2)
{
    return vec1.x()==vec2.x() && vec1.y()==vec2.y();
}

static int clean_up(pl_pgn &out, const pl_pgn &in)
{
    int i, last;
    static pl_vecreray vrr;
    vrr.newsize(0);
    if (in.size() == 0)
	return 0;
    vrr.append(in.vertex(0));
    i = 1;
    while (i<in.size() && in.vertex(i)==vrr[0])
	i++;
    if (i >= in.size())
	return 0;
    vrr.append(in.vertex(i));
    last = 1;
    for (; i< in.size(); i++) {
	if (in.vertex(i) == vrr[last])
	    continue;
	if (colinear(vrr[last-1], vrr[last], in.vertex(i)))
	    vrr[last] = in.vertex(i);
	else {
	    vrr.append(in.vertex(i));
	    last++;
	}
    }
    if (last<2)
	return 0;
    if (colinear(vrr[last-1], vrr[last], vrr[0])) {
	vrr.grow(-1);
	last--;
    }
    if (last<2)
	return 0;
    if (colinear(vrr[last], vrr[0], vrr[1])) {
	vrr[0] = vrr[last];
	vrr.grow(-1);
	last--;
    }
    if (last<2)
	return 0;
    out = vrr;
    return 1;
}

static void convex_pieces2(reray<pl_pgn> &parts, const pl_pgn &pgn)
// Splits a polygon into convex parts. The method used is as follows:
//	- Find a concave vertex
//	- Take the extension of one of its edges
//	- Find nearest intersection
//	- Split polygon with this edge in two pieces
//	- Repeat recursively on the two pieces
{
    int i;
    if (pgn.size() <= 3) {
	parts.append(pgn);
	return;
    }
    //// Find a concave vertex
    int ind;			    // Index of the concave vertex
    ind = -1;
    for (i=0; i<pgn.size() && ind < 0; i++)
        if (is_to_the_left(pgn.vertex(i), 
		    pl_line(pgn.vertex_mod(i-1), pgn.vertex_mod(i+1))))
	    ind = i;
    if (ind<0) {    // no concave vertex
	parts.append(pgn);
	return;
    }
    // Find the edge hit by the extension of the line
    pl_unitvec edgedir, edgenormal; // The edge running from a concave vertex.
    pl_vec concavevt;		    // The concave vertex.
    s_coord bestdist;		    // Distance to nearest intersection
    pl_vec bestinter;		    // Nearest intersection with ray
    int bestind = -1;		    // Index of first hit edge
    pl_vec newvec, lastvec;
    s_coord newx, lastx;
    int newside, lastside;
    s_coord newdist;		    // current distance
    concavevt = pgn.vertex(ind);
    pgn.set_edge_mod(ind-1);
    edgedir = pgn.cur_direction(); // ???? direct direction ?
    edgenormal = edgedir.normal();
    bestdist= max_of_type(s_coord(0));
    lastvec = pgn.vertex_mod(ind+1) - concavevt;
    lastx = dot(lastvec, edgenormal);
    lastside = (lastx>0) ? 1 : (lastx<0) ? -1 : 0;
    for (i=ind+2; i< ind+pgn.size()-1; i++) {
	newvec = pgn.vertex_mod(i) - concavevt;
	newx = dot(newvec, edgenormal);
	newside = (newx>0) ? 1 : (newx<0) ? -1 : 0;
	if (newside != lastside) {
	    // new intersection found
	    if (lastside == 0) {
		newdist = dot(edgedir, lastvec);
	    } else if (newside == 0) {
		newdist = dot(edgedir, newvec);
	    } else {
		s_coord newy, lasty;
		newy = dot(edgedir, newvec);
		lasty = dot(edgedir, lastvec);
		newdist = (lastx*newy-newx*lasty)/(lastx-newx);
	    }
	    if (newdist>=0 && newdist<bestdist) {
		bestdist = newdist;
		bestind = i;
	    }
	}
	lastvec = newvec;
	lastx = newx;
	lastside = newside;
    }
    bestinter = concavevt+bestdist*edgedir;
    //// Cut the polygon in two pieces
    pl_vecreray piece1, piece2;	// The two piece of the polygon
    i = ind;
    while (i < bestind ) {
        piece1.append(pgn.vertex_mod(i));
        i++;
    }
    if ((bestinter-piece1.last()).sqlength() > 0)
	piece1.append(bestinter);
    else
	if (colinear(piece1[0], piece1.last(), piece1[piece1.size()-2]))
	    piece1.grow(-1);
    if ((bestinter-pgn.vertex_mod(i)).sqlength() > 0)
	piece2.append(bestinter);
    else
	if (colinear(pgn.vertex_mod(i), pgn.vertex_mod(i+1),
			    pgn.vertex_mod(ind+pgn.size()-1)))
	    i++;
    while (i <= ind+pgn.size()-1 ) {
        piece2.append(pgn.vertex_mod(i));
        i++;
    }
	
    //// Call recursively
    convex_pieces2(parts, piece1);
    convex_pieces2(parts, piece2);
}


void convex_pieces(reray<pl_pgn> &parts, const pl_pgn &pgn)
// Splits a polygon into convex parts. The method used is as follows:
//	- Find a concave vertex
//	- Take the extension of one of its edges
//	- Find nearest intersection
//	- Split polygon with this edge in two pieces
//	- Repeat recursively on the two pieces
{
    parts.newsize(0);
    //// Make a copy, cleaning it up
    pl_pgn clean_copy;			// The copy
    if (!clean_up(clean_copy, pgn))
	return;
    if (clean_copy.is_clockwise())
	clean_copy.invert_orientation();
    convex_pieces2(parts, clean_copy);
}


/*

//////////////////////////////////////////////////////////////
//							    //
//  Alternative way			    		    //
//							    //
//////////////////////////////////////////////////////////////



void convex_pieces_alt(reray<pl_pgn> &parts, const pl_pgn &pol)
// Splits a polygon into convex parts. The method used is as follows:
//	- Find a concave vertex
//	- Take the extension of one of its edges
//	- Find nearest intersection
//	- Split polygon with this edge in two pieces
//	- Repeat recursively on the two pieces
{
    int i;
    
    //// Make a copy, cleaning it up
    pl_pgn tmp;			// The copy
    pl_vecreray vert;		// Vertices in the copy
    for (i=0; i< pol.size(); i++)
      if (pol.vertex(i).x() != pol.vertex_mod(i+1).x() ||
	    pol.vertex(i).y() != pol.vertex_mod(i+1).y() )
	vert.append(pol.vertex(i));
    tmp = pl_pgn(vert);
	
    //// Check whether convex. If so, ready.
    if (tmp.size() == 3 || tmp.is_convex()) {
	parts.newsize(1);
	parts[0] = tmp;
	return;
    }
    if (tmp.is_clockwise())
	tmp.invert_orientation();

    //// Find a concave vertex
    int ind;				// Index of the concave vertex
    ind = -1;
    for (i=0; ind < 0; i++)
        if (is_to_the_left(tmp.vertex(i), 
		    pl_line(tmp.vertex_mod(i-1), tmp.vertex_mod(i+1))))
	    ind = i;
    
    //// Find the edge hit by the extension of the line
    pl_ray ray;				// The ray running from a concave vertex
    float bestdist= 0.0;		// Distance to nearest intersection
    float dist;				// current distance
    pl_vec bestinter;		// Nearest intersection with ray
    pl_vec inter;			// current intersection
    int bestind = -1;			// Index of first hit edge
    ray = pl_ray(tmp.vertex(ind), tmp.edge_mod(ind-1).direction());
    for (i=0; i< tmp.size(); i++)
	if (i != ind && i != (ind-1+tmp.size()) % tmp.size() ) {
        if (intersect(inter, tmp.edge(i), ray)) {
    	    dist = clearance(tmp.vertex(ind), inter);
    	    if ( bestind < 0 || dist < bestdist) {
		bestdist = dist; bestinter = inter; bestind = i;
	    }
        }
    }
    
    //// Cut the polygon in two pieces
    pl_vecreray piece1, piece2;	// The two piece of the polygon
    i = ind;
    while (i != bestind ) {
        piece1.append(tmp.vertex(i));
        i = (i+1) % tmp.size();
    }
    piece1.append(tmp.vertex(bestind));
    piece1.append(bestinter);
    piece2.append(bestinter);
    i = (bestind+1) % tmp.size();
    while (i != ind ) {
        piece2.append(tmp.vertex(i));
        i = (i+1) % tmp.size();
    }
	
    //// Call recursively
    reray<pl_pgn> secondhalf;		// Pieces from the second recursive call   
    convex_pieces(parts, piece1);
    convex_pieces(secondhalf, piece2);
    for (i=0; i < secondhalf.size(); i++)
	parts.append(secondhalf[i]);   
}

*/
