// $Id: geolib.h,v 1.4 2000/03/21 12:38:18 geert Exp $

#include <PLAGEO/pgn.h>
#include <dcel.h>

//////////////////////////////////////////////////
//						//
//  GeoLib					//
//	A library of higher level routines	//
//	based on plageo				//
//						//
//  Version 2.1, Nov 16, 1994			//
//////////////////////////////////////////////////





//////////////////////////////////////////////////////////////
//							    //
//  COMPUTING CONVEX HULLS				    //
//							    //
//////////////////////////////////////////////////////////////

pl_pgn convex_hull(const pl_vecreray &vert);
// Computes the convex hull of the set of vertices using quickhull

pl_pgn convex_hull(const pl_pgn &pgn);
// Computes the convex hull of the polygon using quickhull on the vertices



//////////////////////////////////////////////////////////////
//							    //
//  SPLITTING POLYGONS IN CONVEX PIECES			    //
//							    //
//////////////////////////////////////////////////////////////

void convex_pieces(reray<pl_pgn> &parts, const pl_pgn &pgn);
// Splits a polygon into convex parts. The method used is as follows:
//	- Find a concave vertex
//	- Take the extension of one of its edges
//	- Find nearest intersection
//	- Split polygon with this edge in two pieces
//	- Repeat recursively on the two pieces



//////////////////////////////////////////////////////////////
//							    //
//  COMPUTING MINKOWSKI SUMS				    //
//							    //
//////////////////////////////////////////////////////////////

extern pl_pgn Minkowski_sum(const pl_pgn &pgn1, const pl_pgn &pgn2);
// Computes the Minkowski sum of the two polygons. Only defined
// when both are convex. Otherwise the Minkowski sums of the
// convex hulls is computed.

extern void Minkowski_sum(reray<pl_pgn> &result,
			const pl_pgn &pgn1, const pl_pgn &pgn2);
// Computes the Minkowski sum of the two polygons. A set of convex polygons
// is returned that together cover the Minkowski sum.



//////////////////////////////////////////////////////////////
//							    //
//		COVERAGE OF POLYGONS			    //
//							    //
//		THE UNION OF POLYGONS			    //
//							    //
//////////////////////////////////////////////////////////////

extern void pgncoverage(
    dcedgelist &dcel,		// The resulting doubly connected edge list.
    const reray<pl_pgn> &in,	// The input polygons.
    int (*equi_class)(int)	// The functions that determines equivalence
				// classes based on the covering number.
);
/*
 * pgncoverage takes a reray of polygons and makes a doubly connected edgelist.
 * This dcel will divide the space in regions depending on by how many
 * polygons a region is covered.
 * The third argument is a function that maps the covering number to an
 * equivalence class. This must be a deterministic function from (non negative)
 * integer to integer.
 * In the resulting dcel, all points of a single cell have a coverage that
 * belongs to the same equivalence class. Points of cells that are separated
 * by a common edge-pair always belong to a different equivalence class.
 * The info field of a cell, when cast to int, contains the equivalence class.
 */


extern void pgnunion(dcedgelist &dcel, const reray<pl_pgn> &in);
/*
 * This function computes the union of a number of polygons.
 * It uses the function 'pgncoverage' to do so.
 * The equivalence function it supplies returns 0 when called with
 * argument 0 and 1 otherwise. So there are two equivalence classes:
 * covered by no polygon and covered by at least one polygon.
 */

#include <min_enclosing_circle.h>

