#include <stdlib.h>
#include <row.h>
#include <PLAGEO/base.h>
#include <PLAGEO/clearance1.h>
#include <PLAGEO/intersect.h>
#include <PLAGEO/curved.h>


// #define COUNT_CALLS 1

#ifdef COUNT_CALLS
static int count1;
static int count2;
#endif

// The following is a specialised version of swap.
// It swaps the values of two or three positions of a row.
// The first parameter gives the row, the second and third parameter give the
// indices. The first index (i), must be smaller than 3.
// The second index (j) must be bigger than i.
// This version of swap takes two steps to swap a value from the beginning of
// the row to a position further than 3. In the meantime the value will be
// checked again. This is useful for keeping `promising' values.
void special_swap(row<int> &r, int i, int j)
{
    int t;
    t = r[j];
    if (j > 3) {
	r[j] = r[3];
	r[3] = r[i];
    } else {
	r[j] = r[i];
    }
    r[i] = t;
}

// compute the minimal circle that contains two points.
void two_points_circle(pl_vec &mid, s_coord &d,
	    pl_vec const& pt0, pl_vec const& pt1)
{
#ifdef COUNT_CALLS
    count1++;
#endif
    mid = 0.5*(pt0 + pt1);
    d = sqclearance(mid, pt1);
}

void middelloodlijn(pl_line &result, pl_vec const& pt1, pl_vec const& pt2)
{
    pl_vec diff, dir, point;
    diff = pt2 - pt1;
    dir = pl_vec(diff.y(), -diff.x());	// normal
    point = pt1 + 0.5*diff;
    result = pl_line(point, point+dir);
}

// Compute the circle that contains three points.
void three_points_circle(pl_vec &mid, s_coord &d,
	    pl_vec const& pt0, pl_vec const& pt1, pl_vec const& pt2)
{
#ifdef COUNT_CALLS
    count1++;
#endif
// bereken snijpunt van twee midelloodlijnen.
    pl_line l1, l2;
    middelloodlijn(l1, pt0, pt1);
    middelloodlijn(l2, pt0, pt2);
    intersect(mid, l1, l2);
// bereken distance van snijpunt tot een van de punten.
    d = sqclearance(mid, pt0);
}



template <class ArrayContainer>
void find_known(pl_vec &mid, s_coord &d, int &r, row<int> &indices,
	int known, int n, ArrayContainer const& points)
{
    if (known == 3) {
	three_points_circle(mid, d, points[indices[0]],
	    points[indices[1]], points[indices[2]]);
	r = 3;
	return;
    }
    two_points_circle(mid, d, points[indices[0]], points[indices[1]]);
    r = 2;
    int i;
    for (i=2; i<n; i++) {
#ifdef COUNT_CALLS
    count2++;
#endif
	if (sqclearance(points[indices[i]], mid) > d) {
	    special_swap(indices, known, i);
//	    random_permutation((row_iterator<int>)(
//		    indices.begin()+(known+1)), (int)(i-known));
	    find_known(mid, d, r, indices, known+1, i+1, points);
	}
    }
}

// random integer R: 0 <= R < n
int random_int(int n)
{
    return rand() % n;
}

/*
template <class RAIt, class T, class Distance>
void random_permutation(RAIt<T, Distance> it, int n)
{
    int i, k;
    for (i=0; i<n; i++) {
	k = random_int(n-i);
	if (k != 0) {
	    T temp(*it);
	    *it = *(it+k);
	    *(it+k) = temp;
	}
	++it;
    }
}
*/

template <class T>
void random_permutation(row_iterator<T> it, int n)
{
    int i, k;
    for (i=0; i<n; i++) {
	k = random_int(n-i);
	if (k != 0) {
	    T temp(*it);
	    *it = *(it+k);
	    *(it+k) = temp;
	}
	++it;
    }
}


template <class ArrayContainer>
void minimal_enclosing_disc(int &i1, int &i2, int &i3,
    pl_vec &mid, s_coord &d, ArrayContainer const& points, int n)
{
    int i, r;
    if (n < 2) {
	if (n <= 0) {
	    i1 = 0;
	    i2 = 0;
	    i3 = 0;
	    mid = pl_origin;
	    d = s_coord(0);
	    return;
	}
	 // n == 1
	i1 = 0;
	i2 = 1;
	i3 = 1;
	mid = points[0];
	d = s_coord(0);
	return;
    }
    row<int> indices(n);
    for (i=0; i<indices.size(); i++)
	indices[i] = i;
    random_permutation((row_iterator<int>)(indices.begin()),
		(int)(indices.size()));
#ifdef COUNT_CALLS
    count1 = 0;
    count2 = 0;
#endif
    find_known(mid, d, r, indices, 0, (int)(indices.size()), points);
#ifdef COUNT_CALLS
    cerr<<"three_point_circle: "<<count1<<".\nsqclearance: "<<count2<<".\n"<<flush;
#endif
    i1 = indices[0];
    i2 = indices[1];
    i3 = (r==2) ? n : indices[2];
}

template <class ArrayContainer>
pl_disc minimal_enclosing_disc(int &i1, int &i2, int &i3,
			    ArrayContainer const& points, int n)
{
    pl_vec mid;
    s_coord square_radius;
    minimal_enclosing_disc(i1, i2, i3, mid, square_radius, points, n);
    return pl_disc(mid, sqrt_pp(square_radius));
}

template <class ArrayContainer>
pl_disc minimal_enclosing_disc(ArrayContainer const& points, int n)
{
    int i1, i2, i3;
    return minimal_enclosing_disc(i1, i2, i3, points, n);
}


