#ifndef GJ_FLAGS_H
#define GJ_FLAGS_H

/**** flags8 ****
 *
 * flags8 supplies 8 flags.
 * initially the flags are not set.
 * one or more flags can be queried by means of is_set(char x).
 * this funtion returns true iff all flags denoted by x are true.
 * the flags denoted by x are those bits of x that are 1, so
 * isvalid(0x2c) (0010 1100) inspects 3 flags.
 * the other routines set, unset or toggle one or more bits.
 *
 ********************/


class flags8 {
    char	flags;			// internally: bit 0:set, 1:not set
public:
		flags8()		{ flags = all_flags; }
    bool	is_set(char x) const	{ return ((x & flags) == 0); }
    void	set(char x)		{ flags &= ~x; }
    void	unset(char x)		{ flags |= x; }
    void	toggle(char x)		{ flags ^= x; }
    enum flags { flag0 = 0x01, 
		 flag1 = 0x02, 
		 flag2 = 0x04, 
		 flag3 = 0x08, 
		 flag4 = 0x10, 
		 flag5 = 0x20, 
		 flag6 = 0x40, 
		 flag7 = 0x80, 
		 all_flags = 0xff }; 
};

typedef unsigned short ushort;

class flags16 {
    ushort  flags;			// internally: bit 0:set, 1:not set
public:
		flags16()		{ flags = all_flags; }
    bool	is_set(ushort x) const	{ return ((x & flags) == 0); }
    void	set(ushort x)		{ flags &= ~x; }
    void	unset(ushort x)		{ flags |= x; }
    void	toggle(ushort x)	{ flags ^= x; }
    enum flags { flag0 = 0x01, 
		 flag1 = 0x02, 
		 flag2 = 0x04, 
		 flag3 = 0x08, 
		 flag4 = 0x10, 
		 flag5 = 0x20, 
		 flag6 = 0x40, 
		 flag7 = 0x80, 
		 flag8 = 0x100, 
		 flag9 = 0x200, 
		 flag10 = 0x400, 
		 flag11 = 0x800, 
		 flag12 = 0x1000, 
		 flag13 = 0x2000, 
		 flag14 = 0x4000, 
		 flag15 = 0x8000, 
		 all_flags = 0xffff }; 
};

typedef unsigned long ulong;

#ifndef __hpux
class flags32 {
    ulong  flags;			// internally: bit 0:set, 1:not set
public:
		flags32()		{ flags = (ulong) all_flags; }
    bool	is_set(ulong x) const	{ return ((x & flags) == 0); }
    void	set(ulong x)		{ flags &= ~x; }
    void	unset(ulong x)		{ flags |= x; }
    void	toggle(ulong x)	{ flags ^= x; }
    enum flags { flag0 =         0x1, 
		 flag1 =         0x2, 
		 flag2 =         0x4, 
		 flag3 =         0x8, 
		 flag4 =        0x10, 
		 flag5 =        0x20, 
		 flag6 =        0x40, 
		 flag7 =        0x80, 
		 flag8 =       0x100, 
		 flag9 =       0x200, 
		 flag10 =      0x400, 
		 flag11 =      0x800, 
		 flag12 =     0x1000, 
		 flag13 =     0x2000, 
		 flag14 =     0x4000, 
		 flag15 =     0x8000, 
		 flag16 =    0x10000, 
		 flag17 =    0x20000, 
		 flag18 =    0x40000, 
		 flag19 =    0x80000, 
		 flag20 =   0x100000, 
		 flag21 =   0x200000, 
		 flag22 =   0x400000, 
		 flag23 =   0x800000, 
		 flag24 =  0x1000000, 
		 flag25 =  0x2000000, 
		 flag26 =  0x4000000, 
		 flag27 =  0x8000000, 
		 flag28 = 0x10000000, 
		 flag29 = 0x20000000, 
		 flag30 = 0x40000000, 
		 flag31 = 0x80000000, 
		 all_flags = 0xffffffff }; 
};
#endif

#endif // GJ_FLAGS_H
