#ifndef RERAYH
#define RERAYH 1

#include <iostream>
#include <cstdlib>
#include <iterator>

#ifdef reray
#define reray reray
#endif

#ifdef reray_iterator
#define reray_iterator reray_iterator
#endif

#ifdef const_reray_iterator
#define const_reray_iterator const_reray_iterator
#endif

template <class T>
class reray_iterator;

template <class T>
class const_reray_iterator;



template<class T>
class   reray {
public:
    typedef T value_type;
    typedef T* pointer;
    typedef T& reference;
    typedef const T& const_reference;
    typedef unsigned int size_type;
    typedef int difference_type;
    typedef reray_iterator<T> iterator;
    typedef const_reray_iterator<T> const_iterator;

                reray()         {reserved_size = sz = 0; v = 0;}
                reray(const reray&);
                reray(unsigned int s);
                ~reray() { if (v != 0) delete [] v;
                            reserved_size = sz = 0;
                         }
    iterator    begin()         {return reray_iterator<T>(*this, 0);}
    const_iterator      begin() const   {
        return (const_reray_iterator<T>(*this, 0));
    }
    iterator    end()           {return reray_iterator<T>(*this, sz);   }
    const_iterator      end()   const   {
        return (const_reray_iterator<T>(*this, sz));
    }
    reray&      operator=(const reray<T> &);
    size_type   size() const            { return sz; }
    size_type   max_size() const {
        return max(size_type(1), size_type(INT_MAX/sizeof(T)));
    }
    T           elem(unsigned int i) const      ;// safe indexing
#if defined(RERAYCHECKS) || defined(BOUNDCHECKS)
    T           &operator[] (unsigned int index) {
        if (index >= size()) {
            cerr<<"Out of bounds reray indexing.\nCore dumped.\n";
            abort();
        }
        return v[index];
    }
const T &   operator[] (unsigned int index) const {
        if (index >= size()) {
            cerr<<"Out of bounds reray indexing.\nCore dumped.\n";
            abort();
        }
        return v[index];
    }
    T           &last() {
        if (size() == 0) {
            cerr<<"Out of bounds reray indexing.\nCore dumped.\n";
            abort();
        }
        return v[sz-1];
    }
const T &   last() const        {
        if (size() == 0) {
            cerr<<"Out of bounds reray indexing.\nCore dumped.\n";
            abort();
        }
        return v[sz-1];
    }
#else
    T           &operator[](unsigned int index)  // unsafe indexing
                { return v[index]; }
const T &       operator[](unsigned int index) const  // unsafe indexing
                { return v[index]; }
    T           &last() { return v[sz-1]; }
const T &       last() const    { return v[sz-1]; }
#endif
    void        newsize(unsigned int);              // invalidates reray
    void        resize(unsigned int);               // keeps common part valid
    void        grow(int i)     { int nsz = sz+i; resize(nsz>0 ? nsz : 0); }
    void        replace (unsigned int index, const T &s);
    void        append(const T &s)      { grow(1); last()=s;}
    void        compactify() const;
    void        sort(int (* compare)(const T &, const T&));
    void        sort(int (* compare)(T, T));
    void        swap_contents(reray<T> &) ;
    void        swap(reray<T> &b) { swap_contents(b); }
    void        swap(unsigned int i, unsigned int j) ;
protected:
    T*                  v;
    unsigned int        reserved_size;
    unsigned int        sz;
};

template <class T>
inline void swap_contents(reray<T> &rr1, reray<T> &rr2)
{ rr1.swap_contents(rr2); }

template <class T>
inline void swap(reray<T> &rr1, reray<T> &rr2)
{ rr1.swap(rr2); }

template <class T>
bool operator== (const reray<T> & x, const reray<T> & y);

template <class T>
bool operator< (const reray<T> & x, const reray<T> & y);

template<class T>
std::istream& operator>>(std::istream &, reray<T> &);

template<class T>
std::ostream& operator<<(std::ostream &, const reray<T> &);

// ********

template<class T>
class reray_iterator: public random_access_iterator<T, int> {
public:
    reray_iterator(): _reray(0), _index(-1) {}
    reray_iterator(reray<T> &r,  int i):_reray(&r), _index(i) {}
    bool operator==(const reray_iterator<T> &other) const {
        return (_reray == other._reray) && (_index == other._index);
    }
#ifdef BOUNDCHECKS
    T& operator*() const {
        if (_reray == 0 || _index<0) {
            cerr<<"Invalid reray iterator dereferenced.\nCore dumped.\n";
            abort();
        }
        return (*_reray)[_index];
    }
    reray_iterator<T>& operator++() {
        if (_reray == 0 || _index<0) {
            cerr<<"Invalid reray iterator incremented.\nCore dumped.\n";
            abort();
        }
        _index++;
        return *this;
    }
    reray_iterator<T> operator++(int) {
        if (_reray == 0 || _index<0) {
            cerr<<"Invalid reray iterator incremented.\nCore dumped.\n";
            abort();
        }
        _index++;
        return reray_iterator<T>(*_reray, _index-1);
    }
    reray_iterator<T>& operator--() {
        if (_reray == 0 || _index<0) {
            cerr<<"Invalid reray iterator incremented.\nCore dumped.\n";
            abort();
        }
        if (_index == 0) {
            cerr<<"reray iterator decremented below zero.\nCore dumped.\n";
            abort();
        }
        _index--;
        return *this;
    }
    reray_iterator<T> operator--(int) {
        if (_reray == 0 || _index<0) {
            cerr<<"Invalid reray iterator incremented.\nCore dumped.\n";
            abort();
        }
        if (_index == 0) {
            cerr<<"reray iterator decremented below zero.\nCore dumped.\n";
            abort();
        }
        _index--;
        return reray_iterator<T>(*_reray, _index+1);
    }
    reray_iterator<T>& operator+=(int n) {
        if (_reray == 0 || _index<0) {
            cerr<<"Invalid reray iterator incremented.\nCore dumped.\n";
            abort();
        }
        _index += n;
        return *this;
    }
    int operator-(const reray_iterator<T> &b) const {
        if (_reray == 0 || _index<0) {
            cerr<<"Invalid reray iterator compared.\nCore dumped.\n";
            abort();
        }
        if (b._reray == 0 || b._index<0) {
            cerr<<"Invalid reray iterator compared.\nCore dumped.\n";
            abort();
        }
        if (_reray != b._reray) {
            cerr<<"Incomparable reray iterators compared.\nCore dumped.\n";
            abort();
        }
        return b._index - _index;
    }
#else
    T& operator*() const {
        return (*_reray)[_index];
    }
    reray_iterator<T>& operator++() {
        _index++;
        return *this;
    }
    reray_iterator<T> operator++(int) {
        _index++;
        return reray_iterator<T>(*_reray, _index-1);
    }
    reray_iterator<T>& operator--() {
        _index--;
        return *this;
    }
    reray_iterator<T> operator--(int) {
        _index--;
        return reray_iterator<T>(*_reray, _index+1);
    }
    reray_iterator<T>& operator+=(int n) {
        _index += n;
        return *this;
    }
    int operator-(const reray_iterator<T> &b) const {
        return b._index - _index;
    }
#endif
    reray_iterator<T>& operator-=(int n) {
        _index += (-n);
        return *this;
    }
protected:
    reray<T> *_reray;
    int     _index;
friend class const_reray_iterator<T>;
};

template <class T>
reray_iterator<T> operator+(const reray_iterator<T> &a, int n)
{
    reray_iterator<T> tmp(a);
    return (tmp += n);
}

template <class T>
reray_iterator<T> operator+(int n, const reray_iterator<T> &a)
{
    reray_iterator<T> tmp(a);
    return (tmp += n);
}

template <class T>
reray_iterator<T> operator-(const reray_iterator<T> &a, int n)
{
    reray_iterator<T> tmp(a);
    return (tmp += (-n));
}

template <class T>
bool operator<(const reray_iterator<T> &a, const reray_iterator<T> &b)
{
    return (b-a > 0);
}

template<class T>
class const_reray_iterator: public random_access_iterator<T, int> {
public:
    const_reray_iterator(): _reray(0), _index(-1) {}
    const_reray_iterator(const reray<T> &r,  int i):_reray(&r), _index(i) {}
    const_reray_iterator(const reray_iterator<T> &ri)
        :_reray(ri._reray), _index(ri._index) {}
    bool operator==(const const_reray_iterator<T> &other) const {
        return (_reray == other._reray) && (_index == other._index);
    }
#ifdef BOUNDCHECKS
    const T& operator*() const {
        if (_reray == 0 || _index<0) {
            cerr<<"Invalid reray iterator dereferenced.\nCore dumped.\n";
            abort();
        }
        return (*_reray)[_index];
    }
    const_reray_iterator<T>& operator++() {
        if (_reray == 0 || _index<0) {
            cerr<<"Invalid reray iterator incremented.\nCore dumped.\n";
            abort();
        }
        _index++;
        return *this;
    }
    const_reray_iterator<T> operator++(int) {
        if (_reray == 0 || _index<0) {
            cerr<<"Invalid reray iterator incremented.\nCore dumped.\n";
            abort();
        }
        _index++;
        return const_reray_iterator<T>(*_reray, _index-1);
    }
    const_reray_iterator<T>& operator--() {
        if (_reray == 0 || _index<0) {
            cerr<<"Invalid reray iterator incremented.\nCore dumped.\n";
            abort();
        }
        if (_index == 0) {
            cerr<<"reray iterator decremented below zero.\nCore dumped.\n";
            abort();
        }
        _index--;
        return *this;
    }
    const_reray_iterator<T> operator--(int) {
        if (_reray == 0 || _index<0) {
            cerr<<"Invalid reray iterator incremented.\nCore dumped.\n";
            abort();
        }
        if (_index == 0) {
            cerr<<"reray iterator decremented below zero.\nCore dumped.\n";
            abort();
        }
        _index--;
        return const_reray_iterator<T>(*_reray, _index+1);
    }
    const_reray_iterator<T>& operator+=(int n) {
        if (_reray == 0 || _index<0) {
            cerr<<"Invalid reray iterator incremented.\nCore dumped.\n";
            abort();
        }
        _index += n;
        return *this;
    }
    int operator-(const const_reray_iterator<T> &b) const {
        if (_reray == 0 || _index<0) {
            cerr<<"Invalid reray iterator compared.\nCore dumped.\n";
            abort();
        }
        if (b._reray == 0 || b._index<0) {
            cerr<<"Invalid reray iterator compared.\nCore dumped.\n";
            abort();
        }
        if (_reray != b._reray) {
            cerr<<"Incomparable reray iterators compared.\nCore dumped.\n";
            abort();
        }
        return b._index - _index;
    }
#else
    const T& operator*() const {
        return (*_reray)[_index];
    }
    const_reray_iterator<T>& operator++() {
        _index++;
        return *this;
    }
    const_reray_iterator<T> operator++(int) {
        _index++;
        return const_reray_iterator<T>(*_reray, _index-1);
    }
    const_reray_iterator<T>& operator--() {
        _index--;
        return *this;
    }
    const_reray_iterator<T> operator--(int) {
        _index--;
        return const_reray_iterator<T>(*_reray, _index+1);
    }
    const_reray_iterator<T>& operator+=(int n) {
        _index += n;
        return *this;
    }
    int operator-(const const_reray_iterator<T> &b) const {
        return b._index - _index;
    }
#endif
    const_reray_iterator<T>& operator-=(int n) {
        _index += (-n);
        return *this;
    }
protected:
    const reray<T> *_reray;
    int     _index;
};

template <class T>
const_reray_iterator<T> operator+(const const_reray_iterator<T> &a, int n)
{
    const_reray_iterator<T> tmp(a);
    return (tmp += n);
}

template <class T>
const_reray_iterator<T> operator+(int n, const const_reray_iterator<T> &a)
{
    const_reray_iterator<T> tmp(a);
    return (tmp += n);
}

template <class T>
const_reray_iterator<T> operator-(const const_reray_iterator<T> &a, int n)
{
    const_reray_iterator<T> tmp(a);
    return (tmp += (-n));
}

template <class T>
bool
operator<(const const_reray_iterator<T> &a, const const_reray_iterator<T> &b)
{
    return (b-a > 0);
}



#undef const_reray_iterator
#undef reray_iterator
#undef reray

#include "reray.c"

#endif

