#ifndef ROWH
#define ROWH 1

#include <iostream>
#include <stdlib.h>
#include <limits.h>
#include <iterator>

// remove possible redefinitions.

#ifdef row
#define row row
#endif

#ifdef row_iterator
#define row_iterator row_iterator
#endif

#ifdef const_row_iterator
#define const_row_iterator const_row_iterator
#endif

template <class T>
class row_iterator;

template <class T>
class const_row_iterator;

template<class T>
class   row {
public:
    typedef T value_type;
    typedef T* pointer;
    typedef T& reference;
    typedef const T& const_reference;
    typedef unsigned int size_type;
    typedef int difference_type;
    typedef row_iterator<T> iterator;
    typedef const_row_iterator<T> const_iterator;

                row()           {sz = 0; v = 0;}
                row(const row &);
                row(unsigned int s);
                ~row()          { if (v != 0) delete [] v; sz=0; v=0;}
    iterator    begin()         {return row_iterator<T>(*this, 0);}
    const_iterator      begin() const   {
        return (const_row_iterator<T>(*this, 0));
    }
    iterator    end()           {return row_iterator<T>(*this, sz);     }
    const_iterator      end()   const   {
        return (const_row_iterator<T>(*this, sz));
    }
    row&        operator=(const row<T> &);
    size_type   size() const            { return sz; }
    size_type   max_size() const {
        return max(size_type(1), size_type(INT_MAX/sizeof(T)));
    }

    void        newsize(unsigned int s) ;
    T           elem(unsigned int i) const      ;// safe indexing
#ifdef BOUNDCHECKS
    T &         operator[] (unsigned int index) {
                    if (index >= size()) {
                        cerr<<"Out of bounds row indexing.\nCore dumped.\n";
                        abort();
                    }
                    return v[index];
                }
const T &       operator[] (unsigned int index) const {
                    if (index >= size()) {
                        cerr<<"Out of bounds row indexing.\nCore dumped.\n";
                        abort();
                    }
                    return v[index];
                }
    T           &last() {
                    if (size() == 0) {
                        cerr<<"Out of bounds row indexing.\nCore dumped.\n";
                        abort();
                    }
                    return v[sz-1]; }
const T &       last() const    {
                    if (size() == 0) {
                        cerr<<"Out of bounds row indexing.\nCore dumped.\n";
                        abort();
                    }
                    return v[sz-1]; }
#else
    T           &operator[](unsigned int index)  // unsafe indexing
                { return v[index]; }
const T &       operator[](unsigned int index) const  // unsafe indexing
                { return v[index]; }
    T           &last() { return v[sz-1]; }
const T &       last() const    { return v[sz-1]; }
#endif
    void        replace (unsigned int index, const T &s);
    void        sort(int (* compare)(const T &, const T&));
    void        sort(int (* compare)(T, T));
    void        swap_contents(row<T> &) ;
    void        swap(unsigned int i, unsigned int j) ;
protected:
    T*                  v;
    unsigned int        sz;
};






template <class T>
inline void swap_contents(row<T> &rr1, row<T> &rr2)
{ rr1.swap_contents(rr2); }

template <class T>
bool operator== (const row<T> & x, const row<T> & y);

template <class T>
bool operator< (const row<T> & x, const row<T> & y);

template<class T>
std::istream& operator>>(std::istream &, row<T> &);

template<class T>
std::ostream& operator<<(std::ostream &, const row<T> &);

// ********

template<class T>
class row_iterator: public random_access_iterator<T, int> {
public:
    row_iterator(): _row(0), _index(-1) {}
    row_iterator(row<T> &r,  int i):_row(&r), _index(i) {}
    bool operator==(const row_iterator<T> &other) const {
        return (_row == other._row) && (_index == other._index);
    }
#ifdef BOUNDCHECKS
    T& operator*() {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator dereferenced.\nCore dumped.\n";
            abort();
        }
        return (*_row)[_index];
    }
const T& operator*() const {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator dereferenced.\nCore dumped.\n";
            abort();
        }
        return (*_row)[_index];
    }
    row_iterator<T>& operator++() {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator incremented.\nCore dumped.\n";
            abort();
        }
        _index++;
        return *this;
    }
    row_iterator<T> operator++(int) {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator incremented.\nCore dumped.\n";
            abort();
        }
        _index++;
        return row_iterator<T>(*_row, _index-1);
    }
    row_iterator<T>& operator--() {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator incremented.\nCore dumped.\n";
            abort();
        }
        if (_index == 0) {
            cerr<<"row iterator decremented below zero.\nCore dumped.\n";
            abort();
        }
        _index--;
        return *this;
    }
    row_iterator<T> operator--(int) {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator incremented.\nCore dumped.\n";
            abort();
        }
        if (_index == 0) {
            cerr<<"row iterator decremented below zero.\nCore dumped.\n";
            abort();
        }
        _index--;
        return row_iterator<T>(*_row, _index+1);
    }
    row_iterator<T>& operator+=(int n) {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator incremented.\nCore dumped.\n";
            abort();
        }
        _index += n;
        return *this;
    }
    int operator-(const row_iterator<T> &b) const {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator compared.\nCore dumped.\n";
            abort();
        }
        if (b._row == 0 || b._index<0) {
            cerr<<"Invalid row iterator compared.\nCore dumped.\n";
            abort();
        }
        if (_row != b._row) {
            cerr<<"Incomparable row iterators compared.\nCore dumped.\n";
            abort();
        }
        return b._index - _index;
    }
#else
    T& operator*()  {
        return (*_row)[_index];
    }
const T& operator*() const {
        return (*_row)[_index];
    }
    row_iterator<T>& operator++() {
        _index++;
        return *this;
    }
    row_iterator<T> operator++(int) {
        _index++;
        return row_iterator<T>(*_row, _index-1);
    }
    row_iterator<T>& operator--() {
        _index--;
        return *this;
    }
    row_iterator<T> operator--(int) {
        _index--;
        return row_iterator<T>(*_row, _index+1);
    }
    row_iterator<T>& operator+=(int n) {
        _index += n;
        return *this;
    }
    int operator-(const row_iterator<T> &b) const {
        return b._index - _index;
    }
#endif
    row_iterator<T>& operator-=(int n) {
        _index += (-n);
        return *this;
    }
protected:
    row<T> *_row;
    int     _index;
    friend class const_row_iterator<T>;
};

template <class T>
row_iterator<T> operator+(const row_iterator<T> &a, int n)
{
    row_iterator<T> tmp(a);
    return (tmp += n);
}

template <class T>
row_iterator<T> operator+(int n, const row_iterator<T> &a)
{
    row_iterator<T> tmp(a);
    return (tmp += n);
}

template <class T>
row_iterator<T> operator-(const row_iterator<T> &a, int n)
{
    row_iterator<T> tmp(a);
    return (tmp += (-n));
}

template <class T>
bool operator<(const row_iterator<T> &a, const row_iterator<T> &b)
{
    return (b-a > 0);
}

template<class T>
class const_row_iterator: public random_access_iterator<T, int> {
public:
    const_row_iterator(): _row(0), _index(-1) {}
    const_row_iterator(const row<T> &r,  int i):_row(&r), _index(i) {}
    const_row_iterator(const row_iterator<T> &ri)
        : _row(ri._row), _index(ri._index) {}
    bool operator==(const const_row_iterator<T> &other) const {
        return (_row == other._row) && (_index == other._index);
    }
#ifdef BOUNDCHECKS
    const T& operator*() const {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator dereferenced.\nCore dumped.\n";
            abort();
        }
        return (*_row)[_index];
    }
    const_row_iterator<T>& operator++() {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator incremented.\nCore dumped.\n";
            abort();
        }
        _index++;
        return *this;
    }
    const_row_iterator<T> operator++(int) {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator incremented.\nCore dumped.\n";
            abort();
        }
        _index++;
        return const_row_iterator<T>(*_row, _index-1);
    }
    const_row_iterator<T>& operator--() {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator incremented.\nCore dumped.\n";
            abort();
        }
        if (_index == 0) {
            cerr<<"row iterator decremented below zero.\nCore dumped.\n";
            abort();
        }
        _index--;
        return *this;
    }
    const_row_iterator<T> operator--(int) {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator incremented.\nCore dumped.\n";
            abort();
        }
        if (_index == 0) {
            cerr<<"row iterator decremented below zero.\nCore dumped.\n";
            abort();
        }
        _index--;
        return const_row_iterator<T>(*_row, _index+1);
    }
    const_row_iterator<T>& operator+=(int n) {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator incremented.\nCore dumped.\n";
            abort();
        }
        _index += n;
        return *this;
    }
    int operator-(const const_row_iterator<T> &b) const {
        if (_row == 0 || _index<0) {
            cerr<<"Invalid row iterator compared.\nCore dumped.\n";
            abort();
        }
        if (b._row == 0 || b._index<0) {
            cerr<<"Invalid row iterator compared.\nCore dumped.\n";
            abort();
        }
        if (_row != b._row) {
            cerr<<"Incomparable row iterators compared.\nCore dumped.\n";
            abort();
        }
        return b._index - _index;
    }
#else
    const T& operator*() const {
        return (*_row)[_index];
    }
    const_row_iterator<T>& operator++() {
        _index++;
        return *this;
    }
    const_row_iterator<T> operator++(int) {
        _index++;
        return const_row_iterator<T>(*_row, _index-1);
    }
    const_row_iterator<T>& operator--() {
        _index--;
        return *this;
    }
    const_row_iterator<T> operator--(int) {
        _index--;
        return const_row_iterator<T>(*_row, _index+1);
    }
    const_row_iterator<T>& operator+=(int n) {
        _index += n;
        return *this;
    }
    int operator-(const const_row_iterator<T> &b) const {
        return b._index - _index;
    }
#endif
    const_row_iterator<T>& operator-=(int n) {
        _index += (-n);
        return *this;
    }
protected:
    const row<T> *_row;
    int     _index;
};

template <class T>
const_row_iterator<T> operator+(const const_row_iterator<T> &a, int n)
{
    const_row_iterator<T> tmp(a);
    return (tmp += n);
}

template <class T>
const_row_iterator<T> operator+(int n, const const_row_iterator<T> &a)
{
    const_row_iterator<T> tmp(a);
    return (tmp += n);
}

template <class T>
const_row_iterator<T> operator-(const const_row_iterator<T> &a, int n)
{
    const_row_iterator<T> tmp(a);
    return (tmp += (-n));
}

template <class T>
bool operator<(const const_row_iterator<T> &a, const const_row_iterator<T> &b)
{
    return (b-a > 0);
}


#undef const_row_iterator
#undef row_iterator
#undef row

#include "row.c"

#endif
