#ifndef PLAGEO_CLEARANCE1_H
#define PLAGEO_CLEARANCE1_H

#include <PLAGEO/base.h>
#include <PLAGEO/onedim.h>


inline s_coord sqclearance(const pl_fixedvec &vt1, const pl_fixedvec & vt2)
{
    return (vt2-vt1).sqlength();
}

// checks the clearance between a fixedvec and a box
extern bool certain_clearance(const pl_fixedvec &, const pl_boundingbox &, s_coord);

// checks the square of the clearance between a fixedvec and a box
extern bool certain_sqclearance(const pl_fixedvec &, const pl_boundingbox &,
	    s_coord sqbound);

// checks the square of clearance between two boxes
extern bool certain_sqclearance(const pl_boundingbox &, const pl_boundingbox &,
	    s_coord sqbound);

// returns true if clearance between two boxes is certainly bigger than bound.
// if false is returned, the clearance may still be bigger
extern bool certain_clearance(const pl_boundingbox &, const pl_boundingbox &,
	    s_coord bound);

extern s_coord sqclearance(const pl_fixedvec &, const pl_line &);

extern s_coord sqclearance(const pl_fixedvec &, const pl_edge_ifc &);

// *****


// ******   VEC   ******

inline s_coord clearance(const pl_fixedvec &vt1, const pl_fixedvec & vt2)
{ return (vt2-vt1).length(); }
extern s_coord clearance(const pl_fixedvec &, const pl_fixedvec &, s_coord );

inline s_coord clearance(const pl_fixedvec &vt1, const pl_point & vt2)
{ return (vt2.vec()-vt1).length(); }
inline s_coord clearance(const pl_fixedvec &pt1, const pl_point &pt2, s_coord bound)
{ return clearance(pt1, pt2.vec(), bound); }

extern s_coord clearance(const pl_fixedvec &, const pl_edge_ifc &);
extern s_coord clearance(const pl_fixedvec &, const pl_edge_ifc &, s_coord bound);

extern s_coord clearance(const pl_fixedvec &, const pl_ray &);
extern s_coord clearance(const pl_fixedvec &, const pl_ray &, s_coord bound);

extern s_coord clearance(const pl_fixedvec &, const pl_line &);
extern s_coord clearance(const pl_fixedvec &, const pl_line &, s_coord bound);


extern s_coord clearance(const pl_fixedvec &, const pl_box &);
extern s_coord clearance(const pl_fixedvec &, const pl_box &, s_coord bound);


// ******   POINT   ******

inline s_coord clearance(const pl_point &vt1, const pl_fixedvec & vt2)
{ return (vt2-vt1.vec()).length(); }
inline s_coord clearance(const pl_point &pt1, const pl_fixedvec &pt2, s_coord bound)
{ return clearance(pt1.vec(), pt2, bound); }

inline s_coord clearance(const pl_point &vt1, const pl_point & vt2)
{ return (vt2.vec()-vt1.vec()).length(); }
inline s_coord clearance(const pl_point &pt1, const pl_point &pt2, s_coord bound)
{ return clearance(pt1.vec(), pt2.vec(), bound); }

inline s_coord clearance(const pl_point &pt, const pl_edge_ifc &edge)
{ return clearance(pt.vec(), edge); }
inline s_coord clearance(const pl_point &pt, const pl_edge_ifc &edge, s_coord bound)
{ return clearance(pt.vec(), edge, bound); }

inline s_coord clearance(const pl_point &pt, const pl_ray &ray)
{ return clearance(pt.vec(), ray); }
inline s_coord clearance(const pl_point &pt, const pl_ray &ray, s_coord bound)
{ return clearance(pt.vec(), ray, bound); }

inline s_coord clearance(const pl_point &pt, const pl_line &line)
{ return clearance(pt.vec(), line); }
inline s_coord clearance(const pl_point &pt, const pl_line &line, s_coord bound)
{ return clearance(pt.vec(), line, bound); }


inline s_coord clearance(const pl_point &pt, const pl_box &box)
{ return clearance(pt.vec(), box); }
inline s_coord clearance(const pl_point &pt, const pl_box &box, s_coord bound)
{ return clearance(pt.vec(), box, bound); }


// ******   EDGE   ******

inline s_coord clearance(const pl_edge_ifc &edge, const pl_fixedvec &vt)
{ return clearance(vt, edge); }
inline s_coord clearance(const pl_edge_ifc &edge, const pl_fixedvec &vt, s_coord bound)
{ return clearance(vt, edge, bound); }

inline s_coord clearance(const pl_edge_ifc &edge, const pl_point &vt)
{ return clearance(vt.vec(), edge); }
inline s_coord clearance(const pl_edge_ifc &edge, const pl_point &vt, s_coord bound)
{ return clearance(vt.vec(), edge, bound); }

extern s_coord clearance(const pl_edge_ifc &, const pl_edge_ifc &);
extern s_coord clearance(const pl_edge_ifc &, const pl_edge_ifc &, s_coord bound);

extern s_coord clearance(const pl_edge_ifc &, const pl_ray &);
extern s_coord clearance(const pl_edge_ifc &, const pl_ray &, s_coord bound);

extern s_coord clearance(const pl_edge_ifc &, const pl_line &);
extern s_coord clearance(const pl_edge_ifc &, const pl_line &, s_coord bound);


extern s_coord clearance(const pl_edge_ifc &, const pl_box &);
extern s_coord clearance(const pl_edge_ifc &, const pl_box &, s_coord bound);

// ******   RAY   ******

inline s_coord clearance(const pl_ray &ray, const pl_fixedvec &vt)
{ return clearance(vt, ray); }
inline s_coord clearance(const pl_ray &ray, const pl_fixedvec &vt, s_coord bound)
{ return clearance(vt, ray, bound); }

inline s_coord clearance(const pl_ray &ray, const pl_point &vt)
{ return clearance(vt.vec(), ray); }
inline s_coord clearance(const pl_ray &ray, const pl_point &vt, s_coord bound)
{ return clearance(vt.vec(), ray, bound); }

inline s_coord clearance(const pl_ray &r, const pl_edge_ifc &e)
{ return clearance(e,r); }
inline s_coord clearance(const pl_ray &r, const pl_edge_ifc &e, s_coord bound)
{ return clearance(e,r, bound); }

extern s_coord clearance(const pl_ray &, const pl_ray &);
extern s_coord clearance(const pl_ray &, const pl_ray &, s_coord bound);

extern s_coord clearance(const pl_ray &, const pl_line &);
extern s_coord clearance(const pl_ray &, const pl_line &, s_coord bound);


extern s_coord clearance(const pl_ray &, const pl_box &);
extern s_coord clearance(const pl_ray &, const pl_box &, s_coord bound);


// ******   LINE   ******

inline s_coord clearance(const pl_line &l, const pl_fixedvec &vt)
{ return clearance(vt, l);}
inline s_coord clearance(const pl_line &l, const pl_fixedvec &vt, s_coord bound)
{ return clearance(vt, l, bound);}

inline s_coord clearance(const pl_line &l, const pl_point &vt)
{ return clearance(vt.vec(), l);}
inline s_coord clearance(const pl_line &l, const pl_point &vt, s_coord bound)
{ return clearance(vt.vec(), l, bound);}

inline s_coord clearance(const pl_line &l, const pl_edge_ifc &e)
{ return clearance(e,l); }
inline s_coord clearance(const pl_line &l, const pl_edge_ifc &e, s_coord bound)
{ return clearance(e,l, bound); }

inline s_coord clearance(const pl_line &l, const pl_ray &r)
{ return clearance(r,l); }
inline s_coord clearance(const pl_line &l, const pl_ray &r, s_coord bound)
{ return clearance(r,l, bound); }


extern s_coord clearance(const pl_line &line, const pl_boundingbox &bb);
extern s_coord clearance(const pl_line &, const pl_boundingbox &, s_coord bound);



// ******   BOX   ******

inline s_coord clearance(const pl_box &box, const pl_fixedvec &vt)
{ return clearance(vt, box); }
inline s_coord clearance(const pl_box &box, const pl_fixedvec &vt, s_coord bound)
{ return clearance(vt, box, bound); }

inline s_coord clearance(const pl_box &box, const pl_point &vt)
{ return clearance(vt.vec(), box); }
inline s_coord clearance(const pl_box &box, const pl_point &vt, s_coord bound)
{ return clearance(vt.vec(), box, bound); }

inline s_coord clearance(const pl_box &bb, const pl_edge_ifc &edge)
{ return clearance(edge, bb); }
inline s_coord clearance(const pl_box &bb, const pl_edge_ifc &edge, s_coord bound)
{ return clearance(edge, bb, bound); }

inline s_coord clearance(const pl_box &bb, const pl_ray &ray)
{ return clearance(ray, bb); }
inline s_coord clearance(const pl_box &bb, const pl_ray &ray, s_coord bound)
{ return clearance(ray, bb, bound); }

inline s_coord clearance(const pl_box &bb, const pl_line &line)
{ return clearance(line, bb); }
inline s_coord clearance(const pl_box &bb, const pl_line &line, s_coord bound)
{ return clearance(line, bb, bound); }

extern s_coord clearance(const pl_box &, const pl_box &);
extern s_coord clearance(const pl_box &, const pl_box &, s_coord bound);


#endif // PLAGEO_CLEARANCE2_H
