#ifndef PLAGEO_CLEARANCE2_H
#define PLAGEO_CLEARANCE2_H

#include <PLAGEO/base.h>
#include <PLAGEO/onedim.h>
#include <PLAGEO/curved.h>
#include <PLAGEO/pgn_ifc.h>
#include <PLAGEO/intervals.h>


// ******   VEC   ******

extern s_coord clearance(const pl_fixedvec &, const pl_arc &);
extern s_coord clearance(const pl_fixedvec &, const pl_arc &, s_coord bound);

extern s_coord clearance(const pl_fixedvec &, const pl_pgn_ifc &);
extern s_coord clearance(const pl_fixedvec &, const pl_pgn_ifc &, s_coord bound);

extern s_coord clearance(const pl_fixedvec &, const pl_disc &);
extern s_coord clearance(const pl_fixedvec &, const pl_disc &, s_coord bound);

extern s_coord clearance(const pl_fixedvec &, const pl_object &);
extern s_coord clearance(const pl_fixedvec &, const pl_object &, s_coord bound);

// ******   POINT   ******

inline s_coord clearance(const pl_point &pt, const pl_arc &arc)
{ return clearance(pt.vec(), arc); }
inline s_coord clearance(const pl_point &pt, const pl_arc &arc, s_coord bound)
{ return clearance(pt.vec(), arc, bound); }

inline s_coord clearance(const pl_point &pt, const pl_pgn_ifc &pgn)
{ return clearance(pt.vec(), pgn); }
inline s_coord clearance(const pl_point &pt, const pl_pgn_ifc &pgn, s_coord bound)
{ return clearance(pt.vec(), pgn, bound); }

inline s_coord clearance(const pl_point &pt, const pl_disc &disc)
{ return clearance(pt.vec(), disc); }
inline s_coord clearance(const pl_point &pt, const pl_disc &disc, s_coord bound)
{ return clearance(pt.vec(), disc, bound); }

inline s_coord clearance(const pl_point &pt, const pl_object &obj)
{ return clearance(pt.vec(), obj); }
inline s_coord clearance(const pl_point &pt, const pl_object &obj, s_coord bound)
{ return clearance(pt.vec(), obj, bound); }

// ******   EDGE   ******


extern s_coord clearance(const pl_edge_ifc &, const pl_arc &);
extern s_coord clearance(const pl_edge_ifc &, const pl_arc &, s_coord bound);

extern s_coord clearance(const pl_edge_ifc &, const pl_pgn_ifc &);
extern s_coord clearance(const pl_edge_ifc &, const pl_pgn_ifc &, s_coord bound);

extern s_coord clearance(const pl_edge_ifc &, const pl_disc &);
extern s_coord clearance(const pl_edge_ifc &, const pl_disc &, s_coord bound);

extern s_coord clearance(const pl_edge_ifc &, const pl_object &);
extern s_coord clearance(const pl_edge_ifc &, const pl_object &, s_coord bound);

// ******   RAY   ******

extern s_coord clearance(const pl_ray &, const pl_arc &);
extern s_coord clearance(const pl_ray &, const pl_arc &, s_coord bound);

extern s_coord clearance(const pl_ray &, const pl_pgn_ifc &);
extern s_coord clearance(const pl_ray &, const pl_pgn_ifc &, s_coord bound);

extern s_coord clearance(const pl_ray &, const pl_disc &);
extern s_coord clearance(const pl_ray &, const pl_disc &, s_coord bound);

extern s_coord clearance(const pl_ray &, const pl_object &);
extern s_coord clearance(const pl_ray &, const pl_object &, s_coord bound);

// ******   LINE   ******

extern s_coord clearance(const pl_line &, const pl_arc &);
extern s_coord clearance(const pl_line &, const pl_arc &, s_coord bound);

extern s_coord clearance(const pl_line &, const pl_pgn_ifc &);
extern s_coord clearance(const pl_line &, const pl_pgn_ifc &, s_coord bound);

extern s_coord clearance(const pl_line &, const pl_disc &);
extern s_coord clearance(const pl_line &, const pl_disc &, s_coord bound);

extern s_coord clearance(const pl_line &, const pl_object &);
extern s_coord clearance(const pl_line &, const pl_object &, s_coord bound);


// ******   ARC   ******

inline s_coord clearance(const pl_arc &c, const pl_fixedvec &vt)
{ return clearance(vt, c); }
inline s_coord clearance(const pl_arc &c, const pl_fixedvec &vt, s_coord bound)
{ return clearance(vt, c, bound); }
inline s_coord clearance(const pl_arc &c, const pl_point &vt)
{ return clearance(vt.vec(), c); }
inline s_coord clearance(const pl_arc &c, const pl_point &vt, s_coord bound)
{ return clearance(vt.vec(), c, bound); }

inline s_coord clearance(const pl_arc &c, const pl_edge_ifc &edge)
{ return clearance(edge, c); }
inline s_coord clearance(const pl_arc &c, const pl_edge_ifc &edge, s_coord bound)
{ return clearance(edge, c, bound); }

inline s_coord clearance(const pl_arc &c, const pl_ray &ray)
{ return clearance(ray, c); }
inline s_coord clearance(const pl_arc &c, const pl_ray &ray, s_coord bound)
{ return clearance(ray, c, bound); }

inline s_coord clearance(const pl_arc &c, const pl_line &line)
{ return clearance(line, c); }
inline s_coord clearance(const pl_arc &c, const pl_line &line, s_coord bound)
{ return clearance(line, c, bound); }

extern s_coord clearance(const pl_arc &, const pl_arc &);
extern s_coord clearance(const pl_arc &, const pl_arc &, s_coord bound);


extern s_coord clearance(const pl_arc &, const pl_box &);
extern s_coord clearance(const pl_arc &, const pl_box &, s_coord bound);

extern s_coord clearance(const pl_arc &, const pl_pgn_ifc &);
extern s_coord clearance(const pl_arc &, const pl_pgn_ifc &, s_coord bound);

extern s_coord clearance(const pl_arc &, const pl_disc &);
extern s_coord clearance(const pl_arc &, const pl_disc &, s_coord bound);

extern s_coord open_clearance(const pl_arc &, const pl_disc &);
extern s_coord open_clearance(const pl_arc &, const pl_disc &, s_coord bound);

extern s_coord clearance(const pl_arc &, const pl_object &);
extern s_coord clearance(const pl_arc &, const pl_object &, s_coord bound);


// ******   BOX   ******

inline s_coord clearance(const pl_box &bb, const pl_arc &arc)
{ return clearance(arc, bb); }
inline s_coord clearance(const pl_box &bb, const pl_arc &arc, s_coord bound)
{ return clearance(arc, bb, bound); }

extern s_coord clearance(const pl_box &, const pl_pgn_ifc &);
extern s_coord clearance(const pl_box &, const pl_pgn_ifc &, s_coord bound);

extern s_coord clearance(const pl_box &, const pl_disc &);
extern s_coord clearance(const pl_box &, const pl_disc &, s_coord bound);

extern s_coord clearance(const pl_box &, const pl_object &);
extern s_coord clearance(const pl_box &, const pl_object &, s_coord bound);


// ******   POLYGON   ******


inline s_coord clearance(const pl_pgn_ifc &pgn, const pl_fixedvec &vt)
{return clearance(vt, pgn); }
inline s_coord clearance(const pl_pgn_ifc &pgn, const pl_fixedvec &vt, s_coord bound)
{return clearance(vt, pgn, bound); }

inline s_coord clearance(const pl_pgn_ifc &pgn, const pl_point &vt)
{return clearance(vt.vec(), pgn); }
inline s_coord clearance(const pl_pgn_ifc &pgn, const pl_point &vt, s_coord bound)
{return clearance(vt.vec(), pgn, bound); }

inline s_coord clearance(const pl_pgn_ifc &pgn, const pl_edge_ifc &edge)
{ return clearance(edge, pgn); }
inline s_coord clearance(const pl_pgn_ifc &pgn, const pl_edge_ifc &edge, s_coord bound)
{ return clearance(edge, pgn, bound); }

inline s_coord clearance(const pl_pgn_ifc &pgn, const pl_ray &ray)
{ return clearance(ray, pgn); }
inline s_coord clearance(const pl_pgn_ifc &pgn, const pl_ray &ray, s_coord bound)
{ return clearance(ray, pgn, bound); }

inline s_coord clearance(const pl_pgn_ifc &pgn, const pl_line &line)
{ return clearance(line, pgn); }
inline s_coord clearance(const pl_pgn_ifc &pgn, const pl_line &line, s_coord bound)
{ return clearance(line, pgn, bound); }

inline s_coord clearance(const pl_pgn_ifc &pgn, const pl_arc &arc)
{ return clearance(arc, pgn); }
inline s_coord clearance(const pl_pgn_ifc &pgn, const pl_arc &arc, s_coord bound)
{ return clearance(arc, pgn, bound); }


inline s_coord clearance(const pl_pgn_ifc &pgn, const pl_box &box)
{ return clearance(box, pgn); }
inline s_coord clearance(const pl_pgn_ifc &pgn, const pl_box &box, s_coord bound)
{ return clearance(box, pgn, bound); }

extern s_coord clearance(const pl_pgn_ifc &, const pl_pgn_ifc &);
extern s_coord clearance(const pl_pgn_ifc &, const pl_pgn_ifc &, s_coord bound);

extern s_coord open_clearance(const pl_pgn_ifc &pgn1, const pl_pgn_ifc &pgn2);
extern s_coord open_clearance(const pl_pgn_ifc &pgn1, const pl_pgn_ifc &pgn2, s_coord bound);

extern s_coord clearance(const pl_pgn_ifc &, const pl_disc &);
extern s_coord clearance(const pl_pgn_ifc &, const pl_disc &, s_coord bound);

extern s_coord clearance(const pl_pgn_ifc &, const pl_object &);
extern s_coord clearance(const pl_pgn_ifc &, const pl_object &, s_coord bound);


// ******   DISC   ******

inline s_coord clearance(const pl_disc &c, const pl_fixedvec &vt)
{ return clearance(vt, c); }
inline s_coord clearance(const pl_disc &c, const pl_fixedvec &vt, s_coord bound)
{ return clearance(vt, c, bound); }

inline s_coord clearance(const pl_disc &c, const pl_point &vt)
{ return clearance(vt.vec(), c); }
inline s_coord clearance(const pl_disc &c, const pl_point &vt, s_coord bound)
{ return clearance(vt.vec(), c, bound); }

inline s_coord clearance(const pl_disc &c, const pl_edge_ifc &edge)
{ return clearance(edge, c); }
inline s_coord clearance(const pl_disc &c, const pl_edge_ifc &edge, s_coord bound)
{ return clearance(edge, c, bound); }

inline s_coord clearance(const pl_disc &c, const pl_ray &ray)
{ return clearance(ray, c); }
inline s_coord clearance(const pl_disc &c, const pl_ray &ray, s_coord bound)
{ return clearance(ray, c, bound); }

inline s_coord clearance(const pl_disc &c, const pl_line &line)
{ return clearance(line, c); }
inline s_coord clearance(const pl_disc &c, const pl_line &line, s_coord bound)
{ return clearance(line, c, bound); }

inline s_coord clearance(const pl_disc &disc, const pl_arc &arc)
{ return clearance(arc, disc); }
inline s_coord clearance(const pl_disc &disc, const pl_arc &arc, s_coord bound)
{ return clearance(arc, disc, bound); }

inline s_coord open_clearance(const pl_disc &disc, const pl_arc &arc)
{ return open_clearance(arc, disc); }
inline s_coord open_clearance(const pl_disc &disc, const pl_arc &arc, s_coord bound)
{ return open_clearance(arc, disc, bound); }

inline s_coord clearance(const pl_disc &c, const pl_box &box)
{ return clearance(box, c); }
inline s_coord clearance(const pl_disc &c, const pl_box &box, s_coord bound)
{ return clearance(box, c, bound); }

inline s_coord clearance(const pl_disc &c, const pl_pgn_ifc &pgn)
{ return clearance(pgn, c); }
inline s_coord clearance(const pl_disc &c, const pl_pgn_ifc &pgn, s_coord bound)
{ return clearance(pgn, c, bound); }

extern s_coord clearance(const pl_disc &, const pl_disc &);
extern s_coord clearance(const pl_disc &, const pl_disc &, s_coord bound);

extern s_coord clearance(const pl_disc &, const pl_object &);
extern s_coord clearance(const pl_disc &, const pl_object &, s_coord bound);


// ******   OBJECT   ******


inline s_coord clearance(const pl_object &c, const pl_fixedvec &vt)
{ return clearance(vt, c); }
inline s_coord clearance(const pl_object &c, const pl_fixedvec &vt, s_coord bound)
{ return clearance(vt, c, bound); }

inline s_coord clearance(const pl_object &c, const pl_point &vt)
{ return clearance(vt.vec(), c); }
inline s_coord clearance(const pl_object &c, const pl_point &vt, s_coord bound)
{ return clearance(vt.vec(), c, bound); }

inline s_coord clearance(const pl_object &c, const pl_edge_ifc &edge)
{ return clearance(edge, c); }
inline s_coord clearance(const pl_object &c, const pl_edge_ifc &edge, s_coord bound)
{ return clearance(edge, c, bound); }

inline s_coord clearance(const pl_object &c, const pl_ray &ray)
{ return clearance(ray, c); }
inline s_coord clearance(const pl_object &c, const pl_ray &ray, s_coord bound)
{ return clearance(ray, c, bound); }

inline s_coord clearance(const pl_object &c, const pl_line &line)
{ return clearance(line, c); }
inline s_coord clearance(const pl_object &c, const pl_line &line, s_coord bound)
{ return clearance(line, c, bound); }

inline s_coord clearance(const pl_object &disc, const pl_arc &arc)
{ return clearance(arc, disc); }
inline s_coord clearance(const pl_object &disc, const pl_arc &arc, s_coord bound)
{ return clearance(arc, disc, bound); }

inline s_coord clearance(const pl_object &c, const pl_box &box)
{ return clearance(box, c); }
inline s_coord clearance(const pl_object &c, const pl_box &box, s_coord bound)
{ return clearance(box, c, bound); }

inline s_coord clearance(const pl_object &c, const pl_pgn_ifc &pgn)
{ return clearance(pgn, c); }
inline s_coord clearance(const pl_object &c, const pl_pgn_ifc &pgn, s_coord bound)
{ return clearance(pgn, c, bound); }

inline s_coord clearance(const pl_object &c, const pl_disc &disc)
{ return clearance(disc, c); }
inline s_coord clearance(const pl_object &c, const pl_disc &disc, s_coord bound)
{ return clearance(disc, c, bound); }

extern s_coord clearance(const pl_object &, const pl_object &);
extern s_coord clearance(const pl_object &, const pl_object &, s_coord bound);

// ******   OTHERS   ******

extern s_coord clearance(const pl_dotted_line &,
			    const pl_dotted_line &);

extern s_coord sqclearance(const pl_pgn_ifc &, const pl_pgn_ifc &, s_coord sqbound);


#endif // PLAGEO_CLEARANCE2_H
