#ifndef PLAGEO_CROSS2_H
#define PLAGEO_CROSS2_H

#include <PLAGEO/base.h>
#include <PLAGEO/onedim.h>
#include <PLAGEO/curved.h>
#include <PLAGEO/pgn_ifc.h>
#include <PLAGEO/intersect.h>

// ******   OBJECT *****

extern bool do_cross(const pl_object &, const pl_fixedvec &);

extern bool do_cross(const pl_object &, const pl_point &);

extern bool do_cross(const pl_object &, const pl_line &);

extern bool do_cross(const pl_object &, const pl_ray &);

extern bool do_cross(const pl_object &, const pl_edge_ifc &);

extern bool do_cross(const pl_object &, const pl_arc &);

extern bool do_cross(const pl_object &, const pl_box &);

extern bool do_cross(const pl_object &, const pl_pgn_ifc &);

extern bool do_cross(const pl_object &, const pl_disc &);
extern bool do_cross(const pl_object &, const pl_object &);


inline bool do_cross(const pl_fixedvec &vec, const pl_object &obj)
{ return do_cross(obj, vec); }


inline bool do_cross(const pl_point &vt, const pl_object &obj)
{ return do_cross(obj, vt.vec()); }

inline bool do_intersect(const pl_edge_ifc &edge, const pl_arc &arc)
{
    pl_vec dummy1, dummy2;
    return intersect(dummy1, dummy2, arc, edge);
}

inline bool do_cross(const pl_edge_ifc &edge, const pl_arc &arc)
{ return do_intersect(edge, arc); }

extern bool do_cross(const pl_edge_ifc &, const pl_pgn_ifc &);
extern bool do_cross(const pl_edge_ifc &, const pl_disc &);



extern bool do_intersect(const pl_ray &ray, const pl_arc &arc);

inline bool do_cross(const pl_ray &ray, const pl_arc &arc)
{ return do_intersect(ray, arc); }

extern bool do_cross(const pl_ray &, const pl_pgn_ifc &);
extern bool do_cross(const pl_ray &, const pl_disc &);


extern bool do_intersect(const pl_line &line, const pl_arc &arc);

inline bool do_cross(const pl_line &line, const pl_arc &arc)
{ return do_intersect(line, arc); }

extern bool do_cross(const pl_line &, const pl_pgn_ifc &);

extern bool do_cross(const pl_line &, const pl_disc &);


// ******   ARC   ******

inline bool do_intersect(const pl_arc &arc, const pl_edge_ifc &edge)
{
    pl_vec dummy1, dummy2;
    return intersect(dummy1, dummy2, arc, edge);
}

inline bool do_intersect(const pl_arc &arc, const pl_ray &ray)
{ return do_intersect(ray, arc); }

inline bool do_intersect(const pl_arc &arc, const pl_line &line)
{ return do_intersect(line, arc); }

extern bool do_intersect(const pl_arc &, const pl_arc &);


inline bool do_cross(const pl_arc &, const pl_fixedvec &)
{ return false; }

inline bool do_cross(const pl_arc &, const pl_point &)
{ return false; }

inline bool do_cross(const pl_arc &arc, const pl_edge_ifc &edge)
{ return do_intersect(arc, edge); }

inline bool do_cross(const pl_arc &arc1, const pl_ray &ray)
{ return do_intersect(arc1, ray); }

inline bool do_cross(const pl_arc &arc, const pl_line &line)
{ return do_intersect(line, arc); }

inline bool do_cross(const pl_arc &arc1, const pl_arc &arc)
{ return do_intersect(arc1, arc); }

extern bool do_cross(const pl_arc &, const pl_box &);

extern bool do_cross(const pl_arc &, const pl_pgn_ifc &);

extern bool do_cross(const pl_arc &arc, const pl_disc &disc);



inline bool do_cross(const pl_box &box, const pl_arc &arc)
{ return do_cross(arc, box); }

extern bool do_cross(const pl_box &, const pl_pgn_ifc &);

extern bool do_cross(const pl_box &, const pl_disc &);


// ******   POLYGON   ******

inline bool do_cross(const pl_pgn_ifc &, const pl_fixedvec &)
{ return false;}

inline bool do_cross(const pl_pgn_ifc &, const pl_point &)
{ return false;}

inline bool do_cross( const pl_pgn_ifc &pgn, const pl_edge_ifc &edge)
{ return do_cross(edge, pgn); }

inline bool do_cross(const pl_pgn_ifc &pgn, const pl_ray &ray)
{ return do_cross(ray, pgn); }

inline bool do_cross(const pl_pgn_ifc &pgn, const pl_line &line)
{ return do_cross(line, pgn); }

inline bool do_cross(const pl_pgn_ifc &pgn, const pl_arc &arc)
{ return do_cross(arc, pgn); }

inline bool do_cross(const pl_pgn_ifc &pgn, const pl_box &box)
{ return do_cross(box, pgn); }

extern bool do_cross(const pl_pgn_ifc &, const pl_pgn_ifc &);

extern bool do_cross(const pl_pgn_ifc &, const pl_disc &);


// ******   DISC   ******

inline bool do_cross(const pl_disc &, const pl_fixedvec &)
{ return false;}

inline bool do_cross(const pl_disc &, const pl_point &)
{ return false;}

inline bool do_cross( const pl_disc &disc, const pl_edge_ifc &edge)
{ return do_cross(edge, disc); }

inline bool do_cross(const pl_disc &disc, const pl_ray &ray)
{ return do_cross(ray, disc); }

inline bool do_cross(const pl_disc &disc, const pl_line &line)
{ return do_cross(line, disc); }

inline bool do_cross(const pl_disc &disc, const pl_arc &arc)
{ return do_cross(arc, disc); }

inline bool do_cross(const pl_disc &disc, const pl_box &box)
{ return do_cross(box, disc); }

inline bool do_cross(const pl_disc &disc, const pl_pgn_ifc &pgn)
{ return do_cross(pgn, disc); }
extern bool do_cross(const pl_disc &, const pl_disc &);
#endif	// PLAGEO_CROSS2_H
