#ifndef PLAGEO_CURVED_H
#define PLAGEO_CURVED_H

#include <PLAGEO/base.h>

class pl_disc: public pl_object {
public:
		    pl_disc(const pl_fixedvec &vt, s_coord radius) {
					_center = vt;
					// assert (radius >= 0)
					_radius = radius; }
		    pl_disc()				{}
		    ~pl_disc()	;
    pl_object_type  type() const;
    pl_box	    bbox() const;
    pl_vec	    center() const			{ return _center; }
    s_coord	    radius() const			{ return _radius; }
    s_coord	    area() const		;
    void	    set_center(const pl_fixedvec &vt)	{ _center = vt; }
    void	    set_radius(s_coord radius)		{ _radius = radius; }
    void	    transform(const pl_transform &tfm);
    void	    translate(const pl_fixedvec &disp) ;
    void	    scale(s_coord fac);
    void	    outer_approximation_d(pl_vecreray &result, s_coord epsilon) const;
    void	    outer_approximation_n(pl_vecreray &result, int n) const;
    void	    inner_approximation_d(pl_vecreray &result, s_coord epsilon) const;
    void	    inner_approximation_n(pl_vecreray &result, int n) const;
protected:
    pl_vec	    _center;
    s_coord	    _radius;
};

class pl_arc: public pl_object {
public:
		    pl_arc(const pl_fixedvec &vt, s_coord radius, pl_angle startangle, 
			pl_angle enclosed_angle) ;
		    pl_arc(const pl_disc &c, pl_angle angle, pl_angle diffangle) ;
		    pl_arc()			{}
		    ~pl_arc();
    pl_object_type  type() const;
    pl_box	    bbox() const ;
    pl_vec	    center() const		    { return _disc.center(); }
    s_coord	    radius() const		    { return _disc.radius(); }
    pl_vec	    begin_vt() const;
    pl_vec	    end_vt() const;
    pl_vec	    small_vt() const;
    pl_vec	    big_vt() const;
    pl_angle	    begin_angle() const			{ return _startangle; }
    pl_angle	    end_angle() const		{ return _startangle + _angle;}
    pl_angle	    enclosed_angle() const		    { return _angle; }
    pl_angle	    small_angle() const;
    pl_angle	    big_angle() const;
    s_coord	    length() const	    {return fabs_pp(radius()*_angle);}
const pl_disc&	    supporting_disc() const	    { return _disc; }
    void	    set_center(const pl_fixedvec &vt) { _disc.set_center(vt); }
    void	    set_radius(s_coord radius)	  { _disc.set_radius(radius); }
    void	    set_angles(pl_angle angle, pl_angle diffangle) ;
    void	    transform(const pl_transform &tfm);
    void	    translate(const pl_fixedvec &disp) ;
    void	    scale(s_coord fac);
    void	    outer_approximation_d(pl_vecreray &result, s_coord epsilon) const;
    void	    outer_approximation_n(pl_vecreray &result, int n) const;
    void	    inner_approximation_d(pl_vecreray &result, s_coord epsilon) const;
    void	    inner_approximation_n(pl_vecreray &result, int n) const;
protected:
    pl_disc	    _disc;
    pl_angle	    _startangle;
    pl_angle	    _angle;
    pl_box	    _bbox;
    void	    make_bbox();
};

// *********** Axis Centered Ellipse : for internal use
class pl_ACE {
public:
		pl_ACE() {}
		pl_ACE(s_coord x, s_coord y)	{_x=x; _y=y;}
    s_coord	x() const			{return _x;}
    s_coord	y() const			{return _y;}
protected:
    s_coord _x, _y;    
};

extern s_coord wn_clearance(pl_vec &wn, const pl_fixedvec &, const pl_ACE &);
extern s_coord wn_clearance(pl_vec &wn, const pl_fixedvec &,
	const pl_ACE &, s_coord bound);
extern s_coord wn_open_clearance(pl_vec &wn,const pl_fixedvec &,const pl_ACE&);
extern s_coord wn_open_clearance(pl_vec &wn, const pl_fixedvec &,
	const pl_ACE &, s_coord bound);
extern s_coord open_clearance(const pl_fixedvec &, const pl_ACE &, s_coord);
extern s_coord clearance(const pl_fixedvec &, const pl_ACE &, s_coord);


#endif // PLAGEO_CURVED_H
