#ifndef PLAGEO_DRAWAREA_H
#define PLAGEO_DRAWAREA_H

#include <PLAGEO/base.h>
#include <PLAGEO/onedim.h>
#include <PLAGEO/curved.h>
#include <PLAGEO/pgn_ifc.h>

const int PL_COLORCOUNT = 256;

extern void pl_color_components(int &red, int &green, int &blue, int col);

class pl_drawarea {
public:
    enum vertexshapes {DOT=0, CROSS=1, SQUARE=2, BOX=3};
    enum areafills {NOFILL=0, FILL=1, OUTLINE=2, FILLOUT=3};
    virtual	    ~pl_drawarea();
    virtual void    pageready()=0;
    virtual void    clear()=0;
    virtual void    draw(const pl_fixedvec &)=0;
    void	    draw(const pl_point &);
    virtual void    draw(const pl_edge_ifc &)=0;
    void	    draw(const pl_ray &);
    void	    draw(const pl_line &);
    virtual void    draw(const pl_arc &)=0;
    void	    draw(const pl_box &);
    virtual void    draw(const pl_pgn_ifc &)=0;
    virtual void    draw(const pl_disc &)=0;
    void	    draw(const pl_object &);
    virtual void    draw_text(const pl_fixedvec&, const char*);
    void	    set_viewingarea(float x, float y, float xsize, float ysize);
    void	    set_viewingarea(float x, float y, float size);
    void	    get_viewingarea(float &x, float &y, float &xsize, float &ysize);
    int		    set_backcol(int c);
    int		    set_textcol(int c);
    int		    set_linecol(int c);
    int		    set_areacol(int c);
    int		    set_areafill(int);
    int		    set_vertexshape(int);
    float	    set_vertexsize(float);
    float	    set_textsize(float);
protected:
    int	    _backcol;
    int	    _linecol;
    int	    _areacol;
    int	    _textcol;
    int	    _areafill;
    int	    _vertexshape;
    float   _vertexsize;
    float   _viewing_x;
    float   _viewing_y;
    float   _viewing_xsize;
    float   _viewing_ysize;
    int	    _viewing_changed;
    float   _textsize;
    void    init_values();
};

#endif // PLAGEO_DRAWAREA_H
