#ifndef PLAGEO_INTERSECT_H
#define PLAGEO_INTERSECT_H

#include <PLAGEO/base.h>
#include <PLAGEO/onedim.h>
#include <PLAGEO/curved.h>
#include <PLAGEO/pgn_ifc.h>
#include <PLAGEO/intervals.h>

//  ******** ROUTINES DELIVERING A WITNESS OF AN INTERSECTION ********

// witnesses of intersections between one dimensional objects
// edge, ray, line, arc and disc (taken as circle)

extern bool intersect(pl_vec &, const pl_line &, const pl_line &);

extern bool intersect(pl_vec &, const pl_ray &, const pl_line &);

inline bool intersect(pl_vec &wn, const pl_line &l, const pl_ray &r)
{ return intersect(wn, r, l); }

extern bool intersect(pl_vec &, const pl_edge_ifc &, const pl_line &);

inline bool intersect(pl_vec &wn, const pl_line &l, const pl_edge_ifc &e)
{ return intersect(wn, e, l); }

extern bool intersect(pl_vec &, const pl_ray&, const pl_ray&);

extern bool intersect(pl_vec & ,const pl_edge_ifc &, const pl_ray&);

inline bool intersect(pl_vec &wn, const pl_ray &r, const pl_edge_ifc &e)
{ return intersect(wn, e, r); }

extern bool intersect(pl_vec &, const pl_edge_ifc &, const pl_edge_ifc &);

extern intervals intersect(const pl_pgn_ifc &, const pl_line &);

inline intervals intersect(const pl_line &l, const pl_pgn_ifc &pgn)
{ return intersect(pgn,  l); }

extern intervals intersect(const pl_pgn_ifc &, const pl_ray &);

inline intervals intersect(const pl_ray &ray, const pl_pgn_ifc &pgn)
{ return intersect(pgn,  ray); }

extern bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_disc &disc, const pl_edge_ifc &edge);

inline bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_edge_ifc &edge, const pl_disc &disc)
{ return intersect(wn1, wn2, disc, edge); }

extern bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_disc &disc, const pl_ray &ray);

inline bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_ray &ray, const pl_disc &disc)
{ return intersect(wn1, wn2, disc, ray); }

extern bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_disc &disc, const pl_line &line);

inline bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_line &line, const pl_disc &disc)
{ return intersect(wn1, wn2, disc, line); }


extern bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_edge_ifc &edge, const pl_arc &arc);

inline bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_arc &arc, const pl_edge_ifc &edge)
{ return intersect(wn1, wn2, edge, arc); }

extern bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_ray &ray, const pl_arc &arc);

inline bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_arc &arc, const pl_ray &ray)
{ return intersect(wn1, wn2, ray, arc); }

extern bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_line &line, const pl_arc &arc);

inline bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_arc &arc, const pl_line &line)
{ return intersect(wn1, wn2, line, arc); }


extern bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_disc &disc1, const pl_disc &disc2);

extern bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_disc &disc, const pl_arc &arc);

inline bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_arc &arc, const pl_disc &disc)
{ return intersect(wn1, wn2, disc, arc); }

extern bool intersect(pl_vec &wn1, pl_vec &wn2, 
		const pl_arc &, const pl_arc &);









#endif	// PLAGEO_INTERSECT_H
