#ifndef PLAGEO_ONEDIM_H
#define PLAGEO_ONEDIM_H

#include <PLAGEO/base.h>


/*******************************
 *
 * one dimensional types (classes) for planar geometry
 *
 * -lines
 * -rays
 * -edges
 *
 *******************************/





class pl_line: public pl_object {
public:
		    pl_line()		{}
		    pl_line(const pl_fixedvec &v1, const pl_fixedvec &v2)
				    : _vt(v1), _d(normalized(v2-v1)) {}
		    pl_line(const pl_fixedvec &vt, const pl_unitvec &uv)
					    : _vt(vt), _d(uv) {}
		    ~pl_line()	;
    pl_object_type  type() const;
    pl_box	    bbox() const ;
const pl_vec &	    ref_vt() const		{ return _vt; }
const pl_unitvec &  direction() const	{ return _d; }
    pl_unitvec	    normal() const		{ return _d.normal(); }
    s_coord	    placeproject(const pl_fixedvec &vec) const ;
    s_coord	    dirproject(const pl_fixedvec &vec) const ;
    pl_vec	    placevec(s_coord co) const	{return _vt + co*_d;}
    pl_vec	    dirvec(s_coord co) const		{return co*_d;}
    void	    transform(const pl_transform &tfm) ;
    void	    translate(const pl_fixedvec &disp) ;
    void	    scale(s_coord fac);
protected:
    pl_vec	    _vt; 
    pl_unitvec	    _d; 
};


class pl_ray: public pl_object {
public:
		    pl_ray()                {}
		    pl_ray(const pl_fixedvec &vt, const pl_unitvec &vec)
			    : _begin_vt(vt), _d(vec) {}
		    ~pl_ray() ;
    pl_object_type  type() const;
    pl_box	    bbox() const ;
const pl_vec &	    begin_vt() const			    {return _begin_vt;}
const pl_unitvec &  direction() const				{ return _d;}
    pl_line	    supporting_line() const { return pl_line(_begin_vt, _d); }
    void	    transform(const pl_transform &tfm) ;
    void	    translate(const pl_fixedvec &disp) ;
    void	    scale(s_coord fac);
protected:
    pl_vec	    _begin_vt;
    pl_unitvec	    _d;      // direction
};


class pl_edge_ifc: public pl_object {
public:
    pl_box	    bbox() const;
    pl_object_type  type() const;
virtual void	    operator=(const pl_edge_ifc &) = 0;
virtual void	    make(const pl_fixedvec &v1, const pl_fixedvec &v2) = 0;
virtual void	    make(const pl_fixedvec &, const pl_unitvec &, s_coord l) = 0;
virtual const pl_vec &begin_vt() const = 0;
virtual const pl_vec &end_vt() const = 0;
virtual pl_line	    supporting_line() const = 0;
virtual s_coord	    length() const = 0;
virtual pl_unitvec  direction() const = 0;
};

class pl_edge_imp1: public pl_edge_ifc {
public:
		    pl_edge_imp1()			{}
		    pl_edge_imp1(const pl_fixedvec &v1, const pl_fixedvec &v2)
			    : _begin_vt(v1), _end_vt(v2)	{}
		    pl_edge_imp1(const pl_edge_ifc &e)
			: _begin_vt(e.begin_vt()), _end_vt(e.end_vt()) {}
		    ~pl_edge_imp1() ;
    void	    operator=(const pl_edge_ifc &);
    void	    operator=(const pl_edge_imp1 &);
    void	    make(const pl_fixedvec &v1, const pl_fixedvec &v2);
    void	    make(const pl_fixedvec &, const pl_unitvec &, s_coord l);
const pl_vec &	    begin_vt() const ;
const pl_vec &	    end_vt() const ;
    pl_line	    supporting_line() const;
    s_coord	    length() const;
    pl_unitvec	    direction() const;
    void	    transform(const pl_transform &tfm) ;
    void	    translate(const pl_fixedvec &disp) ;
    void	    scale(s_coord fac);
protected:
    pl_vec	    _begin_vt, _end_vt;
};

class pl_edge_imp2: public pl_edge_ifc {
public:
		    pl_edge_imp2()			{}
		    pl_edge_imp2(const pl_fixedvec &v1, const pl_fixedvec &v2) ;
		    pl_edge_imp2(const pl_fixedvec &, const pl_unitvec &, s_coord l);
		    pl_edge_imp2(const pl_edge_ifc &) ;
		    ~pl_edge_imp2() ;
    void	    operator=(const pl_edge_ifc &);
    void	    operator=(const pl_edge_imp2 &);
    void	    make(const pl_fixedvec &v1, const pl_fixedvec &v2);
    void	    make(const pl_fixedvec &, const pl_unitvec &, s_coord l);
const pl_vec &	    begin_vt() const ;
const pl_vec &	    end_vt() const ;
    pl_line	    supporting_line() const;
    s_coord	    length() const;
    pl_unitvec	    direction() const;
const pl_unitvec &  direction_by_ref() const {return _line.direction();}
    void	    transform(const pl_transform &tfm) ;
    void	    translate(const pl_fixedvec &disp) ;
    void	    scale(s_coord fac);
protected:
    pl_line	    _line;
    s_coord	    _length;
    pl_vec	    _end_vt;
};

typedef pl_edge_imp1 pl_edge;

extern pl_vec placeproject(const pl_fixedvec &vt, const pl_line &l);
extern pl_vec dirproject(const pl_fixedvec &vt, const pl_line &l);


inline pl_unitvec normal(const pl_edge_ifc &e)
{
    return e.supporting_line().normal();
}


//typedef reray<pl_edge> pl_edgereray;


bool is_to_the_left(const pl_fixedvec &, const pl_line &);

bool is_to_the_right(const pl_fixedvec &, const pl_line &);

bool is_above(const pl_fixedvec &, const pl_line &);

bool is_below(const pl_fixedvec &, const pl_line &);

#endif // PLAGEO_ONEDIM_H
