#ifndef PLAGEO_OVERLAP1_H
#define PLAGEO_OVERLAP1_H

#include <PLAGEO/base.h>
#include <PLAGEO/onedim.h>



//  ******** INTERSECTION DECISION ROUTINES********


// ******   VEC   ******

extern bool lies_inside(const pl_fixedvec &, const pl_box &);

inline bool do_overlap(const pl_fixedvec &vec, const pl_box &box)
{ return lies_inside(vec, box); }

// ******   POINT   ******

inline bool lies_inside(const pl_point &pt, const pl_box &box)
{ return lies_inside(pt.vec(), box); }


inline bool do_overlap(const pl_point &vt, const pl_box &box)
{ return lies_inside(vt.vec(), box); }


// ******   EDGE   ******

extern bool do_intersect(const pl_edge_ifc &e1, const pl_edge_ifc &e2);

extern bool do_intersect(const pl_edge_ifc &e1, const pl_ray &e2);

extern bool do_intersect(const pl_edge_ifc &e, const pl_line &l);

inline bool do_overlap(const pl_edge_ifc &, const pl_fixedvec &)
{ return false; }

inline bool do_overlap(const pl_edge_ifc &, const pl_point &)
{ return false; }

inline bool do_overlap(const pl_edge_ifc &edge1, const pl_edge_ifc &edge2)
{ return do_intersect(edge1, edge2); }

inline bool do_overlap(const pl_edge_ifc &edge, const pl_ray &ray)
{ return do_intersect(edge, ray); }

inline bool do_overlap(const pl_edge_ifc &edge, const pl_line &line)
{ return do_intersect(edge, line); }

extern bool do_overlap(const pl_edge_ifc &, const pl_box &);

// ******   RAY   ******

extern bool do_intersect(const pl_ray &, const pl_ray &);

extern bool do_intersect(const pl_ray &, const pl_line &);


inline bool do_overlap(const pl_ray &, const pl_fixedvec &)
{ return false; }

inline bool do_overlap(const pl_ray &, const pl_point &)
{ return false; }

inline bool do_overlap(const pl_ray &ray, const pl_edge_ifc &edge)
{ return do_intersect(edge, ray); }

inline bool do_overlap(const pl_ray &ray1, const pl_ray &ray)
{ return do_intersect(ray1, ray); }

inline bool do_overlap(const pl_ray &ray, const pl_line &line)
{ return do_intersect(ray, line); }

extern bool do_overlap(const pl_ray &, const pl_box &);



// ******   LINE   ******

inline bool do_overlap(const pl_line &, const pl_fixedvec &)
{ return false; }

inline bool do_overlap(const pl_line &, const pl_point &)
{ return false; }

inline bool do_overlap(const pl_line &line, const pl_edge_ifc &edge)
{ return do_intersect(edge, line); }

inline bool do_overlap(const pl_line &line1, const pl_ray &ray)
{ return do_intersect(ray, line1); }

inline bool do_overlap(const pl_line &, const pl_line &)
{ return true; }

extern bool do_overlap(const pl_line &, const pl_box &);


// ******   BOX   ******

inline bool do_overlap(const pl_box &box, const pl_fixedvec &vec)
{ return lies_inside(vec, box);}

inline bool do_overlap(const pl_box &box, const pl_point &vt)
{ return lies_inside(vt, box);}

inline bool do_overlap( const pl_box &box, const pl_edge_ifc &edge)
{ return do_overlap(edge, box); }

inline bool do_overlap(const pl_box &box, const pl_ray &ray)
{ return do_overlap(ray, box); }

inline bool do_overlap(const pl_box &box, const pl_line &line)
{ return do_overlap(line, box); }

extern bool do_overlap(const pl_box &, const pl_box &);


#endif //PLAGEO_OVERLAP1_H
