#ifndef PLAGEO_OVERLAP2_H
#define PLAGEO_OVERLAP2_H

#include <PLAGEO/base.h>
#include <PLAGEO/onedim.h>
#include <PLAGEO/curved.h>
#include <PLAGEO/pgn_ifc.h>
#include <PLAGEO/cross2.h>

extern bool lies_inside(const pl_fixedvec &, const pl_pgn_ifc &);

extern bool lies_inside(const pl_fixedvec &, const pl_disc &);

inline bool lies_inside(const pl_point &pt, const pl_pgn_ifc &pgn)
{ return lies_inside(pt.vec(), pgn); }
inline bool lies_inside(const pl_point &pt, const pl_disc &disc)
{ return lies_inside(pt.vec(), disc); }

extern bool do_overlap(const pl_object &, const pl_fixedvec &);
extern bool do_overlap(const pl_object &, const pl_point &);
extern bool do_overlap(const pl_object &, const pl_line &);
extern bool do_overlap(const pl_object &, const pl_ray &);
extern bool do_overlap(const pl_object &, const pl_edge_ifc &);
extern bool do_overlap(const pl_object &, const pl_arc &);
extern bool do_overlap(const pl_object &, const pl_box &);
extern bool do_overlap(const pl_object &, const pl_pgn_ifc &);
extern bool do_overlap(const pl_object &, const pl_disc &);
extern bool do_overlap(const pl_object &, const pl_object &);


inline bool do_overlap(const pl_fixedvec &vec, const pl_pgn_ifc &pgn)
{ return lies_inside(vec, pgn); }
inline bool do_overlap(const pl_fixedvec &vec, const pl_disc &disc)
{ return lies_inside(vec, disc); }
inline bool do_overlap(const pl_fixedvec &vec, const pl_object &obj)
{ return do_overlap(obj, vec); }
inline bool do_overlap(const pl_point &vt, const pl_pgn_ifc &pgn)
{ return lies_inside(vt.vec(), pgn); }
inline bool do_overlap(const pl_point &vt, const pl_disc &disc)
{ return lies_inside(vt.vec(), disc); }
inline bool do_overlap(const pl_point &vt, const pl_object &obj)
{ return do_overlap(obj, vt.vec()); }
inline bool do_overlap(const pl_edge_ifc &edge, const pl_arc &arc)
{ return do_intersect(edge, arc); }

extern bool do_overlap(const pl_edge_ifc &, const pl_pgn_ifc &);
extern bool do_overlap(const pl_edge_ifc &, const pl_disc &);
inline bool do_overlap(const pl_ray &ray, const pl_arc &arc)
{ return do_intersect(ray, arc); }

extern bool do_overlap(const pl_ray &, const pl_pgn_ifc &);
extern bool do_overlap(const pl_ray &, const pl_disc &);
inline bool do_overlap(const pl_line &line, const pl_arc &arc)
{ return do_intersect(line, arc); }

extern bool do_overlap(const pl_line &, const pl_pgn_ifc &);
extern bool do_overlap(const pl_line &, const pl_disc &);
inline bool do_overlap(const pl_arc &, const pl_fixedvec &)
{ return false; }
inline bool do_overlap(const pl_arc &, const pl_point &)
{ return false; }

inline bool do_overlap(const pl_arc &arc, const pl_edge_ifc &edge)
{ return do_intersect(arc, edge); }

inline bool do_overlap(const pl_arc &arc, const pl_ray &ray)
{ return do_intersect(arc, ray); }

inline bool do_overlap(const pl_arc &arc, const pl_line &line)
{ return do_intersect(arc, line); }

inline bool do_overlap(const pl_arc &arc1, const pl_arc &arc)
{ return do_intersect(arc1, arc); }

extern bool do_overlap(const pl_arc &, const pl_box &);
extern bool do_overlap(const pl_arc &, const pl_pgn_ifc &);
extern bool do_overlap(const pl_arc &, const pl_disc &);


inline bool do_overlap(const pl_box &box, const pl_arc &arc)
{ return do_overlap(arc, box); }

extern bool do_overlap(const pl_box & , const pl_pgn_ifc & );
extern bool do_overlap(const pl_box & , const pl_disc & );


inline bool do_overlap(const pl_pgn_ifc &pgn, const pl_fixedvec &vec)
{ return lies_inside(vec, pgn);}

inline bool do_overlap(const pl_pgn_ifc &pgn, const pl_point &vt)
{ return lies_inside(vt.vec(), pgn);}

inline bool do_overlap( const pl_pgn_ifc &pgn, const pl_edge_ifc &edge)
{ return do_overlap(edge, pgn); }

inline bool do_overlap(const pl_pgn_ifc &pgn, const pl_ray &ray)
{ return do_overlap(ray, pgn); }

inline bool do_overlap(const pl_pgn_ifc &pgn, const pl_line &line)
{ return do_overlap(line, pgn); }

inline bool do_overlap(const pl_pgn_ifc &pgn, const pl_arc &arc)
{ return do_overlap(arc, pgn); }

inline bool do_overlap(const pl_pgn_ifc &pgn, const pl_box &box)
{ return do_overlap(box, pgn); }

extern bool do_overlap(const pl_pgn_ifc & , const pl_pgn_ifc & );
extern bool do_overlap(const pl_pgn_ifc & , const pl_disc & );


inline bool do_overlap(const pl_disc &disc, const pl_fixedvec &vec)
{ return lies_inside(vec, disc);}

inline bool do_overlap(const pl_disc &disc, const pl_point &vt)
{ return lies_inside(vt.vec(), disc);}

inline bool do_overlap( const pl_disc &disc, const pl_edge_ifc &edge)
{ return do_overlap(edge, disc); }

inline bool do_overlap(const pl_disc &disc, const pl_ray &ray)
{ return do_overlap(ray, disc); }

inline bool do_overlap(const pl_disc &disc, const pl_line &line)
{ return do_overlap(line, disc); }

inline bool do_overlap(const pl_disc &disc, const pl_arc &arc)
{ return do_overlap(arc, disc); }

inline bool do_overlap(const pl_disc &disc, const pl_box &box)
{ return do_overlap(box, disc); }
inline bool do_overlap(const pl_disc &disc, const pl_pgn_ifc &pgn)
{ return do_overlap(pgn, disc); }
extern bool do_overlap(const pl_disc & , const pl_disc & );

#endif //PLAGEO_OVERLAP2_H
