#ifndef PLAGEO_PGN_H
#define PLAGEO_PGN_H

#include <PLAGEO/pgn_ifc.h>
#include <flags.h>
#include <row.h>
#include <reray.h>



typedef unsigned short ushort;


class pl_pgn_flags {
protected:
    flags16	    _flags;
    enum pgnflags{
		    bbflag = flags16::flag0,
		    sortflag = flags16::flag1, 
		    okflag = flags16::flag2,
		    clockwiseflag = flags16::flag3,
		    convexflag = flags16::flag4, 
		    is_convexf = flags16::flag5,  
		    is_clockwisef = flags16::flag6,  
		    is_okf = flags16::flag7, 
		    edgesflag = flags16::flag8, 
		    areaflag = flags16::flag9
    };
};

// ******** vertex and edge representation ********

class pl_pgn_vertices: virtual public pl_pgn_ifc {
public:
    int		    size() const;
const pl_vec &	    vertex(int i) const;
const row<pl_vec> & all_vertices() const {return _vertices;}
protected:
// mutators.
    void	    operator=(const pl_pgn_ifc &);
    void	    operator=(const pl_pgn_vertices &);
    void	    _make(const pl_pgn_ifc &);
    void	    _make(const pl_pgn_vertices &);
    void	    _invert_orientation();
    void	    _translate(const pl_fixedvec &vec);
    void	    _scale(s_coord);
    void	    _transform(const pl_transform &);
    void	    make_start(int n);
    void	    set_vertex(int i, const pl_vec &);
// data
    row<pl_vec>	    _vertices;
};

class pl_pgn_stored_edge_imp2
// this stores also the vertices.
    : virtual public pl_pgn_ifc {
public:
    int			    size() const;
const pl_vec &		    vertex(int i) const;
    void		    set_edge(int i) const;
    const pl_edge_ifc &	    cur_edge() const;
    const pl_unitvec &	    cur_direction() const;
const row<pl_edge_imp2> &   all_edges() const {return _edges;}
const pl_edge_ifc &	    edge(int i) const {return _edges[i];}
protected:
// mutators.
    void		    operator=(const pl_pgn_ifc &);
    void		    operator=(const pl_pgn_stored_edge_imp2 &);
    void		    _make(const pl_pgn_ifc &);
    void		    _make(const pl_pgn_stored_edge_imp2 &);
    void		    _invert_orientation();
    void		    _translate(const pl_fixedvec &vec);
    void		    _scale(s_coord);
    void		    _transform(const pl_transform &);
    void		    make_start(int n);
    void		    set_vertex(int i, const pl_vec &);
    void		    _make_ready();
// data
    row<pl_edge_imp2>	    _edges;
    int			    _cur_edge;
};

class pl_pgn_cached_edge_imp2
    : virtual public pl_pgn_ifc, virtual public pl_pgn_flags {
public:
    void		    set_edge(int i) const;
    const pl_edge_ifc &	    cur_edge() const;
    const pl_unitvec &	    cur_direction() const;
const row<pl_edge_imp2> &   all_edges() const;
const pl_edge_ifc &	    edge(int i) const;
protected:
// mutators.
    void		    operator=(const pl_pgn_ifc &);
    void		    operator=(const pl_pgn_cached_edge_imp2 &);
    void		    _make(const pl_pgn_ifc &);
    void		    _make(const pl_pgn_cached_edge_imp2 &);
    void		    _invert_orientation();
    void		    _translate(const pl_fixedvec &vec);
    void		    _scale(s_coord);
    void		    _transform(const pl_transform &);
    void		    _make_ready();
    void		    compute_edges();
// data
    row<pl_edge_imp2>	    _edges;
    int			    _cur_edge;
};

class pl_pgn_otf_edge_imp2  // otf means on the fly
    : virtual public pl_pgn_ifc {
public:
    void		set_edge(int i) const;
    const pl_edge_ifc &	cur_edge() const;
    const pl_unitvec &	    cur_direction() const;
protected:
    pl_edge_imp2	_cur_edge;
};

// ******** ordering of vertices ********

class pl_pgn_stored_order
    : virtual public pl_pgn_ifc {
public:
    int		    element(int i) const;
const pl_vec &	    velement(int i) const;
    int		    order(int i) const;
protected:
// mutators.
    void	    operator=(const pl_pgn_ifc &);
    void	    operator=(const pl_pgn_stored_order &);
    void	    _make(const pl_pgn_ifc &);
    void	    _make(const pl_pgn_stored_order &);
    void	    ncsort();
    void	    _invert_orientation();
    void	    _scale(s_coord);
    void	    _transform(const pl_transform &);
    void	    _make_ready();
// data.
    row<ushort>	    _el;		// als i<j dan vl[el[i]]<=vl[el[j]]
    row<ushort>	    _ord;		// ord[j]==i <-> el[i]==j
};

class pl_pgn_cached_order
    : virtual public pl_pgn_stored_order, virtual public pl_pgn_flags {
public:
    int		    element(int i) const;
const pl_vec &	    velement(int i) const;
    int		    order(int i) const;
protected:
    void	    operator=(const pl_pgn_ifc &);
    void	    operator=(const pl_pgn_cached_order &);
    void	    _make(const pl_pgn_ifc &);
    void	    _make(const pl_pgn_cached_order &);
    void	    sort() const;
    void	    _invert_orientation();
    void	    _scale(s_coord);
    void	    _transform(const pl_transform &);
    void	    _make_ready();
};

// ******** bounding box ********

class pl_pgn_stored_bbox: virtual public pl_pgn_ifc {
public:
    pl_box	    bbox() const;
protected:
// mutators.
    void	    operator=(const pl_pgn_ifc &);
    void	    operator=(const pl_pgn_stored_bbox &);
    void	    _make(const pl_pgn_ifc &);
    void	    _make(const pl_pgn_stored_bbox &);
    void	    _translate(const pl_fixedvec &vec);
    void	    _scale(s_coord);
    void	    _transform(const pl_transform &);
    void	    _make_ready();
    void	    makebbox() ;
// data.
    pl_box	    _bb;
};

class pl_pgn_cached_bbox
    : virtual public pl_pgn_stored_bbox, virtual public pl_pgn_flags {
public:
    pl_box	    bbox() const;
protected:
    void	    operator=(const pl_pgn_ifc &);
    void	    operator=(const pl_pgn_cached_bbox &);
    void	    _make(const pl_pgn_ifc &);
    void	    _make(const pl_pgn_cached_bbox &);
    void	    _translate(const pl_fixedvec &vec);
    void	    _scale(s_coord);
    void	    _transform(const pl_transform &);
    void	    _make_ready();
};

// ******** area ********

class pl_pgn_otf_area: virtual public pl_pgn_ifc {
public:
    s_coord	    area() const;
protected:
    s_coord	    compute_area();
};

class pl_pgn_stored_area: virtual public pl_pgn_otf_area {
public:
    s_coord	    area() const;
protected:
// mutators.
    void	    operator=(const pl_pgn_ifc &);
    void	    operator=(const pl_pgn_stored_area &);
    void	    _make(const pl_pgn_ifc &);
    void	    _make(const pl_pgn_stored_area &);
    void	    _scale(s_coord);
    void	    _transform(const pl_transform &);
    void	    _make_ready();
// data.
    s_coord	    _area;
};

class pl_pgn_cached_area
    : virtual public pl_pgn_stored_area, virtual public pl_pgn_flags {
public:
    s_coord	    area() const;
protected:
    void	    operator=(const pl_pgn_ifc &);
    void	    operator=(const pl_pgn_cached_area &);
    void	    _make(const pl_pgn_ifc &);
    void	    _make(const pl_pgn_cached_area &);
    void	    _scale(s_coord);
    void	    _transform(const pl_transform &);
    void	    _make_ready();
};

// ******** is_ok ********

class pl_pgn_otf_ok
    : virtual public pl_pgn_ifc {
public:
    bool	    is_ok() const;
protected:
    bool	    compute_ok();
};

class pl_pgn_cached_ok
    : virtual public pl_pgn_otf_ok, virtual public pl_pgn_flags {
public:
    bool	    is_ok() const;
protected:
    void	    operator=(const pl_pgn_ifc &);
    void	    operator=(const pl_pgn_cached_ok &);
    void	    _make(const pl_pgn_ifc &);
    void	    _make(const pl_pgn_cached_ok &);
    void	    _scale(s_coord);
    void	    _make_ready();
};

// ******** is_clockwise ********

class pl_pgn_otf_clockwise
    : virtual public pl_pgn_ifc {
public:
    bool	    is_clockwise() const;
protected:
    bool	    compute_clockwise();
};

class pl_pgn_cached_clockwise
    : virtual public pl_pgn_otf_clockwise, virtual public pl_pgn_flags {
public:
    bool	    is_clockwise() const;
protected:
    void	    operator=(const pl_pgn_ifc &);
    void	    operator=(const pl_pgn_cached_clockwise &);
    void	    _make(const pl_pgn_ifc &);
    void	    _make(const pl_pgn_cached_clockwise &);
    void	    _invert_orientation();
    void	    _scale(s_coord);
    void	    _transform(const pl_transform &);
    void	    _make_ready();
};

// ******** is_convex ********

class pl_pgn_otf_convex
    : virtual public pl_pgn_ifc {
public:
    bool	    is_convex() const;
protected:
    bool	    compute_convex();
};

class pl_pgn_cached_convex
    : virtual public pl_pgn_otf_convex, virtual public pl_pgn_flags {
public:
    bool	    is_convex() const;
protected:
    void	    operator=(const pl_pgn_ifc &);
    void	    operator=(const pl_pgn_cached_convex &);
    void	    _make(const pl_pgn_ifc &);
    void	    _make(const pl_pgn_cached_convex &);
    void	    _make_ready();
};


// ********   ********

class pl_pgn_imp2:
	virtual public pl_pgn_ifc, 
	virtual public pl_pgn_stored_edge_imp2,
	virtual public pl_pgn_stored_bbox,
	virtual public pl_pgn_stored_area,
	virtual public pl_pgn_stored_order,
	virtual public pl_pgn_otf_ok,
	virtual public pl_pgn_otf_convex,
	virtual public pl_pgn_otf_clockwise
{
public:
		    pl_pgn_imp2() {}
		    ~pl_pgn_imp2();
    void	    operator=(const pl_pgn_ifc &);
    void	    operator=(const pl_pgn_imp2 &);
    void	    invert_orientation();
    void	    translate(const pl_fixedvec &vec);
    void	    scale(s_coord);
    void	    transform(const pl_transform &);
    void	    make_ready();
};

class pl_pgn_imp3:
	virtual public pl_pgn_ifc, 
	virtual public pl_pgn_vertices, 
	virtual public pl_pgn_cached_edge_imp2, 
	virtual public pl_pgn_cached_bbox, 
	virtual public pl_pgn_cached_area, 
	virtual public pl_pgn_cached_order,
	virtual public pl_pgn_cached_ok,
	virtual public pl_pgn_cached_convex,
	virtual public pl_pgn_cached_clockwise {
public:
		    pl_pgn_imp3() {}
		    ~pl_pgn_imp3();
    void	    operator=(const pl_pgn_ifc &);
    void	    operator=(const pl_pgn_imp3 &);
    void	    invert_orientation();
    void	    translate(const pl_fixedvec &vec);
    void	    scale(s_coord);
    void	    transform(const pl_transform &);
    void	    make_ready();
};


// ********

class pl_pgn_imp1 :
	virtual public pl_pgn_ifc, 
	virtual public pl_pgn_cached_edge_imp2, 
	virtual public pl_pgn_cached_bbox, 
	virtual public pl_pgn_cached_area, 
	virtual public pl_pgn_cached_order,
	virtual public pl_pgn_cached_ok,
	virtual public pl_pgn_cached_convex,
	virtual public pl_pgn_cached_clockwise {
public:
		    pl_pgn_imp1() {}
		    pl_pgn_imp1(const pl_vecreray &vl): _vertices(vl) {}
		    ~pl_pgn_imp1();
const pl_vec &	    vertex(int i) const;
    int		    size() const;
    void	    operator=(const pl_pgn_ifc &);
    void	    operator=(const pl_pgn_imp1 &);
    void	    invert_orientation();
    void	    translate(const pl_fixedvec &vec);
    void	    scale(s_coord);
    void	    transform(const pl_transform &);
const pl_vecreray & all_vertices() const {return _vertices;}
    void	    all_vertices(pl_vecreray &result) const {result = _vertices;}
protected:
    pl_vecreray	    _vertices;
protected:
    void	    make_start(int n);
    void	    set_vertex(int i, const pl_vec &);
    void	    make_ready();
};

typedef pl_pgn_imp1 pl_pgn;

#endif // PLAGEO_PGN_H
