#ifndef PLAGEO_TRANSFORM_H
#define PLAGEO_TRANSFORM_H

#include <PLAGEO/base.h>


class pl_unit_tfm : public pl_transform{
public:
    virtual pl_vec	operator*(const pl_vec &) const =0;
    virtual pl_unitvec	operator*(const pl_unitvec &) const =0;
    virtual pl_angle	operator*(pl_angle) const = 0;
    s_coord	operator*(s_coord) const;
};


class pl_uniscale: public pl_transform {
public:
		pl_uniscale()				: _scale(1) {}
		pl_uniscale(s_coord f)			: _scale(f) {}
    pl_vec	operator*(const pl_vec &) const ;
    pl_unitvec	operator*(const pl_unitvec &) const ;
    pl_angle	operator*(pl_angle) const;
    s_coord	operator*(s_coord) const ;
    s_coord	factor() const				{ return _scale;}
protected:
    s_coord	_scale;
};

class pl_translation: public pl_unit_tfm {
public:
		pl_translation()			: _trans(0, 0) {}
		pl_translation(const pl_fixedvec &v)		:_trans(v) {}
    pl_vec	operator*(const pl_vec &) const ;
    s_coord	operator*(s_coord) const ;
    pl_unitvec	operator*(const pl_unitvec &) const ;
    pl_angle	operator*(pl_angle) const;
    pl_vec	vec() const				    { return _trans;}
protected:
    pl_vec	_trans;
};

class pl_rotation: public pl_unit_tfm {
public:
		pl_rotation() : _angle(0) {_rot[0]=1; _rot[1]=0;}
		pl_rotation(pl_angle theta) ;
    pl_vec	operator*(const pl_vec &) const ;
    s_coord	operator*(s_coord) const ;
    pl_unitvec	operator*(const pl_unitvec &) const ;
    pl_angle	operator*(pl_angle) const;
    pl_angle	angle() const ;
protected:
    s_coord	_rot[2];
    pl_angle	_angle;
};

class pl_rotra: public pl_unit_tfm {
public:
		pl_rotra()  : _trans(0, 0), _angle(0) {_rot[0]=1;_rot[1]=0;}
		pl_rotra(pl_angle theta, const pl_fixedvec &vec) ;
		pl_rotra(pl_angle theta) ;
		pl_rotra(const pl_fixedvec &vec) ;
    pl_rotra	inverse() const;
    pl_vec	operator*(const pl_vec &) const ;
    s_coord	operator*(s_coord) const ;
    pl_unitvec	operator*(const pl_unitvec &) const ;
    pl_angle	operator*(pl_angle) const;
    pl_rotra	operator*(const pl_rotra &) const;
    void	get_params(pl_angle &theta, pl_vec &vec) const;
    bool	operator==(const pl_rotra &) const;
    void	set_x(s_coord x)			{ _trans.set_x(x); }
    s_coord	x() const				{ return _trans.x();}
    s_coord	get_x() const				{ return _trans.x();}
    void	set_y(s_coord y)			{ _trans.set_y(y); }
    s_coord	y() const				{ return _trans.y();}
    s_coord	get_y() const				{ return _trans.y();}
    void	set_angle(pl_angle theta) ;
    pl_angle	get_angle() const ;
protected:
    pl_angle	_angle;
    s_coord	_rot[2];
    pl_vec	_trans;
};

class pl_ortho_tfm: public pl_transform {
public:
		pl_ortho_tfm() : _angle(0), _scale(1), _trans(0, 0) {_tfm[0]=1; _tfm[1]=0;}
		pl_ortho_tfm(pl_angle, s_coord, const pl_fixedvec &) ;
    pl_vec	operator*(const pl_vec &) const ;
    pl_unitvec	operator*(const pl_unitvec &) const ;
    pl_angle	operator*(pl_angle) const;
    s_coord	operator*(s_coord) const ;
    pl_ortho_tfm	operator*(const pl_ortho_tfm &) const;
    bool	operator==(const pl_ortho_tfm &) const;
    void	get_params(pl_angle &theta, s_coord &scale, pl_vec &trans) ;
protected:
    pl_angle	_angle;
    s_coord	_scale;
    s_coord	_tfm[2];    // transformation matrix (top row),
    pl_vec	_trans;
};
#endif // PLAGEO_TRANSFORM_H
