#include <PLAGEO/basebase.h>

#ifdef __sgi

#include <fp_class.h>

bool is_finite_pp(float x)
{
    switch (fp_class_f(x)) {
    case FP_SNAN:
    case FP_QNAN:
    case FP_POS_INF:
    case FP_NEG_INF:
	return 0;
    default:
	return 1;
    }
}

bool is_finite_pp(double x)
{
    switch (fp_class_d(x)) {
    case FP_SNAN:
    case FP_QNAN:
    case FP_POS_INF:
    case FP_NEG_INF:
	return 0;
    default:
	return 1;
    }
}

#endif // __sgi



#ifdef __hpux


bool is_finite_pp(float x)
{
    return finitef(x);
}

bool is_finite_pp(double x)
{
    return finite(x);
}

#endif //  _hp_ux




// **** new stuff




#if !defined(__sgi) && !defined(__sun) && !defined(__hpux) && !defined(__linux)


bool is_valid_pp(float d)
{
    return (d == d);               /* !!! */
}

bool is_finite_pp(float d)
{
    return (d == d) && (is_valid_pp(d-d));
}


#else  // custom definitions.

#ifdef __sgi

// implementation for SGI IRIX 5.3.
#include <fp_class.h>


bool is_finite_pp(float d)
{
    switch (fp_class_f(d)) {
    case FP_POS_NORM:
    case FP_NEG_NORM:
    case FP_POS_ZERO:
    case FP_NEG_ZERO:
    case FP_POS_DENORM:
    case FP_NEG_DENORM:
        return true;
    case FP_SNAN:
    case FP_QNAN:
    case FP_POS_INF:
    case FP_NEG_INF:
        return false;
    }
    return false; // NOT REACHED
}

bool is_valid_pp(float d)
{
    switch (fp_class_f(d)) {
    case FP_POS_NORM:
    case FP_NEG_NORM:
    case FP_POS_ZERO:
    case FP_NEG_ZERO:
    case FP_POS_INF:
    case FP_NEG_INF:
    case FP_POS_DENORM:
    case FP_NEG_DENORM:
        return true;
    case FP_SNAN:
    case FP_QNAN:
        return false;
    }
    return false; // NOT REACHED
}

#endif // __sgi

#ifdef __hpux

// implementation for HP


bool is_valid_pp(float f)
{
    return isnanf(f) == 0;
}

bool is_finite_pp(float f)
{
    switch (fpclassifyf(f)) {
    case FP_PLUS_NORM:
    case FP_MINUS_NORM:
    case FP_PLUS_ZERO:
    case FP_MINUS_ZERO:
    case FP_PLUS_DENORM:
    case FP_MINUS_DENORM:
        return true;
    case FP_PLUS_INF:
    case FP_MINUS_INF:
    case FP_SNAN:
    case FP_QNAN:
        return false;
    }
    return false; // NOT REACHED
}

#endif // __hpux

#ifdef __sun

// implementation for SUN

#ifdef __SVR4
#include <ieeefp.h>
#endif // __SVR4

#ifdef __svr4__
#include <ieeefp.h>
#endif //  __svr4__

// implementation for Sun


bool is_valid_pp(float f)
{
    return isnanf(f) == 0;
}

bool is_finite_pp(float f)
{
    return finite(f);
}

#endif // __sun

#ifdef __linux
// Implementation for Linux

bool is_finite_pp(float f)
{
  return finite(f) != 0;
}

bool is_valid_pp(float f)
{
  return isnan(f) == 0;
}


#endif // __linux
#endif // custom definitions.

// **** Double.C






#if !defined(__sgi) && !defined(__sun) && !defined(__hpux) && !defined(__linux)

bool is_valid_pp(double d)
{
    return (d == d);             /* !!! */
}

bool is_finite_pp(double d)
{
    return (d == d) && (CGAL_is_valid_pp(d-d));
}

#else  // custom definitions.

#ifdef __sgi

// implementation for SGI IRIX 5.3.
#include <fp_class.h>

bool is_finite_pp(double d)
{
    switch (fp_class_d(d)) {
    case FP_POS_NORM:
    case FP_NEG_NORM:
    case FP_POS_ZERO:
    case FP_NEG_ZERO:
    case FP_POS_DENORM:
    case FP_NEG_DENORM:
        return true;
    case FP_SNAN:
    case FP_QNAN:
    case FP_POS_INF:
    case FP_NEG_INF:
        return false;
    }
    return false; // NOT REACHED
}

bool is_valid_pp(double d)
{
    switch (fp_class_d(d)) {
    case FP_POS_NORM:
    case FP_NEG_NORM:
    case FP_POS_ZERO:
    case FP_NEG_ZERO:
    case FP_POS_INF:
    case FP_NEG_INF:
    case FP_POS_DENORM:
    case FP_NEG_DENORM:
        return true;
    case FP_SNAN:
    case FP_QNAN:
        return false;
    }
    return false; // NOT REACHED
}

#endif // __sgi

#ifdef __hpux

// implementation for HP

bool is_valid_pp(double d)
{
    return isnan(d) == 0;
}

bool is_finite_pp(double d)
{
    switch (fpclassify(d)) {
    case FP_PLUS_NORM:
    case FP_MINUS_NORM:
    case FP_PLUS_ZERO:
    case FP_MINUS_ZERO:
    case FP_PLUS_DENORM:
    case FP_MINUS_DENORM:
        return true;
    case FP_PLUS_INF:
    case FP_MINUS_INF:
    case FP_SNAN:
    case FP_QNAN:
        return false;
    }
    return false; // NOT REACHED
}

#endif // __hpux

#ifdef __sun

// implementation for SUN

#ifdef __SVR4
#include <ieeefp.h>
#endif // __SVR4

#ifdef __svr4__
#include <ieeefp.h>
#endif //  __svr4__

bool is_finite_pp(double d)
{
  if(finite(d)){
    return true;
  }
  return false; // NOT REACHED
}

bool is_valid_pp(double d)
{
  return isnan(d) == 0;
}

#endif // __sun

#ifdef __linux

// Implementation for Linux

bool is_finite_pp(double d)
{
  return finite(d) != 0;
}

bool is_valid_pp(double d)
{
  return isnan(d) == 0;
}

#endif // __linux
#endif // of custom definitions



