#include <gl/gl.h>
#include <gl/get.h>
#include <PLAGEO/glwin.h>
#include <PLAGEO/glsubwin.h>

static Matrix unitmat = { {1, 0, 0, 0}, {0, 1, 0, 0}, {0, 0, 1, 0}, {0, 0, 0, 1}};

void pl_use_colormap(int col)
{
    ::color(col);
}

void pl_use_rgbcolor(int col)
{
    long color_comp[3];
    int red, green, blue;
    pl_color_components(red, green, blue, col);
    color_comp[0] = red;
    color_comp[1] = green;
    color_comp[2] = blue;
    c3i(color_comp);
}

void pl_use_autocolor(int col)
{
    long displaymode;
    displaymode = getdisplaymode();
    if (displaymode == DMSINGLE || displaymode==DMDOUBLE)
	pl_use_colormap(col);
    else
	pl_use_rgbcolor(col);
}

pl_glsubwin::pl_glsubwin()
{
    set_renderarea(0, 0, 10, 10);
    _cleanpage = 1;
    _colorfunc = pl_use_autocolor;
    init_values();
}

pl_glsubwin::pl_glsubwin(int x, int y, int dx, int dy)
{
    set_renderarea(x, y, dx, dy);
    _cleanpage = 1;
    _colorfunc = pl_use_autocolor;
    init_values();
}

pl_glsubwin::~pl_glsubwin()
{
    pageready();    // make sure that viewport is popped.
}

void pl_glsubwin::set_renderarea(int x, int y, int dx, int dy)
{
    if (dx == 0) dx=1;
    if (dx>0) {
	_winx = x; _windx = dx;
    } else {
	_winx = x+dx; _windx = -dx;
    }
    if (dy == 0) dy=1;
    if (dy>0) {
	_winy = y; _windy = dy;
    } else {
	_winy = y+dy; _windy = -dy;
    }
}

void pl_glsubwin::init_page()
{
    if (!_cleanpage)
	return;
    _cleanpage = 0;
    _prevmode = getmmode();
    
    if (_prevmode == MSINGLE) {
	getmatrix(_savemat);
    } else {
	mmode(MPROJECTION);
	getmatrix(_savemat);
    }
    mmode(MVIEWING);
    concave(1);
    pushviewport();
    ortho2(_viewing_x, _viewing_x+_viewing_xsize,
	    _viewing_y, _viewing_y+_viewing_ysize);
    viewport(Screencoord(_winx), Screencoord(_winx+_windx-1), 
		Screencoord(_winy), Screencoord(_winy+_windy-1));
    loadmatrix(unitmat);
    _colorfunc(_backcol);
    ::clear();
}

void pl_glsubwin::pageready()
{
    if (_cleanpage)
	return;
    _cleanpage = 1;
    popviewport();
    if (_prevmode == MSINGLE) {
	mmode(MSINGLE);
	loadmatrix(_savemat);
    } else {
	mmode(MPROJECTION);
	loadmatrix(_savemat);
	mmode(short(_prevmode));
    }
}

// dirty procedure to quickly pass a vertex to the the GL.
// makes use of the internal representation of a vertex.
static inline void vv(const pl_fixedvec &vt)
{
    v2f(& (s_coord &)(vt.x()));
}

static void pl_draw(const pl_fixedvec &vt, int how, s_coord radius)
{ 
    pl_vec vtx;
    switch(how) {
    case 0:	// dot
	bgnpoint();
	vv(vt);
	endpoint();
	break;
    case 1:	// cross
    default:
	vtx.set_x(vt.x());
	vtx.set_y(vt.y() - radius);
	bgnline();
	vv(vtx);
	vtx.set_y(vt.y() + radius);
	vv(vtx);
	endline();
	vtx.set_x(vt.x() - radius);
	vtx.set_y(vt.y());
	bgnline();
	vv(vtx);
	vtx.set_x(vt.x() + radius);
	vv(vtx);
	endline();
	break;
    case 2:	// square
	vtx.set_x(vt.x() - radius);
	vtx.set_y(vt.y() - radius);
	bgnclosedline();
	vv(vtx);
	vtx.set_x(vt.x() + radius);
	vv(vtx);
	vtx.set_y(vt.y() + radius);
	vv(vtx);
	vtx.set_x(vt.x() - radius);
	vv(vtx);
	endclosedline();
	break;
    case 3:	// box
	vtx.set_x(vt.x() - radius);
	vtx.set_y(vt.y() - radius);
	bgnpolygon();
	vv(vtx);
	vtx.set_x(vt.x() + radius);
	vv(vtx);
	vtx.set_y(vt.y() + radius);
	vv(vtx);
	vtx.set_x(vt.x() - radius);
	vv(vtx);
	endpolygon();
	break;
    }
}

static void pl_draw(const pl_edge_ifc &e)
{
    bgnline();
	vv(e.begin_vt());
	vv(e.end_vt());
    endline();
}
    
/*
static void pl_draw(const pl_pgn_ifc &pgn, int how)
{
    int i;
    switch (how) {
    case 1:
    default:
        bgnpolygon();
        for(i=0; i<pgn.size(); i++) {
            vv(pgn.vertex(i));
	}
        endpolygon();
	break;
    case 0:
        bgnclosedline();
        for(i=0; i<pgn.size(); i++) {
            vv(pgn.vertex(i));
	}
        endclosedline();
        break;
    }
}
*/    

void pl_glsubwin::clear()
{
    init_page();
    _colorfunc(_backcol);
    ::clear();
}

void pl_glsubwin::draw(const pl_fixedvec &vt)
{
    init_page();
    float viewing_size = _viewing_xsize>_viewing_ysize ? _viewing_xsize : _viewing_ysize;
    float radius = _vertexsize * viewing_size/1000;
    _colorfunc(_linecol);
    pl_draw(vt, _vertexshape,  radius);
}

void pl_glsubwin::draw(const pl_edge_ifc &e)
{
    init_page();
    _colorfunc(_linecol);
    pl_draw(e);
}

void pl_glsubwin::draw(const pl_pgn_ifc &pgn)
{
    init_page();
    switch (_areafill) {
    case FILL:
    case FILLOUT:
    default:
	_colorfunc(_areacol);
	pl_draw(pgn, 1);
	if (_areafill == FILL)
	    break;
    case OUTLINE:
	_colorfunc(_linecol);
	pl_draw(pgn, 0);
        break;
    }
}

void pl_glsubwin::draw(const pl_disc &disc)
{
    init_page();
    switch (_areafill) {
    case FILL:
    case FILLOUT:
    default:
	_colorfunc(_areacol);
	circf(disc.center().x(), disc.center().y(), disc.radius());
	if (_areafill == FILL)
	    break;
    case OUTLINE:
	_colorfunc(_linecol);
	circ(disc.center().x(), disc.center().y(), disc.radius());
        break;
    }
}

void pl_glsubwin::draw(const pl_arc &arc)
{
    init_page();
    const s_coord to_deg = 1800/M_PI;
    _colorfunc(_linecol);
    ::arc(arc.center().x(), arc.center().y(), arc.radius(),
	    Angle(to_deg*arc.small_angle().normalised_value(0)),
	    Angle(to_deg*arc.big_angle().normalised_value(0)));
}

