#include <gl/gl.h>
#include <gl/device.h>
#include <string.h>
#include <PLAGEO/glwin.h>

// ***********  Procedures for GL windows *****************

pl_glwindow::active_win::active_win(long winid)
{
    _keepwin = winget();
    _curwin = winid;
    if (_keepwin != winid) winset(winid);
}

pl_glwindow::active_win::~active_win()
{
    if (_keepwin != _curwin && _keepwin >= 0) winset(_keepwin);
}

// dirty procedure to quickly pass a vertex to the the GL.
// makes use of the internal representation of a vertex.
static inline void vv(const pl_fixedvec &vt)
{
    v2f(& (s_coord &)(vt.x()));
}

static void pl_draw(const pl_fixedvec &vt, int how, s_coord radius)
{ 
    pl_vec vtx;
    switch(how) {
    case 0:	// dot
	bgnpoint();
	vv(vt);
	endpoint();
	break;
    case 1:	// cross
    default:
	vtx.set_x(vt.x());
	vtx.set_y(vt.y() - radius);
	bgnline();
	vv(vtx);
	vtx.set_y(vt.y() + radius);
	vv(vtx);
	endline();
	vtx.set_x(vt.x() - radius);
	vtx.set_y(vt.y());
	bgnline();
	vv(vtx);
	vtx.set_x(vt.x() + radius);
	vv(vtx);
	endline();
	break;
    case 2:	// square
	vtx.set_x(vt.x() - radius);
	vtx.set_y(vt.y() - radius);
	bgnclosedline();
	vv(vtx);
	vtx.set_x(vt.x() + radius);
	vv(vtx);
	vtx.set_y(vt.y() + radius);
	vv(vtx);
	vtx.set_x(vt.x() - radius);
	vv(vtx);
	endclosedline();
	break;
    case 3:	// box
	vtx.set_x(vt.x() - radius);
	vtx.set_y(vt.y() - radius);
	bgnpolygon();
	vv(vtx);
	vtx.set_x(vt.x() + radius);
	vv(vtx);
	vtx.set_y(vt.y() + radius);
	vv(vtx);
	vtx.set_x(vt.x() - radius);
	vv(vtx);
	endpolygon();
	break;
    }
}

static void pl_draw(const pl_edge_ifc &e)
{
    bgnline();
	vv(e.begin_vt());
	vv(e.end_vt());
    endline();
}
    
void pl_draw(const pl_pgn_ifc &pgn, int how)
{
    int i;
    switch (how) {
    case 1:
    default:
        bgnpolygon();
        for(i=0; i<pgn.size(); i++) {
            vv(pgn.vertex(i));
	}
        endpolygon();
	break;
    case 0:
        bgnclosedline();
        for(i=0; i<pgn.size(); i++) {
            vv(pgn.vertex(i));
	}
        endclosedline();
        break;
    }
}
    

void pl_glwindow::draw(const pl_fixedvec &vt)
{
    win_init();
    active_win aw(winid);
    float viewing_size = _viewing_xsize>_viewing_ysize ? _viewing_xsize : _viewing_ysize;
    float radius = _vertexsize * viewing_size/1000;
    color(_linecol);
    pl_draw(vt, _vertexshape,  radius);
}

void pl_glwindow::draw(const pl_edge_ifc &e)
{
    win_init();
    active_win aw(winid);
    color(_linecol);
    pl_draw(e);
}

void pl_glwindow::draw(const pl_pgn_ifc &pgn)
{
    win_init();
    active_win aw(winid);
    switch (_areafill) {
    case FILL:
    case FILLOUT:
    default:
	color(_areacol);
	pl_draw(pgn, 1);
	if (_areafill == FILL)
	    break;
    case OUTLINE:
	color(_linecol);
	pl_draw(pgn, 0);
        break;
    }
}

void pl_glwindow::draw(const pl_disc &disc)
{
    win_init();
    active_win aw(winid);
    switch (_areafill) {
    case FILL:
    case FILLOUT:
    default:
	color(_areacol);
	circf(disc.center().x(), disc.center().y(), disc.radius());
	if (_areafill == FILL)
	    break;
    case OUTLINE:
	color(_linecol);
	circ(disc.center().x(), disc.center().y(), disc.radius());
        break;
    }
}

void pl_glwindow::draw(const pl_arc &arc)
{
    win_init();
    const s_coord to_deg = 1800/M_PI;
    active_win aw(winid);
    color(_linecol);
    ::arc(arc.center().x(), arc.center().y(), arc.radius(),
	    Angle(to_deg*arc.small_angle().normalised_value(0)),
	    Angle(to_deg*arc.big_angle().normalised_value(0)));
}

long pl_glwindow::identifier() const
{
    ((pl_glwindow *)this)->win_init();	// cast away const
    return winid;
}

pl_vec pl_glwindow::get_mouseposition()
{
    long screenx, screeny, winx, winy, orgx, orgy;
    pl_vec result;
    win_init();
    active_win aw(winid);
    screenx = getvaluator(MOUSEX);
    screeny = getvaluator(MOUSEY);
    getorigin(&orgx, &orgy);
    winx = screenx-orgx;
    winy = screeny-orgy;
    result.set_x((winx*_viewing_xsize)/_windx + _viewing_x);
    result.set_y((winy*_viewing_ysize)/_windy + _viewing_y);
    return result;
}
    
int pl_glwindow::get_mousestatus(int button)
{
    Device mouseno;
    win_init();
    mouseno = (button==1) ? LEFTMOUSE : (button==2) ? MIDDLEMOUSE : RIGHTMOUSE;
    return getbutton(mouseno);
}

pl_glwindow::pl_glwindow()
{
    winid = -1;
    _ds = 0;
    _winx = 100;
    _winy = 100;
    _windx = -100;
    _windy = -100;
    _name = 0;
    init_values();
}

pl_glwindow::pl_glwindow(int dx, int dy)
{
    winid = -1;
    _ds = 0;
    _winx = dx;
    _winy = dy;
    _windx = -100;
    _windy = -100;
    _name = 0;
    init_values();
}

pl_glwindow::pl_glwindow(int x, int y, int dx, int dy)
{
    winid = -1;
    _ds = 0;
    if (dx <= 0 || dy<=0) {
	_winx = 100;
	_winy = 100;
	_windx = -100;
	_windy = -100;
    } else {
	_winx = x;
	_winy = y;
	_windx = dx;
	_windy = dy;
    }
    _name = 0;
    init_values();
}

pl_glwindow::~pl_glwindow()
{
    hide();
}


void pl_glwindow::hide()
{
    if (winid >= 0) {
	winclose(winid);
	winid = -1;
    }
}

int pl_glwindow::set_directshow(int i)
{
    int oldds = _ds;
    _ds = i;
    if (winid >= 0) {
	active_win aw(winid);
	if (_ds)
	    singlebuffer();
	else
	    doublebuffer();
	gconfig();
    }
    return oldds;
}

void pl_glwindow::clear()
{
    win_init();
    active_win aw(winid);
    color(_backcol);
    ::clear();
}

void pl_glwindow::pageready()
{
    win_init();
    if (!_ds) {
	active_win aw(winid);
	swapbuffers();
    }
    _background_dirty = 1;
}

void pl_glwindow::set_name(const char *name)
{
    if (name == 0)
	return;
    _name = strdup(name);   // this space is never reclaimed.
}

void pl_glwindow::win_init()
{
    if (winid < 0) {
	long prevwin = winget();
	if (_windx<0)
	    keepaspect(_winx, _winy);
	else
	    prefposition(_winx, _winx+_windx, _winy, _winy+_windy);
	winid = winopen(_name==0 ?"plwin":_name);
	if (winid < 0) {
	    if (prevwin >= 0) winset(prevwin);
	    return;
	}
	getorigin(&_winx, &_winy);
	getsize(&_windx, &_windy);
	if (!_ds)
	    doublebuffer();
	concave(1);
	gconfig();
	_viewing_changed = 1;
	_background_dirty = 1;
	if (prevwin >= 0) winset(prevwin);
    }
    if (_viewing_changed) {
	_viewing_changed = 0;
	active_win aw(winid);
	ortho2(_viewing_x, _viewing_x+_viewing_xsize,
		_viewing_y, _viewing_y+_viewing_ysize);
    }
    if (_background_dirty) {
	_background_dirty = 0;
	active_win aw(winid);
	reshapeviewport();
	color(_backcol);
	::clear();
    }
}


