
#include <iostream.h>
#include <fstream.h>
#include <PLAGEO/io.h>



ostream & operator<<(ostream &ofile, const pl_angle &angle)
{
    return ofile << angle.value();
}

ostream & operator<<(ostream &ofile, const pl_fixedvec &vec)
{
    return ofile << vec.x() <<' '<<vec.y();
}

ostream & operator<<(ostream &ofile, const pl_edge_ifc &edge)
{
    return ofile << edge.begin_vt() << ' ' << edge.end_vt();
}

ostream & operator<<(ostream &ofile, const pl_ray &ray)
{
    return ofile << ray.begin_vt() << ' ' << ray.direction();
}

ostream & operator<<(ostream &ofile, const pl_line &line)
{
    return ofile << line.ref_vt() << ' ' << line.direction();
}

ostream & operator<<(ostream &ofile, const pl_disc &disc)
{
    return ofile << disc.center() << ' ' << disc.radius();
}

ostream & operator<<(ostream &ofile, const pl_box &box)
{
    if (box.is_empty())
	ofile << "1 1 -1 -1";
    else
	ofile << box.xmin() << ' '
	      << box.ymin() << ' '
	      << box.xmax() << ' '
	      << box.ymax();
    return ofile;
}

ostream & operator<<(ostream &ofile, const pl_arc &arc)
{
    return ofile << arc.center() << ' ' << arc.radius() << ' '
		 << arc.begin_angle() << ' '  << arc.enclosed_angle() ;
}

ostream & operator<<(ostream &ofile, const pl_pgn_ifc &pgn)
{
    int i;
    ofile << pgn.size();
    for (i=0; i<pgn.size(); i++) {
	ofile << '\n' << pgn.vertex(i);
    }
    return ofile;
}


// **************************************************

istream & operator>>(istream &ifile, pl_angle &angle)
{
    s_coord x;
    ifile >> x;
    angle = x;
    return ifile;
}


istream & operator>>(istream &ifile, pl_vec &vec)
{
    s_coord x, y;
    ifile >> x >> y ;
    vec.set_x(x);
    vec.set_y(y);
    return ifile;
}

istream & operator>>(istream &ifile, pl_unitvec &uvec)
{
    pl_vec vec;
    s_coord l;
    const s_coord epsilon = 0.1;
    ifile >> vec;
    if (!ifile) return ifile;
    l = factorise(uvec, vec) - 1;
    if (l>epsilon || l<-epsilon)
	ifile.clear(ios::badbit|ifile.rdstate());
    return ifile;
}

istream & operator>>(istream &ifile, pl_point &point)
{
    pl_vec vec;
    ifile >> vec;
    if (!ifile) return ifile;
    point = vec;
    return ifile;
}


istream & operator>>(istream &ifile, pl_edge_ifc &edge)
{
    pl_vec vt1, vt2;
    ifile >> vt1 >> vt2;
    edge.make(vt1, vt2);
    return ifile;
}

istream & operator>>(istream &ifile, pl_ray &ray)
{
    pl_vec vt1;
    pl_unitvec uvec;
    ifile >> vt1 >> uvec;
    ray = pl_ray(vt1, uvec);
    return ifile;
}

istream & operator>>(istream &ifile, pl_line &line)
{
    pl_vec vt1;
    pl_unitvec uvec;
    ifile >> vt1 >> uvec;
    line = pl_line(vt1, uvec);
    return ifile;
}

istream & operator>>(istream &ifile, pl_box &box)
{
    pl_vec vt1, vt2;
    ifile >> vt1 >> vt2;
    if (vt1.x() > vt2.x()+0.5)
	box.make_empty();
    else {
	box.add_first_point(vt1);
	box.add_non_first_point(vt2);
    }
    return ifile;
}

istream & operator>>(istream &ifile, pl_disc &disc)
{
    pl_vec vt;
    s_coord radius;
    ifile >> vt >> radius;
    if (radius < 0)
	ifile.clear(ios::badbit|ifile.rdstate());
    if (ifile.good())
	disc = pl_disc(vt, radius);
    return ifile;
}

istream & operator>>(istream &ifile, pl_arc &arc)
{
    pl_vec vt;
    s_coord radius;
    pl_angle start, enclosed;
    ifile >> vt >> radius >> start >> enclosed;
    if (radius < 0)
	ifile.clear(ios::badbit|ifile.rdstate());
    if (ifile.good())
	arc = pl_arc(vt, radius, start, enclosed);
    return ifile;
}

/*
istream & operator>>(istream &ifile, pl_vecreray &r)
{
    int i, sz;
    ifile >> sz;
    if (!ifile)
	return ifile;
    if (sz < 0) {
	ifile.clear(ios::badbit|ifile.rdstate());
	return ifile;
    }
    r.newsize(sz);
    for (i=0; i<sz; i++) {
	ifile >> r[i];
    }
    return ifile;
}

istream & operator>>(istream &ifile, pl_edgereray &r)
{
    int i, sz;
    ifile >> sz;
    if (!ifile)
	return ifile;
    if (sz < 0) {
	ifile.clear(ios::badbit|ifile.rdstate());
	return ifile;
    }
    r.newsize(sz);
    for (i=0; i<sz; i++) {
	ifile >> r[i];
    }
    return ifile;
}
*/

istream & operator>>(istream &ifile, pl_pgn_ifc &pgn)
{
    int i, n;
    pl_vec vt;
    ifile >> n;
    if (!ifile) {
	pgn.make_start(0);
	pgn.make_ready();
	return ifile;
    }
    if (n < 0) {
	ifile.clear(ios::badbit|ifile.rdstate());
	return ifile;
    }
    pgn.make_start(n);
    for (i=0; i<n; i++) {
	ifile >> vt;
	pgn.set_vertex(i, vt);
    }
// The following is risky if the file is at EOF.
//    if (!ifile.good()) {
//	pgn.make_start(0);
//    }
    pgn.make_ready();
    return ifile;
}



