// pl_ogl_widget.C


#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <X11/keysym.h>
#include <X11/StringDefs.h>
#include <GL/GLwMDrawA.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <stdio.h>
#include <stdlib.h>

#include <PLAGEO/ogl_widget.h>
 

// static void stop_on_escape(Widget, XtPointer, XtPointer);

static void draw_scene_callback (Widget, XtPointer,
                                  XtPointer);

static void do_resize(Widget, XtPointer, XtPointer);

static void init_window(Widget, XtPointer, XtPointer);

static void error()
{
    abort();
}


pl_ogl_widget::pl_ogl_widget(Widget parent, char *name, bool draw_immediate)
{
    widget_init(parent, name, draw_immediate);
}

pl_ogl_widget::~pl_ogl_widget()
{
    // destroy _gl_widget
}

void pl_ogl_widget::widget_init(Widget parent, char *name, bool draw_immediate)
{
    Arg args[20];
    int n;
    _glx_context = 0;
    _first_time = true;
    n = 0;
    XtSetArg(args[n], GLwNrgba, 1); n++;
    if (!draw_immediate) {
	XtSetArg(args[n], GLwNdoublebuffer, 1); n++;
    }
    _ogl_widget = GLwCreateMDrawingArea(parent, name, args, n);
    XtAddCallback(_ogl_widget, GLwNexposeCallback,
                   draw_scene_callback, (XtPointer) this);
    XtAddCallback(_ogl_widget, GLwNresizeCallback, do_resize,
                   (XtPointer) this);
    XtAddCallback(_ogl_widget, GLwNginitCallback, init_window,
                   (XtPointer) this);
    XtManageChild (_ogl_widget);
}

void pl_ogl_widget::make_current()
{
    GLwDrawingAreaMakeCurrent(_ogl_widget, _glx_context);
    if (_viewing_changed) {
	init_projection();
    }
}

static void redraw_scene(Widget , const reray<GLuint> & draw_list)
{
    int i;
    for (i=0; i< draw_list.size(); i++) {
	glCallList(draw_list[i]);
    }
    glFlush();
//    glXSwapBuffers (XtDisplay(w), XtWindow(w));
}

/*
static void stop_on_escape(Widget , XtPointer , XtPointer call)
{
    char buffer[1];
    KeySym keysym;
    GLwDrawingAreaCallbackStruct *call_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;
    switch(call_data->event->type)     {
    case KeyRelease:
	// It is necessary to convert the keycode to a
	// keysym before it is possible to check if it is
	// an escape.
	if (XLookupString( (XKeyEvent *) call_data->event,
			 buffer, 1, &keysym,
			 (XComposeStatus *) NULL ) == 1
		    && keysym == (KeySym) XK_Escape)
	    exit(0);
	break;
    default:
	break;
    }
}
*/

static void draw_scene_callback(Widget w, XtPointer client,
    XtPointer call)
{
    GLwDrawingAreaCallbackStruct *call_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;
    pl_ogl_widget *client_data;
    client_data = (pl_ogl_widget *) client;
    client_data->make_current();
    if (client_data->_first_time) {
	client_data->set_viewport(0, 0, call_data->width, call_data->height);
	client_data->_first_time = false;
    }
    redraw_scene(w, client_data->draw_list());
    if (client_data->_cleanpage)
	client_data->pageready();
}

static void do_resize(Widget , XtPointer client, XtPointer call)
{
    GLwDrawingAreaCallbackStruct *call_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;
    pl_ogl_widget *client_data;
    client_data = (pl_ogl_widget *) client;
    client_data->make_current();
    client_data->set_viewport(0, 0, call_data->width, call_data->height);
}

static void init_window(Widget w, XtPointer client,
                        XtPointer)
{
    Arg args[1];
    XVisualInfo *vi;
    pl_ogl_widget *client_data;
    client_data = (pl_ogl_widget *) client;
    XtSetArg(args[0], GLwNvisualInfo, &vi);
    XtGetValues(w, args, 1);
    client_data->_glx_context
	    = glXCreateContext(XtDisplay(w), vi, 0, GL_FALSE);
    if (client_data->_glx_context == 0)
	error();
}


void pl_ogl_widget::clear()
{
    int i;
    make_current();
    for (i=0; i< _draw_list.size(); i++)
	glDeleteLists(_draw_list[i], 1);
    _draw_list.newsize(0);
    GLuint list_index = glGenLists(1);
    if(list_index!=0) {
	glNewList(list_index, GL_COMPILE_AND_EXECUTE);
    }
    pl_ogl_renderer::clear();
    if(list_index!=0) {
	glEndList();
	_draw_list.append(list_index);
    }
}

void pl_ogl_widget::draw(const pl_fixedvec &vt)
{
    make_current();
    GLuint list_index=glGenLists(1);
    if(list_index!=0) {
	glNewList(list_index, GL_COMPILE_AND_EXECUTE);
    }
    pl_ogl_renderer::draw(vt);
    if(list_index!=0) {
	glEndList();
	_draw_list.append(list_index);
    }
}

void pl_ogl_widget::draw(const pl_edge_ifc &edge)
{
    make_current();
    GLuint list_index=glGenLists(1);
    if(list_index!=0) {
	glNewList(list_index, GL_COMPILE_AND_EXECUTE);
    }
    pl_ogl_renderer::draw(edge);
    if(list_index!=0) {
	glEndList();
	_draw_list.append(list_index);
    }
}

void pl_ogl_widget::draw(const pl_pgn_ifc &pgn)
{
    make_current();
    GLuint list_index=glGenLists(1);
    if(list_index!=0) {
	glNewList(list_index, GL_COMPILE_AND_EXECUTE);
    }
    pl_ogl_renderer::draw(pgn);
    if(list_index!=0) {
	glEndList();
	_draw_list.append(list_index);
    }
}

void pl_ogl_widget::draw(const pl_disc &disc)
{
    make_current();
    GLuint list_index=glGenLists(1);
    if(list_index!=0) {
	glNewList(list_index, GL_COMPILE_AND_EXECUTE);
    }
    pl_ogl_renderer::draw(disc);
    if(list_index!=0) {
	glEndList();
	_draw_list.append(list_index);
    }
}

void pl_ogl_widget::draw(const pl_arc &arc)
{
    make_current();
    GLuint list_index=glGenLists(1);
    if(list_index!=0) {
	glNewList(list_index, GL_COMPILE_AND_EXECUTE);
    }
    pl_ogl_renderer::draw(arc);
    if(list_index!=0) {
	glEndList();
	_draw_list.append(list_index);
    }
}

void pl_ogl_widget::draw_text(const pl_fixedvec &vt, const char *text)
{
    make_current();
    GLuint list_index=glGenLists(1);
    if(list_index!=0) {
	glNewList(list_index, GL_COMPILE_AND_EXECUTE);
    }
    pl_ogl_renderer::draw_text(vt, text);
    if(list_index!=0) {
	glEndList();
	_draw_list.append(list_index);
    }
}

void pl_ogl_widget::pageready()
{
    make_current();
    pl_ogl_renderer::pageready();
    GLwDrawingAreaSwapBuffers(_ogl_widget);
}

