#include <bool.h>
#include <PLAGEO/ogl_window.h>

#include <Xm/Xm.h>
#include <Xm/Form.h>

XtAppContext pl_ogl_window::_app_context;
Widget pl_ogl_window::_toplevel;

void pl_ogl_window::global_init()
{
    static bool _initialised = false;
    if (_initialised)
	return;
    _initialised = true;
    static char *dummy_argv[] = {"ogl_window", 0};
    int argc = 1;
    static String fallback_resources[] = {
         "*rgba: TRUE",
//         "*doublebuffer: TRUE",
//         "*allocateBackground: TRUE",
         NULL
    };
    _toplevel = XtAppInitialize(&_app_context, "OglWindow", NULL,
                                0, &argc, dummy_argv,
                                fallback_resources, NULL, 0);
}

pl_ogl_window::pl_ogl_window(char *name, bool draw_immediate)
{
    int n;
    Arg args[2];
    global_init();
    n = 0;
    _shell_widget = XtAppCreateShell(name, "OglWindow",
	    topLevelShellWidgetClass, XtDisplay(_toplevel), args, n);
    widget_init(_shell_widget, name, draw_immediate);
    XtRealizeWidget(_shell_widget);
    do XtAppProcessEvent(_app_context, XtIMAll);
    while (_glx_context == 0 || !XtIsRealized(_shell_widget) || _first_time);
}

pl_ogl_window::~pl_ogl_window()
{
    XtDestroyWidget(_shell_widget);
}

void end_wait(XtPointer client_data, XtIntervalId *)
{
    bool *waitp = (bool *) client_data;
    *waitp = false;
}

void pl_ogl_window::wait(float seconds)
{
    XtIntervalId timer;
    bool waiting = true;
    if (seconds < 0.0) {
	XtAppMainLoop(_app_context);
    } else {
	unsigned long rounded_time = (unsigned long) (1000*seconds);
	if (rounded_time < 1)
	    rounded_time = 1;
	timer = XtAppAddTimeOut(_app_context, rounded_time, end_wait, &waiting);
	do XtAppProcessEvent(_app_context, XtIMAll);
	while (waiting);
    }
}

struct wait_data {
	wait_data() {button_number = -1;}
    int mouse_x, mouse_y;
    int button_number;
};

static void handle_button(Widget , XtPointer client, XtPointer call)
{
    XButtonEvent *button_event;
    GLwDrawingAreaCallbackStruct *call_data;
    wait_data *client_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;
    client_data = (wait_data *) client;
    switch(call_data->event->type)     {
    case ButtonPress:
	button_event = (XButtonEvent *) call_data->event;
	client_data->mouse_x = button_event->x;
	client_data->mouse_y = call_data->height - button_event->y - 1;
	switch (button_event->button) {
	case Button1:
	    client_data->button_number = pl_ogl_window::BUTTON1;
	    break;
	case Button2:
	    client_data->button_number = pl_ogl_window::BUTTON2;
	    break;
	case Button3:
	    client_data->button_number = pl_ogl_window::BUTTON3;
	    break;
	}
	break;
    default:
	break;
    }
}

static void button_timeout(XtPointer client, XtIntervalId *)
{
    wait_data *client_data;
    client_data = (wait_data *) client;
    if (client_data->button_number == -1)
	client_data->button_number = -2;
}

pl_ogl_window::buttons
pl_ogl_window::wait_for_click(pl_vec &pos, float seconds)
{
    XtIntervalId timer;
    wait_data wait_state;
    if (seconds >= 0) {
	unsigned long rounded_time = (unsigned long) (1000*seconds);
	if (rounded_time < 1)
	    rounded_time = 1;
	timer = XtAppAddTimeOut(_app_context, rounded_time,
			button_timeout, (XtPointer) (&wait_state));
    }
    XtAddCallback(_ogl_widget, GLwNinputCallback, handle_button,
                   (XtPointer) (&wait_state));
    do XtAppProcessEvent(_app_context, XtIMAll);
    while (wait_state.button_number == -1);
    if (seconds > 0) {
	XtRemoveTimeOut(timer);
    }
    XtRemoveCallback(_ogl_widget, GLwNinputCallback, handle_button, 
		    (XtPointer) (&wait_state));
    if (wait_state.button_number < 0)
	return pl_ogl_window::TIME_OUT;
    // compute the x and y value in scene coordinates.
    to_scene_coords(pos, wait_state.mouse_x, wait_state.mouse_y);
    return (buttons) (wait_state.button_number);
}
