#include <PLAGEO/base.h>
#include <PLAGEO/onedim.h>
#include <PLAGEO/intervals.h>

const s_coord scoord = 3;
static s_coord big = max_of_type(scoord);
static s_coord littl = min_of_type(scoord);

pl_line::~pl_line() {}

pl_object_type pl_line::type() const
{
    return PL_LINE;
}

pl_box pl_line::bbox() const
{
    static int initialised = 0;
    static pl_box result;
    if (!initialised) {
	initialised = 1;
	result.add_first_point(pl_vec(littl, littl));
	result.add_non_first_point(pl_vec(big, big));
    }
    return result;  
}

void pl_line::transform(const pl_transform &tfm)
{
    _vt = tfm * _vt;
    _d = tfm * _d;
}

void pl_line::translate(const pl_fixedvec &trans)
{
    _vt += trans;
}

void pl_line::scale(s_coord fac)
{
    _vt *= fac;
    if (fac < 0)
	_d = -_d;
}

s_coord pl_line::placeproject(const pl_fixedvec &vec) const
{
    return dot(vec-_vt, _d);
}

s_coord pl_line::dirproject(const pl_fixedvec &vec) const
{
    return dot(vec, _d);
}


pl_ray::~pl_ray() {}

pl_object_type pl_ray::type() const
{
    return PL_RAY;
}

pl_box pl_ray::bbox() const
{
    static int initialised = 0;
    static pl_box result;
    if (!initialised) {
	s_coord xextreme,  yextreme;
	initialised = 1;
	result.add_first_point(_begin_vt);
	xextreme = _d.x() > 0 ? big : littl;
	yextreme = _d.y() > 0 ? big : littl;
	result.add_non_first_point(pl_vec(xextreme, yextreme));
    }
    return result;  
}

void pl_ray::transform(const pl_transform &tfm)
{
    _begin_vt = tfm * _begin_vt;
    _d = tfm * _d;
}

void pl_ray::translate(const pl_fixedvec &trans)
{
    _begin_vt += trans;
}

void pl_ray::scale(s_coord fac)
{
    _begin_vt *= fac;
    if (fac < 0)
	_d = -_d;
}

// ********

pl_object_type pl_edge_ifc::type() const
{
    return PL_EDGE;
}

// return the bounding box of an edge
pl_boundingbox pl_edge_ifc::bbox() const
{
    pl_boundingbox     bb;
    bb.add_first_point(begin_vt());
    bb.add_non_first_point(end_vt());
    return bb;
}

// ********

pl_edge_imp1::~pl_edge_imp1() {}

void pl_edge_imp1::operator=(const pl_edge_ifc &other)
{
    _begin_vt = other.begin_vt();
    _end_vt = other.end_vt();
}

void pl_edge_imp1::operator=(const pl_edge_imp1 &other)
{
    _begin_vt = other._begin_vt;
    _end_vt = other._end_vt;
}

void pl_edge_imp1::make(const pl_fixedvec &v1, const pl_fixedvec &v2)
{
    _begin_vt = v1;
    _end_vt = v2;
}

void pl_edge_imp1::make(const pl_fixedvec &vt, const pl_unitvec &dir, s_coord l)
{
    _begin_vt = vt;
    _end_vt = vt + l*dir;
}

const pl_vec &pl_edge_imp1::begin_vt() const
{ return _begin_vt;}

const pl_vec &pl_edge_imp1::end_vt() const
{ return _end_vt;}

pl_line pl_edge_imp1::supporting_line() const
{return pl_line(_begin_vt,_end_vt);}

s_coord pl_edge_imp1::length() const
{return (_end_vt-_begin_vt).length();}

pl_unitvec pl_edge_imp1::direction() const
{return normalized(_end_vt-_begin_vt); }

void pl_edge_imp1::transform(const pl_transform &tfm)
{
    _begin_vt = tfm * _begin_vt;
    _end_vt = tfm * _end_vt;
}

void pl_edge_imp1::translate(const pl_fixedvec &disp)
{
    _begin_vt += disp;
    _end_vt += disp;
}

void pl_edge_imp1::scale(s_coord fac)
{
    _begin_vt *= fac;
    _end_vt *= fac;
}

// ********

pl_edge_imp2::pl_edge_imp2(const pl_fixedvec &v1, const pl_fixedvec &v2)
    : _end_vt(v2)
{
    pl_unitvec dir;
    _length = factorise(dir, v2-v1);
    _line = pl_line(v1, dir);
}

pl_edge_imp2::pl_edge_imp2(const pl_fixedvec &vt, const pl_unitvec &dir, s_coord l)
    : _line(vt, dir),  _length(l)
{
    _end_vt = vt + l*dir;
}

pl_edge_imp2::pl_edge_imp2(const pl_edge_ifc &e)
{
    _line = e.supporting_line();
    _length = e.length();
    _end_vt = e.end_vt();
}

pl_edge_imp2::~pl_edge_imp2() {}

void pl_edge_imp2::operator=(const pl_edge_ifc &other)
{
    _end_vt = other.end_vt();
    _line = other.supporting_line();
    _length = other.length();
}

void pl_edge_imp2::operator=(const pl_edge_imp2 &other)
{
    _end_vt = other._end_vt;
    _line = other._line;
    _length = other._length;
}

void pl_edge_imp2::make(const pl_fixedvec &v1, const pl_fixedvec &v2)
{
    pl_unitvec dir;
    _length = factorise(dir, v2-v1);
    _line = pl_line(v1, dir);
    _end_vt = v2;
}

void pl_edge_imp2::make(const pl_fixedvec &vt, const pl_unitvec &dir, s_coord l)
{
    _line = pl_line(vt, dir);
    _length = l;
    _end_vt = vt + l*dir;
}

const pl_vec &pl_edge_imp2::begin_vt() const
{ return _line.ref_vt();}

const pl_vec &pl_edge_imp2::end_vt() const
{ return _end_vt; }

pl_line pl_edge_imp2::supporting_line() const
{ return _line; }

s_coord pl_edge_imp2::length() const
{ return _length; }

pl_unitvec pl_edge_imp2::direction() const
{ return _line.direction(); }

void pl_edge_imp2::transform(const pl_transform &tfm)
{
    _line.transform(tfm);
    _length = tfm * _length;
    _end_vt = tfm * _end_vt;
}

void pl_edge_imp2::translate(const pl_fixedvec &disp)
{
    _line.translate(disp);
    _end_vt += disp;
}

void pl_edge_imp2::scale(s_coord fac)
{
    _line.scale(fac);
    _length *= fac;
    _end_vt *= fac;
}

// ************    ******************


pl_vec placeproject(const pl_fixedvec &vec, const pl_line &l)
{
    const pl_vec &vt_on_line = l.ref_vt();
    const pl_unitvec &dir = l.direction();
    return vt_on_line + dot(vec-vt_on_line, dir) * dir;
}

pl_vec dirproject(const pl_fixedvec &vec, const pl_line &l)
{
    const pl_unitvec &dir = l.direction();
    return dot(vec, dir) * dir;
}

bool is_to_the_left(const pl_fixedvec &vec, const pl_line &line)
{
    return cross(line.direction(), vec-line.ref_vt()) > 0;
}

bool is_to_the_right(const pl_fixedvec &vec, const pl_line &line)
{
    return cross(line.direction(), vec-line.ref_vt()) < 0;
}

// does a vertex lie above a line ?
bool is_above(const pl_fixedvec &vec, const pl_line & line)
{
    pl_vec dir(line.direction());
    if (dir.x() < 0 || (dir.x() == 0 && dir.y() < 0))
	return (is_to_the_right(vec, line));
    else
	return (is_to_the_left(vec, line));
}

// does a vertex lie below a line ?
bool is_below(const pl_fixedvec &vec, const pl_line & line)
{
    pl_vec dir(line.direction());
    if (dir.x() < 0 || (dir.x() == 0 && dir.y() < 0))
	return (is_to_the_left(vec, line));
    else
	return (is_to_the_right(vec, line));
}




