#include <assert.h>
#include <math.h>
#include <stdlib.h>
#include <stdarg.h>
#include <PLAGEO/pgn.h>
//#include <PLAGEO/cross1.h>


static const pl_pgn_imp1 dummypgn;

pl_pgn_imp1::~pl_pgn_imp1() {}

void pl_pgn_imp1::operator=(const pl_pgn_imp1 &other)
{
    _vertices = other.all_vertices();
    pl_pgn_cached_edge_imp2::_make(*(pl_pgn_cached_edge_imp2*)&other);
    pl_pgn_cached_bbox::_make(*(pl_pgn_cached_bbox*)&other);
    pl_pgn_cached_area::_make(*(pl_pgn_cached_area*)&other);
    pl_pgn_cached_order::_make(*(pl_pgn_cached_order*)&other);
}

void pl_pgn_imp1::make_start(int n)
{
    _vertices.newsize(n);
}

void pl_pgn_imp1::set_vertex(int i, const pl_vec &vec)
{
    _vertices[i] = vec;
}


int pl_pgn_imp1::size() const
{
     return _vertices.size();
}

const pl_vec & pl_pgn_imp1::vertex(int i) const
{
    return _vertices[i];
}

void pl_pgn_imp1::operator=(const pl_pgn_ifc &other)
{
    int i;
    _vertices.newsize(other.size());
    for (i=0; i < _vertices.size(); i++)
	_vertices[i] = other.vertex(i);
    pl_pgn_cached_edge_imp2::_make(other);
    pl_pgn_cached_bbox::_make(other);
    pl_pgn_cached_area::_make(other);
    pl_pgn_cached_order::_make(other);
}

void pl_pgn_imp1::invert_orientation()
{
    int i, j, sz;
    pl_vec t;
    sz = size()-1;
    for (i=0, j=sz-1; i < j; i++, j--) {
	t = _vertices[i];
	_vertices[i] = _vertices[j];
	_vertices[j] = t;
    }
    pl_pgn_cached_edge_imp2::_invert_orientation();
    pl_pgn_cached_order::_invert_orientation();
    pl_pgn_cached_clockwise::_invert_orientation();
}


void pl_pgn_imp1::translate(const pl_fixedvec &vec)
{
    for (int i = 0; i < size(); i++)
	_vertices[i] +=  vec;
    pl_pgn_cached_edge_imp2::_translate(vec);
    pl_pgn_cached_bbox::_translate(vec);
}

void pl_pgn_imp1::scale(s_coord fac)
{
    int i;
    for (i = 0; i < _vertices.size(); i++)
	_vertices[i] *= fac;
    pl_pgn_cached_edge_imp2::_scale(fac);
    pl_pgn_cached_bbox::_scale(fac);
    pl_pgn_cached_area::_scale(fac);
    pl_pgn_cached_order::_scale(fac);
    pl_pgn_cached_clockwise::_scale(fac);
}

void pl_pgn_imp1::transform(const pl_transform &tfm)
{
    for (int i = 0; i < size(); i++)
	_vertices[i] = tfm * _vertices[i];
    pl_pgn_cached_edge_imp2::_transform(tfm);
    pl_pgn_cached_bbox::_transform(tfm);
    pl_pgn_cached_area::_transform(tfm);
    pl_pgn_cached_order::_transform(tfm);
    pl_pgn_cached_clockwise::_transform(tfm);
}

void pl_pgn_imp1::make_ready()
{
    pl_pgn_cached_edge_imp2::_make_ready();
    pl_pgn_cached_bbox::_make_ready();
    pl_pgn_cached_area::_make_ready();
    pl_pgn_cached_order::_make_ready();
    pl_pgn_cached_ok::_make_ready();
    pl_pgn_cached_convex::_make_ready();
    pl_pgn_cached_clockwise::_make_ready();
}

