#include <assert.h>
#include <PLAGEO/pgn.h>
#include <PLAGEO/cross1.h>

int pl_pgn_vertices::size() const
{
    return _vertices.size();
}
const pl_vec & pl_pgn_vertices::vertex(int i) const
{
    return _vertices[i];
}

void pl_pgn_vertices::operator=(const pl_pgn_ifc &other)
{
    make_start(other.size());
    for (int i=0; i<other.size(); i++)
	set_vertex(i, other.vertex(i));
}

void pl_pgn_vertices::operator=(const pl_pgn_vertices &other)
{
    _vertices = other._vertices;
}

void pl_pgn_vertices::_make(const pl_pgn_ifc &other)
{
    make_start(other.size());
    for (int i=0; i<other.size(); i++)
	set_vertex(i, other.vertex(i));
}

void pl_pgn_vertices::_make(const pl_pgn_vertices &other)
{
    _vertices = other._vertices;
}

void pl_pgn_vertices::_invert_orientation()
{
    int i, j;
    pl_vec t;
    for (i=0, j=_vertices.size()-1; i<j; i++, j--) {
	t = _vertices[i];
	_vertices[i] = _vertices[j];
	_vertices[j] = t;
    }
}

void pl_pgn_vertices::_translate(const pl_fixedvec &vec)
{
    int i;
    for (i=0; i<_vertices.size(); i++)
	_vertices[i] += vec;
}

void pl_pgn_vertices::_scale(s_coord fac)
{
    int i;
    for (i=0; i<_vertices.size(); i++)
	_vertices[i] *= fac;
}

void pl_pgn_vertices::_transform(const pl_transform &tfm)
{
    int i;
    for (i=0; i<_vertices.size(); i++)
	_vertices[i] = tfm*_vertices[i];
}

void pl_pgn_vertices::make_start(int n)
{
    _vertices.newsize(n);
}

void pl_pgn_vertices::set_vertex(int i, const pl_vec &vec)
{
    _vertices[i] = vec;
}

// ********

int pl_pgn_stored_edge_imp2::size() const
{
    return _edges.size();
}

const pl_vec & pl_pgn_stored_edge_imp2::vertex(int i) const
{
    return _edges[i].begin_vt();
}

void pl_pgn_stored_edge_imp2::set_edge(int i) const
{
    ((pl_pgn_stored_edge_imp2*)this)->_cur_edge = i;
}

const pl_edge_ifc & pl_pgn_stored_edge_imp2::cur_edge() const
{
    return _edges[_cur_edge];
}

const pl_unitvec & pl_pgn_stored_edge_imp2::cur_direction() const
{
    return _edges[_cur_edge].direction_by_ref();
}

void pl_pgn_stored_edge_imp2::operator=(const pl_pgn_ifc &other)
{
    make_start(other.size());
    for (int i=0; i<other.size(); i++)
	set_vertex(i, other.vertex(i));
    _make_ready();
}

void pl_pgn_stored_edge_imp2::operator=(const pl_pgn_stored_edge_imp2 &other)
{
    _edges = other._edges;
    _cur_edge = other._cur_edge;
}

void pl_pgn_stored_edge_imp2::_make(const pl_pgn_ifc &other)
{
    make_start(other.size());
    for (int i=0; i<other.size(); i++)
	set_vertex(i, other.vertex(i));
    _make_ready();
}

void pl_pgn_stored_edge_imp2::_make(const pl_pgn_stored_edge_imp2 &other)
{
    _edges = other._edges;
    _cur_edge = other._cur_edge;
}

void pl_pgn_stored_edge_imp2::_invert_orientation()
{
    int i, j, sz;
    sz = _edges.size();
    if (sz == 0)
	return;
    row<pl_edge_imp2> new_edges(sz);
    for (i=0, j= sz-1; i<sz-1; i++, j--)
	new_edges[j].make(_edges[i+1].begin_vt(),
			-_edges[i].direction(), _edges[i].length());
    new_edges[j].make(_edges[0].begin_vt(),
		    -_edges[i].direction(), _edges[i].length());
    new_edges.swap_contents(_edges);
}

void pl_pgn_stored_edge_imp2::_translate(const pl_fixedvec &vec)
{
    int i;
    for (i=0; i<_edges.size(); i++)
	_edges[i].translate(vec);
}

void pl_pgn_stored_edge_imp2::_scale(s_coord fac)
{
    int i;
    for (i=0; i<_edges.size(); i++)
	_edges[i].scale(fac);
}

void pl_pgn_stored_edge_imp2::_transform(const pl_transform &tfm)
{
    int i;
    for (i=0; i<_edges.size(); i++)
	_edges[i].transform(tfm);
}

void pl_pgn_stored_edge_imp2::make_start(int n)
{
    _edges.newsize(n);
}

void pl_pgn_stored_edge_imp2::set_vertex(int i, const pl_vec &vec)
{
    _edges[i].make(vec, pl_xdir, 0);	// dir and length are dummy.
}

void pl_pgn_stored_edge_imp2::_make_ready()
{
    int i;
    if (_edges.size() == 0)
	return;
    for (i=0; i<_edges.size()-1; i++) {
	_edges[i].make(vertex(i), vertex(i+1));
    }
    _edges[i].make(vertex(i), vertex(0));
}

// ********

void pl_pgn_cached_edge_imp2::set_edge(int i) const
{
    ((pl_pgn_cached_edge_imp2*)this)->_cur_edge = i;
}

const pl_edge_ifc & pl_pgn_cached_edge_imp2::cur_edge() const
{
    ((pl_pgn_cached_edge_imp2 *)this)->compute_edges();
    return _edges[_cur_edge];
}

const pl_unitvec & pl_pgn_cached_edge_imp2::cur_direction() const
{
    ((pl_pgn_cached_edge_imp2 *)this)->compute_edges();
    return _edges[_cur_edge].direction_by_ref();
}

void pl_pgn_cached_edge_imp2::operator=(const pl_pgn_ifc &)
{
    _flags.unset(edgesflag);
}

void pl_pgn_cached_edge_imp2::operator=(const pl_pgn_cached_edge_imp2 &)
{
    _flags.unset(edgesflag);
}

void pl_pgn_cached_edge_imp2::_make(const pl_pgn_ifc &)
{
    _flags.unset(edgesflag);
}

void pl_pgn_cached_edge_imp2::_make(const pl_pgn_cached_edge_imp2 &)
{
    _flags.unset(edgesflag);
}

void pl_pgn_cached_edge_imp2::_invert_orientation()
{
    _flags.unset(edgesflag);
}

void pl_pgn_cached_edge_imp2::_translate(const pl_fixedvec &)
{
    _flags.unset(edgesflag);
}

void pl_pgn_cached_edge_imp2::_scale(s_coord)
{
    _flags.unset(edgesflag);
}

void pl_pgn_cached_edge_imp2::_transform(const pl_transform &)
{
    _flags.unset(edgesflag);
}

void pl_pgn_cached_edge_imp2::_make_ready()
{
    _flags.unset(edgesflag);
}

void pl_pgn_cached_edge_imp2::compute_edges()
{
    int i, sz;
    if (_flags.is_set(edgesflag))
	return;
    _flags.set(edgesflag);
    sz = size();
    _edges.newsize(sz);
    if (sz == 0)
	return;
    for (i=0; i<sz-1; i++) {
	_edges[i].make(vertex(i), vertex(i+1));
    }
    _edges[i].make(vertex(i), vertex(0));
}

// ********

void pl_pgn_otf_edge_imp2::set_edge(int i) const
{
    int next = i+1;
    if (next >= size())
	next = 0;
    ((pl_pgn_otf_edge_imp2*)this)->_cur_edge.make(vertex(i), vertex(next));
}

const pl_edge_ifc & pl_pgn_otf_edge_imp2::cur_edge() const
{
    return _cur_edge;
}

const pl_unitvec & pl_pgn_otf_edge_imp2::cur_direction() const
{
    return _cur_edge.direction_by_ref();
}


// ********

void pl_pgn_cached_order::operator=(const pl_pgn_ifc &)
{
    _flags.unset(sortflag);
}

void pl_pgn_cached_order::operator=(const pl_pgn_cached_order &)
{
    _flags.unset(sortflag);
}

void pl_pgn_cached_order::_make(const pl_pgn_ifc &)
{
    _flags.unset(sortflag);
}

void pl_pgn_cached_order::_make(const pl_pgn_cached_order &)
{
    _flags.unset(sortflag);
}

int pl_pgn_cached_order::element(int i) const
{
    sort();
    return _el[i];
}

const pl_vec & pl_pgn_cached_order::velement(int i) const
{
    sort();
    return vertex(_el[i]);
}

int pl_pgn_cached_order::order(int i) const
{
    sort();
    return _ord[i];
}

void pl_pgn_cached_order::sort() const
{
    if (!_flags.is_set(sortflag)) {
	((pl_pgn_cached_order*)this)->_flags.set(sortflag);
	((pl_pgn_cached_order*)this)->ncsort();
    }
}

void pl_pgn_cached_order::_invert_orientation()
{
    _flags.unset(sortflag);
}

void pl_pgn_cached_order::_scale(s_coord fac)
{
    if (fac <= 0)
	_flags.unset(sortflag);
}

void pl_pgn_cached_order::_transform(const pl_transform &)
{
    _flags.unset(sortflag);
}

void pl_pgn_cached_order::_make_ready()
{
    _flags.unset(sortflag);
}

// ********

void pl_pgn_stored_order::operator=(const pl_pgn_ifc &)
{
    ncsort();
}

void pl_pgn_stored_order::operator=(const pl_pgn_stored_order &other)
{
    _el = other._el;
    _ord = other._ord;
}

void pl_pgn_stored_order::_make(const pl_pgn_ifc &)
{
    ncsort();
}

void pl_pgn_stored_order::_make(const pl_pgn_stored_order &other)
{
    _el = other._el;
    _ord = other._ord;
}

int pl_pgn_stored_order::element(int i) const
{
    return _el[i];
}

const pl_vec & pl_pgn_stored_order::velement(int i) const
{
    return vertex(_el[i]);
}

int pl_pgn_stored_order::order(int i) const
{
    return _ord[i];
}

void pl_pgn_stored_order::_invert_orientation()
{
    ncsort();
}

void pl_pgn_stored_order::_scale(s_coord fac)
{
    if (fac <= 0)
	ncsort();
}

void pl_pgn_stored_order::_transform(const pl_transform &)
{
    ncsort();
}

void pl_pgn_stored_order::_make_ready()
{
    ncsort();
}

/*
 * the following alternative of sorting uses the generic sort routine.
 * It is simpler but slower, therefore not chosen.
row<pl_vec> *sortvtcs;

int vertexcomp(unsigned short i, unsigned short j)
{
    return compare((*sortvtcs)[i], (*sortvtcs)[j]);
}

void pl_pgn_stored_order::ncsort()
{
    int i;
    sortvtcs = &_vertices;
    _el.newsize(size());
    _ord.newsize(size());
// initialiseer eerst volgorde array
    for (i = 0; i < size(); i++)
	_el[i] = (unsigned short) i;
    _el.sort(vertexcomp);
    for (i = 0; i < size(); i++)
	_ord[_el[i]] = (unsigned short) i;
}
*/

void pl_pgn_stored_order::ncsort()
{
    int             low[9], high[9];
// plek voor stack van sorteerintervallen
    int             sp = 0;
    unsigned short   t, pivindex;
    int             s, e, i, j;
    pl_vec         pivot;
    _el.newsize(size());
    _ord.newsize(size());
// initialiseer eerst volgorde array
    for (i = 0; i < size(); i++)
	_el[i] = (unsigned short) i;
// zet eerste sorteerinterval op de stack
    low[sp] = 0;
    high[sp] = size() - 1;
    sp++;			// PUSH(0,this->size()-1);
    while (sp != 0) {
	sp--;
	s = low[sp];
	e = high[sp];		// POP(s,e);
    // sorteren van interval >= 3 elementen
	while (e - s >= 2) {
	// bepaal de volgorde van het eerste,tweede en laatste element.
	// swap de grootste waarde naar achteren.
	// gebruik de middelste waarde als pivot om te partitioneren.
	    if (vertex(_el[s]) < vertex(_el[e])) {
		if (vertex(_el[s + 1]) < vertex(_el[e])) {
		    if (vertex(_el[s]) < vertex(_el[s + 1]))
			pivindex = s + 1;
		    else
			pivindex = s;
		} else {
		    t = _el[e];
		    _el[e] = _el[s + 1];
		    _el[s+1] = t;
		    pivindex = s + 1;
		}
	    } else if (vertex(_el[s + 1]) < vertex(_el[e])) {
		t = _el[s];
		_el[s] = _el[e];
		_el[e] = t;
		pivindex = s;
	    } else if (vertex(_el[s]) > vertex(_el[s + 1])) {
		t = _el[s];
		_el[s] = _el[e];
		_el[e] = t;
		pivindex = s + 1;
	    } else {
		t = _el[s + 1];
		_el[s+1] = _el[e];
		_el[e] = t;
		pivindex = s;
	    }
	    pivot = vertex(_el[pivindex]);
	// partitioneer nu het array.
	// plaats vooraan de keys kleiner dan pivot, achteraan de grotere.
	    i = s + 2;
	    j = e - 1;
	    while (i <= j) {
		while (vertex(_el[i]) <= pivot && i<e)
		    i++;
		while (vertex(_el[j]) >= pivot && j>s)
		    j--;
		if (i < j) {
		    t = _el[i];
		    _el[i] = _el[j];
		    _el[j] = t;
		    i++;
		    j--;
		} else {
		    if (i==j) {
		    // this should not happen, but may happen if
		    // vertex[_el[i]] contains NaN.
		    assert(false);
//		    cerr<<"Error detected by PlaGeo:\n"
//			<<"  polygon contains invalid vertex "
//			<<vertex(_el[i])<<".\n";
		    i++;
		    }
		}
	    }
	// plaats de pivotwaarde op goede plek.
	    if (j > pivindex) {
		t = _el[pivindex];
		_el[pivindex] = _el[j];
		_el[j] = t;
		j--;
	    }
	// push het grootste deelinterval op de stack, ga de volgende
	// ronde verder met het kleinste interval.
	    if (e - i > j - s) {
		low[sp] = i;
		high[sp] = e;
		sp++;		// PUSH(i,e);
		e = j;
	    } else {
		low[sp] = s;
		high[sp] = j;
		sp++;		// PUSH(s,j);
		s = i;
	    }
	}
    // interval van grootte 2 sorteren
	if (e - s == 1 && vertex(_el[s]) > vertex(_el[s + 1])) {
	    t = _el[s];
	    _el[s] = _el[e];
	    _el[e] = t;
	}
    }
// nu goedzetten van plaatsarray
    for (i = 0; i < size(); i++)
	_ord[_el[i]] = (unsigned short) i;
}

// ********

pl_box pl_pgn_cached_bbox::bbox() const
{
    if (!_flags.is_set(bbflag)) {
	pl_pgn_cached_bbox *ncthis = (pl_pgn_cached_bbox*) this;
	ncthis->_flags.set(bbflag);
	ncthis->makebbox();
    }
    return _bb;
}

void pl_pgn_cached_bbox::operator=(const pl_pgn_ifc &)
{
    _flags.unset(bbflag);
}

void pl_pgn_cached_bbox::operator=(const pl_pgn_cached_bbox &other)
{
    if (other._flags.is_set(bbflag)) {
	_flags.set(bbflag);
	_bb = other._bb;
	
    } else {
	_flags.unset(bbflag);
    }
}

void pl_pgn_cached_bbox::_make(const pl_pgn_ifc &)
{
    _flags.unset(bbflag);
}

void pl_pgn_cached_bbox::_make(const pl_pgn_cached_bbox &other)
{
    if (other._flags.is_set(bbflag)) {
	_flags.set(bbflag);
	_bb = other._bb;
	
    } else {
	_flags.unset(bbflag);
    }
}

void pl_pgn_cached_bbox::_translate(const pl_fixedvec &vec)
{
    if (_flags.is_set(bbflag))
	_bb.translate(vec);
}

void pl_pgn_cached_bbox::_scale(s_coord fac)
{
    if (_flags.is_set(bbflag))
	_bb.scale(fac);
}

void pl_pgn_cached_bbox::_transform(const pl_transform &)
{
    _flags.unset(bbflag);
}

void pl_pgn_cached_bbox::_make_ready()
{
    _flags.unset(bbflag);
}

// ********

pl_box pl_pgn_stored_bbox::bbox() const
{
    return _bb;
}

void pl_pgn_stored_bbox::operator=(const pl_pgn_ifc &other)
{
    _bb = other.bbox();
}

void pl_pgn_stored_bbox::operator=(const pl_pgn_stored_bbox &other)
{
    _bb = other._bb;
}

void pl_pgn_stored_bbox::_make(const pl_pgn_ifc &other)
{
    _bb = other.bbox();
}

void pl_pgn_stored_bbox::_make(const pl_pgn_stored_bbox &other)
{
    _bb = other._bb;
}

void pl_pgn_stored_bbox::makebbox()
{
    pl_pgn_stored_bbox *ncthis = (pl_pgn_stored_bbox*) this;
    if (size() < 1) {
	ncthis->_bb.make_empty();
	return;
    }
    ncthis->_bb.add_first_point(vertex(0));
    for (int i=1; i<size(); i++) {
	ncthis->_bb.add_non_first_point(vertex(i));
    }
}

void pl_pgn_stored_bbox::_translate(const pl_fixedvec &vec)
{
    _bb.translate(vec);
}

void pl_pgn_stored_bbox::_scale(s_coord fac)
{
    _bb.scale(fac);
}

void pl_pgn_stored_bbox::_transform(const pl_transform &)
{
    makebbox();
}

void pl_pgn_stored_bbox::_make_ready()
{
    makebbox();
}

// ********

s_coord pl_pgn_otf_area::area() const
{
    return ((pl_pgn_otf_area *)this)->compute_area();
}


s_coord pl_pgn_otf_area::compute_area()
{
    int i, maxn;
    s_coord t;
    maxn = size()-1;
    if (maxn <2) {
	return 0;
    }
    t = vertex(maxn).y() * vertex(0).x() - vertex(0).y() * vertex(maxn).x();
    for (i=0; i<maxn; i++)
	t += vertex(i).y() * vertex(i+1).x() - vertex(i+1).y() * vertex(i).x();
    return fabs_pp(t/2);
}

// ********

void pl_pgn_cached_area::operator=(const pl_pgn_ifc &)
{
    _flags.unset(areaflag);
}

void pl_pgn_cached_area::operator=(const pl_pgn_cached_area &other)
{
    if (other._flags.is_set(areaflag)) {
	_flags.set(areaflag);
	((pl_pgn_cached_area*)this)->_area = other._area;
    } else {
	_flags.unset(areaflag);
    }
}

void pl_pgn_cached_area::_make(const pl_pgn_ifc &)
{
    _flags.unset(areaflag);
}

void pl_pgn_cached_area::_make(const pl_pgn_cached_area &other)
{
    if (other._flags.is_set(areaflag)) {
	_flags.set(areaflag);
	((pl_pgn_cached_area*)this)->_area = other._area;
    } else {
	_flags.unset(areaflag);
    }
}

s_coord pl_pgn_cached_area::area() const
{
    if (!_flags.is_set(areaflag)) {
	pl_pgn_cached_area *ncthis = (pl_pgn_cached_area *) this;
	ncthis->_flags.set(areaflag);
	ncthis->_area = ncthis->compute_area();
    }
    return _area;
}

void pl_pgn_cached_area::_scale(s_coord fac)
{
    if(_flags.is_set(areaflag)) {
	((pl_pgn_cached_area*)this)->_area = (fac*fac)*_area;
    }
}

void pl_pgn_cached_area::_transform(const pl_transform &)
{
    _flags.unset(areaflag);
}

void pl_pgn_cached_area::_make_ready()
{
    _flags.unset(areaflag);
}

// ********

void pl_pgn_stored_area::operator=(const pl_pgn_ifc &other)
{
    _area = other.area();
}

void pl_pgn_stored_area::operator=(const pl_pgn_stored_area &other)
{
    _area = other._area;
}

void pl_pgn_stored_area::_make(const pl_pgn_ifc &other)
{
    _area = other.area();
}

void pl_pgn_stored_area::_make(const pl_pgn_stored_area &other)
{
    _area = other._area;
}

s_coord pl_pgn_stored_area::area() const
{
    return _area;
}

void pl_pgn_stored_area::_scale(s_coord fac)
{
    _area = (fac*fac)*_area;
}

void pl_pgn_stored_area::_transform(const pl_transform &)
{
    _area = compute_area();
}

void pl_pgn_stored_area::_make_ready()
{
    _area = compute_area();
}

// ********

bool pl_pgn_otf_ok::is_ok() const
{
    return ((pl_pgn_otf_ok *)this)->compute_ok();
}

struct checklists{
    int endvt;
    int edgeno;
    checklists *next;
};

static checklists *checklist=0;

static bool forbidden_intersection(const pl_pgn_ifc &pgn, const checklists *cl1, const checklists *cl2)
{
    if (cl1->endvt == cl2->endvt)
	return false;
    pgn.set_edge(cl1->edgeno);
    pl_edge edge1 = pgn.cur_edge();
    pgn.set_edge(cl2->edgeno);
    return do_intersect(edge1, pgn.cur_edge());
}

static int insertion_check(const pl_pgn_ifc &pgn, int vnum, int pred, int succ)
{
    checklists *above, *below;
    checklists       *e1, *e2;
// search between which edges 'vnum' lies
    above = 0; below = checklist;
    while (below != 0 &&
	    (pgn.set_edge(below->edgeno), is_below(pgn.vertex(vnum),
	       pl_line(pgn.vertex(below->endvt), pgn.cur_direction())))) {
	above = below; below = below->next;
    }
// insert the two edges starting in vertex vnum
    e1 = new checklists;
    e2 = new checklists;
    e2->next = below;
    e1->next = e2;
    if (above == 0) checklist = e1; else above->next = e1;
// waarom gebruikt volgende regel niet directions ?????
    if (cross(pgn.vertex(pred) - pgn.vertex(vnum),
		pgn.vertex(succ) - pgn.vertex(vnum)) < 0) {
    // vnum-pred lies above vnum-succ
	e1->edgeno = pred;
	e1->endvt = pred;
	e2->edgeno = vnum;
	e2->endvt = succ;
   } else {	
    // vnum-pred lies below vnum-succ
	e1->edgeno = vnum;
	e1->endvt = succ;
	e2->edgeno = pred;
	e2->endvt = pred;
    }
    if (above != 0 && forbidden_intersection(pgn, above, e1))
	return 1;
    if (below != 0 && forbidden_intersection(pgn, below, e2))
	return 1;
    return 0;
}

static int transition_check(const pl_pgn_ifc &pgn, int vnum, int tonum, int edgeno)
{  
    checklists *above, *current;
// search between which edges 'vnum' lies
    above = 0; current = checklist;
    while (current->endvt != vnum) {
	above = current; current = current->next;
    }
    current->endvt = tonum;
    current->edgeno = edgeno;
    if (above != 0 && forbidden_intersection(pgn, above, current))
	return 1;
    if (current->next != 0 && forbidden_intersection(pgn, current, current->next))
	return 1;
    return 0;
}

static int deletion_check(const pl_pgn_ifc &pgn, int vnum)
{
    checklists *above, *current, *below;
// search between which edges 'vnum' lies
    above = 0; current = checklist;
    while (current->endvt != vnum) {
	above = current; current = current->next;
    }
    if (current->next==0 || current->next->endvt != vnum) {
	return 1;
    }
    below = current->next->next;
    if (above == 0)
	checklist = below;
    else
	above->next = below;
    if (above != 0 && below != 0 && forbidden_intersection(pgn, above, below))
	return 1;
    return 0;
}

static int check_event(const pl_pgn_ifc &pgn, int vnum)
{
    int             nb1, nb2, result;
    int             vertextype = 0;
// nb1 and nb2 are the vertices preceding and succeeding 'vnum' in 'pg'
    nb1 = vnum == 0 ? pgn.size() - 1 : vnum - 1;
    nb2 = vnum == pgn.size() - 1 ? 0 : vnum + 1;
// decide what type of vertex 'vnum' is
// if both edges go to right then insertionpoint (type 0)
// if both edges come from the left then deletionpoint (type 3)
// otherwise a transitionpoint (type 1 or 2)
    if (pgn.order(nb1) < pgn.order(vnum))
	vertextype++;
    if (pgn.order(nb2) < pgn.order(vnum))
	vertextype += 2;
    switch (vertextype) {
    case 0:	// vnum is insertionpoint
	result = insertion_check(pgn, vnum, nb1, nb2);
	break;
    case 1:
	result = transition_check(pgn, vnum, nb2, vnum);
	break;
    case 2:
	result = transition_check(pgn, vnum, nb1, nb1);
	break;
    case 3:
	result = deletion_check(pgn, vnum);
	break;
    }
    return result;
}

bool pl_pgn_otf_ok::compute_ok()
{
// cleanup list of edges crossing sweepline
    checklists       *tmp;
    int i=0;
    while (checklist != NULL) {
	tmp = checklist;
	checklist = checklist->next;
	delete          tmp;
    }
    for(i=0; i < size(); i++) {
	if (check_event(*this, element(i))) {
	    return false;
	}
    }
    return true;
}

// ********

void pl_pgn_cached_ok::operator=(const pl_pgn_ifc &)
{
    _flags.unset(okflag);
}

void pl_pgn_cached_ok::operator=(const pl_pgn_cached_ok &other)
{
    if (other._flags.is_set(okflag)) {
	_flags.set(okflag);
	if (other._flags.is_set(is_okf))
	    _flags.set(is_okf);
	else
	    _flags.unset(is_okf);
    } else {
	_flags.unset(okflag);
    }
}

void pl_pgn_cached_ok::_make(const pl_pgn_ifc &)
{
    _flags.unset(okflag);
}

void pl_pgn_cached_ok::_make(const pl_pgn_cached_ok &other)
{
    if (other._flags.is_set(okflag)) {
	_flags.set(okflag);
	if (other._flags.is_set(is_okf))
	    _flags.set(is_okf);
	else
	    _flags.unset(is_okf);
    } else {
	_flags.unset(okflag);
    }
}

bool pl_pgn_cached_ok::is_ok() const
{
    if (!_flags.is_set(okflag)) {
	pl_pgn_cached_ok *ncthis = (pl_pgn_cached_ok*)this;
	ncthis->_flags.set(okflag);
	if (ncthis->compute_ok()) 
	    ncthis->_flags.set(is_okf);
	else
	    ncthis->_flags.unset(is_okf);
    }
    return _flags.is_set(is_okf);
}

void pl_pgn_cached_ok::_scale(s_coord fac)
{
    if (_flags.is_set(okflag) && fac == 0)
	_flags.unset(is_okf);
}

void pl_pgn_cached_ok::_make_ready()
{
    _flags.unset(okflag);
}

// ********

bool pl_pgn_otf_convex::is_convex() const
{
    return ((pl_pgn_otf_convex *)this)->compute_convex();
}

bool pl_pgn_otf_convex::compute_convex()
{
    bool dir;
    pl_unitvec newdirection,  olddirection;
    int i;
    if (size() < 4) {
	return true;
    }
    set_edge(size()-1);
    olddirection = cur_direction();
    set_edge(0);
    newdirection = cur_direction();
    dir = (cross(olddirection, newdirection) < 0);
    for (i=1; i<size(); i++) {
	olddirection = newdirection;
	set_edge(i);
	newdirection = cur_direction();
	if ((cross(olddirection, newdirection) < 0) != dir) {
	    return false;
	}
    }
    return true;
}

// ********

void pl_pgn_cached_convex::operator=(const pl_pgn_ifc &)
{
    _flags.unset(convexflag);
}

void pl_pgn_cached_convex::operator=(const pl_pgn_cached_convex &other)
{
    if (other._flags.is_set(convexflag)) {
	_flags.set(convexflag);
	if (other._flags.is_set(is_convexf))
	    _flags.set(is_convexf);
	else
	    _flags.unset(is_convexf);
    } else {
	_flags.unset(convexflag);
    }
}

void pl_pgn_cached_convex::_make(const pl_pgn_ifc &)
{
    _flags.unset(convexflag);
}

void pl_pgn_cached_convex::_make(const pl_pgn_cached_convex &other)
{
    if (other._flags.is_set(convexflag)) {
	_flags.set(convexflag);
	if (other._flags.is_set(is_convexf))
	    _flags.set(is_convexf);
	else
	    _flags.unset(is_convexf);
    } else {
	_flags.unset(convexflag);
    }
}

bool pl_pgn_cached_convex::is_convex() const
{
    if (!_flags.is_set(convexflag)) {
	pl_pgn_cached_convex *ncthis = (pl_pgn_cached_convex*)this;
	ncthis->_flags.set(convexflag);
	if (ncthis->compute_convex())
	    ncthis->_flags.set(is_convexf);
	else
	    ncthis->_flags.unset(is_convexf);
    }
    return _flags.is_set(is_convexf);
}

void pl_pgn_cached_convex::_make_ready()
{
    _flags.unset(convexflag);
}

// ********

bool pl_pgn_otf_clockwise::is_clockwise() const
{
    return ((pl_pgn_otf_clockwise *)this)->compute_clockwise();
}

// next procedure goes wrong if leftmost vertex is duplicated.
bool pl_pgn_otf_clockwise::compute_clockwise()
{
    int leftnum, prevnum;
    bool clockwise;
    if (size() <= 2)
	return true;		    // degenerate case.
    leftnum = element(0);	    // leftmost vertex.
    prevnum = (leftnum == 0) ? size()-1 : leftnum-1;
    if (vertex(prevnum).x() == vertex(leftnum).x()) {
	clockwise = vertex(prevnum).y() < vertex(leftnum).y();
    } else {
	set_edge(leftnum);
	pl_unitvec ldirection = cur_direction();
	set_edge(prevnum);
	clockwise = (cross(cur_direction(), ldirection) < 0);
    }
    return clockwise;
}

// ********

void pl_pgn_cached_clockwise::operator=(const pl_pgn_ifc &)
{
    _flags.unset(clockwiseflag);
}

void pl_pgn_cached_clockwise::operator=(const pl_pgn_cached_clockwise &other)
{
    if (other._flags.is_set(clockwiseflag)) {
	_flags.set(clockwiseflag);
	if (other._flags.is_set(is_clockwisef))
	    _flags.set(is_clockwisef);
	else
	    _flags.unset(is_clockwisef);
    } else {
	_flags.unset(clockwiseflag);
    }
}

void pl_pgn_cached_clockwise::_make(const pl_pgn_ifc &)
{
    _flags.unset(clockwiseflag);
}

void pl_pgn_cached_clockwise::_make(const pl_pgn_cached_clockwise &other)
{
    if (other._flags.is_set(clockwiseflag)) {
	_flags.set(clockwiseflag);
	if (other._flags.is_set(is_clockwisef))
	    _flags.set(is_clockwisef);
	else
	    _flags.unset(is_clockwisef);
    } else {
	_flags.unset(clockwiseflag);
    }
}

bool pl_pgn_cached_clockwise::is_clockwise() const
{
    if (!_flags.is_set(clockwiseflag)) {
	pl_pgn_cached_clockwise *ncthis = (pl_pgn_cached_clockwise*)this;
	ncthis->_flags.set(clockwiseflag);
	if (ncthis->compute_clockwise())
	    ncthis->_flags.set(is_clockwisef);
	else
	    ncthis->_flags.unset(is_clockwisef);
    }
    return _flags.is_set(is_clockwisef);
}

void pl_pgn_cached_clockwise::_invert_orientation()
{
    _flags.toggle(is_clockwisef);
}

void pl_pgn_cached_clockwise::_scale(s_coord fac)
{
    if (_flags.is_set(clockwiseflag) && fac < 0)
	_flags.toggle(is_clockwisef);
}

void pl_pgn_cached_clockwise::_transform(const pl_transform &)
{
    _flags.unset(clockwiseflag);
}

void pl_pgn_cached_clockwise::_make_ready()
{
    _flags.unset(clockwiseflag);
}

// ********

pl_pgn_imp2::~pl_pgn_imp2()
{}

void pl_pgn_imp2::operator=(const pl_pgn_ifc &other)
{
    pl_pgn_stored_edge_imp2::_make(other);
    pl_pgn_stored_bbox::_make(other);
    pl_pgn_stored_area::_make(other);
    pl_pgn_stored_order::_make(other);
}

void pl_pgn_imp2::operator=(const pl_pgn_imp2 &other)
{
    pl_pgn_stored_edge_imp2::_make(*(pl_pgn_stored_edge_imp2*)&other);
    pl_pgn_stored_bbox::_make(*(pl_pgn_stored_bbox*)&other);
    pl_pgn_stored_area::_make(*(pl_pgn_stored_area*)&other);
    pl_pgn_stored_order::_make(*(pl_pgn_stored_order*)&other);
}

void pl_pgn_imp2::invert_orientation()
{
    pl_pgn_stored_edge_imp2::_invert_orientation();
    pl_pgn_stored_order::_invert_orientation();
}


void pl_pgn_imp2::translate(const pl_fixedvec &vec)
{
    pl_pgn_stored_edge_imp2::_translate(vec);
    pl_pgn_stored_bbox::_translate(vec);
}

void pl_pgn_imp2::scale(s_coord fac)
{
    pl_pgn_stored_edge_imp2::_scale(fac);
    pl_pgn_stored_bbox::_scale(fac);
    pl_pgn_stored_area::_scale(fac);
    pl_pgn_stored_order::_scale(fac);
}

void pl_pgn_imp2::transform(const pl_transform &tfm)
{
    pl_pgn_stored_edge_imp2::_transform(tfm);
    pl_pgn_stored_bbox::_transform(tfm);
    pl_pgn_stored_area::_transform(tfm);
    pl_pgn_stored_order::_transform(tfm);
}

void pl_pgn_imp2::make_ready()
{
    pl_pgn_stored_edge_imp2::_make_ready();
    pl_pgn_stored_bbox::_make_ready();
    pl_pgn_stored_area::_make_ready();
    pl_pgn_stored_order::_make_ready();
}

// ********

pl_pgn_imp3::~pl_pgn_imp3()
{}

void pl_pgn_imp3::operator=(const pl_pgn_ifc &other)
{
    pl_pgn_vertices::_make(other);
    pl_pgn_cached_edge_imp2::_make(other);
    pl_pgn_cached_bbox::_make(other);
    pl_pgn_cached_area::_make(other);
    pl_pgn_cached_order::_make(other);
}

void pl_pgn_imp3::operator=(const pl_pgn_imp3 &other)
{
    pl_pgn_vertices::_make(*(pl_pgn_vertices*)&other);
    pl_pgn_cached_edge_imp2::_make(*(pl_pgn_cached_edge_imp2*)&other);
    pl_pgn_cached_bbox::_make(*(pl_pgn_cached_bbox*)&other);
    pl_pgn_cached_area::_make(*(pl_pgn_cached_area*)&other);
    pl_pgn_cached_order::_make(*(pl_pgn_cached_order*)&other);
}

void pl_pgn_imp3::invert_orientation()
{
    pl_pgn_vertices::_invert_orientation();
    pl_pgn_cached_edge_imp2::_invert_orientation();
    pl_pgn_cached_order::_invert_orientation();
    pl_pgn_cached_clockwise::_invert_orientation();
}


void pl_pgn_imp3::translate(const pl_fixedvec &vec)
{
    pl_pgn_vertices::_translate(vec);
    pl_pgn_cached_edge_imp2::_translate(vec);
    pl_pgn_cached_bbox::_translate(vec);
}

void pl_pgn_imp3::scale(s_coord fac)
{
    pl_pgn_vertices::_scale(fac);
    pl_pgn_cached_edge_imp2::_scale(fac);
    pl_pgn_cached_bbox::_scale(fac);
    pl_pgn_cached_area::_scale(fac);
    pl_pgn_cached_order::_scale(fac);
    pl_pgn_cached_clockwise::_scale(fac);
}

void pl_pgn_imp3::transform(const pl_transform &tfm)
{
    pl_pgn_vertices::_transform(tfm);
    pl_pgn_cached_edge_imp2::_transform(tfm);
    pl_pgn_cached_bbox::_transform(tfm);
    pl_pgn_cached_area::_transform(tfm);
    pl_pgn_cached_order::_transform(tfm);
    pl_pgn_cached_clockwise::_transform(tfm);
}

void pl_pgn_imp3::make_ready()
{
    pl_pgn_cached_edge_imp2::_make_ready();
    pl_pgn_cached_bbox::_make_ready();
    pl_pgn_cached_area::_make_ready();
    pl_pgn_cached_order::_make_ready();
    pl_pgn_cached_ok::_make_ready();
    pl_pgn_cached_convex::_make_ready();
    pl_pgn_cached_clockwise::_make_ready();
}

// ********

pl_pgn_imp2 dummy2;
pl_pgn_imp3 dummy3;
