#include <PLAGEO/coincidence.h>
#include <PLAGEO/sweeplist.h>

#include <stdlib.h>




struct edge_info {
    int		end,		// the index of the endpoint vertex.
		id,		// the ident of the polygon (1 or 2).
		side;		// this edge is an upside or downside of pgn.
				// A down side has pgn above it.
    pl_line	line;		// supporting line of edge.
};

static const int up = 0;
static const int down = 1;

inline edge_info *conv(void *p)
{   return (edge_info *) p; }

static int cmp(const pl_vec &vt, void *info)
{
    return (is_below(vt, conv(info)->line)) ? -1 : 1;
}

static int insertion_event(pl_sweeplist &list, pl_sweep_item *sweep_items,
    const pl_pgn_ifc & pg, int pol_id, int vnum, int nb1, int nb2)
{
// vnum is insertionpoint
    edge_info *e1, *e2;
    pl_sweep_item it1, it2;
    e1 = new edge_info;
    e2 = new edge_info;
// search between which edges 'vnum' lies
    if (list.is_empty()) {
	it1 = list.insert_first(e1);
    } else {
	it1 = list.find_and_insert(pg.vertex(vnum), e1);
    }
    it2 = list.insert_after(it1, e2);
// check if vnum lies inside the other polygon
    pl_sweep_item succ = list.succ(it2);
    edge_info *esucc;
    if (succ != 0) {
	esucc = conv(list.info(succ));
	if (esucc->side == up && esucc->id != pol_id)
	    return 1;   // overlap found.
    }
// insert the two edges starting in vertex vnum
    e1->id = e2->id = pol_id;
    if (succ == 0 || esucc->side == down) {
	e1->side = down;
	e2->side = up;
    } else {
	e1->side = up;
	e2->side = down;
    }
    if (cross(pg.vertex(nb1) - pg.vertex(vnum),
	    pg.vertex(nb2) - pg.vertex(vnum)) < 0) {
    // vnum-nb1 lies above vnum-nb2
	pg.set_edge(nb1);
	e2->line = pg.cur_edge().supporting_line();
	e2->end = nb1;
	sweep_items[nb1] = it2;
	pg.set_edge(vnum);
	e1->line = pg.cur_edge().supporting_line();
	e1->end = nb2;
	sweep_items[nb2] = it1;
    } else {	
    // vnum-nb1 lies below vnum-nb2
	pg.set_edge(vnum);
	e2->line = pg.cur_edge().supporting_line();
	e2->end = nb2;
	sweep_items[nb2] = it2;
	pg.set_edge(nb1);
	e1->line = pg.cur_edge().supporting_line(); // opposite direction ???
//	e1->line = pl_line(pg.vertex(nb1), -pg.direction(nb1));
	e1->end = nb1;
	sweep_items[nb1] = it1;
    }
    return 0;
}

static int transition_event(pl_sweeplist &list, pl_sweep_item *sweep_items,
    const pl_pgn_ifc & pg, int vnum, int nb, const pl_unitvec &dir)
{
    // vnum is transitionpoint: nb0->vnum->nb
    // zoek in lijst tot je edge met vnum als eindpunt vindt.
	pl_sweep_item it;
	edge_info *ei;
	it = sweep_items[vnum];
	ei = conv(list.info(it));
    // als dit een upper edge is, check dan of is_below(vnum,bovenbuur)
    // anders check of above(vnum,benedenbuur)
    // als check faalt return 1;(overlap)
	if (ei->side == up) {
	    pl_sweep_item itprev = list.pred(it);
	    if (itprev != 0
		&& is_above(pg.vertex(vnum), conv(list.info(itprev))->line))
		return 1;
	} else {
	    pl_sweep_item itnext = list.succ(it);
	    if (itnext != NULL
		&& is_below(pg.vertex(vnum), conv(list.info(itnext))->line))
		return 1;
	}
    // vervang edge door edge vnum->nb;
	ei->end = nb;
	ei->line = pl_line(pg.vertex(vnum), dir);
	sweep_items[nb] = it;
	return 0;
}

static int deletion_event(pl_sweeplist &list, pl_sweep_item *sweep_items,
			const pl_pgn_ifc & pg, int pol_id, int vnum)
{
    pl_sweep_item it1, it2;
    edge_info *e1, *e2;
    // it1 is one of the segments to the endpoint.
    it1 = sweep_items[vnum];
    e1 = conv(list.info(it1));
    it2 = list.succ(it1);
    if (it2 == 0
	    || (e2 = conv(list.info(it2)), e2->end != vnum)
	    || e2->id != pol_id) {
	it2 = it1;
	e2 = e1;
	it1 = list.pred(it2);
	if (it1 == 0
		|| (e1 = conv(list.info(it1)), e1->end != vnum)
		|| e1->id != pol_id)
	    return 1; // overlap found.
    }
// it1 is the lower segment to the endpoint.
// it2 is the upper segment to the endpoint.
// check whther is_below(vnum,bovenbuur)
    pl_sweep_item it;
    it = list.pred(it1);
    if (it != 0 && is_above(pg.vertex(vnum), conv(list.info(it))->line))
	return 1;
// check of above(vnum,2*benedenbuur)
    it = list.succ(it2);
    if (it != 0 && is_below(pg.vertex(vnum), conv(list.info(it))->line))
	return 1;
// laat twee edges uit list weg
    list.remove(it1);
    list.remove(it2);
    delete e1;
    delete e2;
    return 0;
}

// handle a vertex encountered by the sweep line
// check if overlap takes place
static int overlap_event(pl_sweeplist &list, pl_sweep_item *sweep_items, 
	const pl_pgn_ifc & pg, int pol_id, int vnum)
{
    int             nb1, nb2;
    int             vertextype = 0;
    pl_vec	    vec;
    
// nb1 and nb2 are the vertices preceding and succeedeing 'vnum' in 'pg'
    nb1 = vnum == 0 ? pg.size() - 1 : vnum - 1;
    nb2 = vnum == pg.size() - 1 ? 0 : vnum + 1;
// decide what type of vertex 'vnum' is
// if both edges go to right then insertionpoint (type 0)
// if both edges come from the left then deletionpoint (type 3)
// otherwise a transitionpoint (type 1 or 2)
    if (pg.order(nb1) < pg.order(vnum))
	vertextype++;
    if (pg.order(nb2) < pg.order(vnum))
	vertextype += 2;
    switch (vertextype) {
    case 0:
    // vnum is insertionpoint
	return insertion_event(list, sweep_items, pg, pol_id, vnum, nb1, nb2);
    case 1:
    // vnum is transitionpoint: nb1->vnum->nb2
	pg.set_edge(vnum);
	return transition_event(list, sweep_items,
		    pg, vnum, nb2, pg.cur_direction());
    case 2:
    // vnum is transitionpoint: nb2->vnum->nb1
	pg.set_edge(vnum);
	return transition_event(list, sweep_items,
		    pg, vnum, nb1, pg.cur_direction());
    case 3:
    // vnum is deletionpoint
	return deletion_event(list, sweep_items, pg, pol_id, vnum);
    }
    return 0;
}

/*
 * do_overlap test op overlappen van twee polygons middels een sweepline alg.
 * de polygonen moeten simpel zijn. tijdens de sweep wordt een lijst met
 * edges bijgehouden. bij vertices worden edges toegevoegd/weggehaald en
 * gecheckt op overlap. als er overlap optreedt kan de ordening van edges in
 * de war raken. dit kan echter geen kwaad. overlap wordt dan toch
 * ontdekt.(linkste overlap)
 */
 
bool do_overlap2(const pl_pgn_ifc & pgn1, const pl_pgn_ifc & pgn2)
{
// eerst bounding-box-test
    const int INITSIZE = 10;
    static int size1 = INITSIZE;
    static int size2 = INITSIZE;
    static pl_sweep_item *sweep_items1 = new pl_sweep_item[INITSIZE];
    static pl_sweep_item *sweep_items2 = new pl_sweep_item[INITSIZE];
    if (!do_overlap(pgn1.bbox(), pgn2.bbox()))
	return false;
    pl_sweeplist list(cmp);
    int l1 = 0, l2 = 0;
    if (pgn1.size() > size1) {
	size1 = pgn1.size();
	delete [] sweep_items1;
	sweep_items1 = new pl_sweep_item[size1];
    }
    if (pgn2.size() > size2) {
	size2 = pgn2.size();
	delete [] sweep_items2;
	sweep_items2 = new pl_sweep_item[size2];
    }
// cleanup list of edges crossing sweepline
    while (l1 < pgn1.size() && l2 < pgn2.size()) {
	if (pgn1.velement(l1) < pgn2.velement(l2)) {
	    if (overlap_event(list, sweep_items1, pgn1, 1, pgn1.element(l1)))
		return true;
	    l1++;
	} else {
	    if (overlap_event(list, sweep_items2, pgn2, 2, pgn2.element(l2)))
		return true;
	    l2++;
	}
    }
    return false;
}

