#include <PLAGEO/drawarea.h>
#include <PLAGEO/psfile.h>
#include <PLAGEO/io.h>

const int STD_IMAGESIZE = 500;
const int STD_X_OFFSET = 20;
const int STD_Y_OFFSET = 20;


void pl_psfile::setcol(int col)
{
    if (col<0 || col>=PL_COLORCOUNT) {
	setcol(0);
	return;
    }
    if (_marked[col] == '\0') {
	int red, green, blue;
	pl_color_components(red, green, blue, col);
	out << "colordict " << col <<" [" << red/255.0 << ' '
	    << green/255.0 << ' ' << blue/255.0 << "] put\n";
 	_marked[col] = '\01';
    }
    out << "colordict " << col << " get aload pop setrgbcolor\n";
}


void pl_psfile::draw(const pl_fixedvec &vt)
{
    page_init();
    float viewing_size = _viewing_xsize>_viewing_ysize ?
	    _viewing_xsize :
	    _viewing_ysize;
    float radius = _vertexsize * viewing_size/1000.0;
    setcol(_linecol);
    switch(_vertexshape) {
    case 0:	// dot
	radius = viewing_size/1000.0;
	// FALLTHROUGH
    case 3:	// box
	out<<(vt.x()-radius)<<' '<<vt.y()-radius<<" moveto\n";
	out<<2*radius<<" 0 rlineto\n";
	out<<"0 "<<2*radius<<" rlineto\n";
	out<<-2*radius<<" 0 rlineto\n";
	out<<"closepath\n";
	out<<"fill\n";
	break;
    case 1:	// cross
    default:
	out<<(vt.x()-radius)<<' '<<vt.y()<<" moveto\n";
	out<<2*radius<<" 0 rlineto\n";
	out<<-radius<<' '<<-radius<<" rmoveto\n";
	out<<"0 "<<2*radius<<" rlineto\n";
	out<<"stroke\n";
	break;
    case 2:	// square
	out<<(vt.x()-radius)<<' '<<vt.y()-radius<<" moveto\n";
	out<<2*radius<<" 0 rlineto\n";
	out<<"0 "<<2*radius<<" rlineto\n";
	out<<-2*radius<<" 0 rlineto\n";
	out<<"closepath\n";
	out<<"stroke\n";
	break;
    }
}

void pl_psfile::draw(const pl_edge_ifc &e)
{
    page_init();
    setcol(_linecol);
    out<<e.begin_vt()<<" moveto "<<e.end_vt()<<" lineto\n";
    out<<"stroke\n";
}

void pl_psfile::draw(const pl_pgn_ifc &pgn)
{
    page_init();
    int i;
    if (pgn.size() == 0)
	return;
    out<<pgn.vertex(0)<<" moveto\n";
    for(i=1; i<pgn.size(); i++)
	out<<pgn.vertex(i)<<" lineto\n";
    out<<"closepath\n";
    if (_areafill== FILL || _areafill== FILLOUT) {
	setcol(_areacol);
	out<<"gsave fill grestore\n";
    }
    if (_areafill== OUTLINE || _areafill== FILLOUT) {
	setcol(_linecol);
	out<<"gsave stroke grestore\n";
    }
    out<<"newpath\n";
}

void pl_psfile::draw(const pl_disc &disc)
{
    page_init();
    out<<" newpath\n";
    out<<disc.center()<<' '<<disc.radius()<<" 0 360 arc\n";
    out<<"closepath\n";
    if (_areafill== FILL || _areafill== FILLOUT) {
	setcol(_areacol);
	out<<"gsave fill grestore\n";
    }
    if (_areafill== OUTLINE || _areafill== FILLOUT) {
	setcol(_linecol);
	out<<"gsave stroke grestore\n";
    }
    out<<"newpath\n";
}

void pl_psfile::draw(const pl_arc &arc)
{
    page_init();
    const s_coord to_deg = 180/M_PI;
    out<<" newpath\n";
    out<<arc.center()<<' '<<arc.radius()<<' '
	<<to_deg*arc.small_angle().value()<<' '
	<<to_deg*arc.big_angle().value()<<" arc\n";
    setcol(_linecol);
    out<<"stroke\n";
}

void pl_psfile::draw_text(const pl_fixedvec &vt, const char *str)
{
    page_init();
    float viewing_size = _viewing_xsize>_viewing_ysize ?
	    _viewing_xsize :
	    _viewing_ysize;
    out<<"/Helvetica findfont\n";
    out<<(_textsize * viewing_size/1000.0)<<" scalefont\n";
    out<<"setfont\n";
    setcol(_textcol);
    out<<vt<<" moveto\n("<<str<<") show\n";
}

pl_psfile::pl_psfile(ostream &os) : out(os)
{
    init_values();
    _x_offset = STD_X_OFFSET;
    _y_offset = STD_Y_OFFSET;
    _image_xsize = STD_IMAGESIZE;
    _image_ysize = STD_IMAGESIZE;
    _pagenumber = 0;
    _newpage = 1;
}

pl_psfile::pl_psfile(ostream &os, int x, int y, int xs, int ys) : out(os)
{
    init_values();
    _x_offset = x;
    _y_offset = y;
    _image_xsize = abs(xs);
    _image_ysize = abs(ys);
    _pagenumber = 0;
    _newpage = 1;
}


pl_psfile::~pl_psfile()
{
    out<<flush;
}

void pl_psfile::clear()
{
    page_init();
    setcol(_backcol);
    out<<_viewing_x<<' '<<_viewing_y<<" moveto\n";
    out<<_viewing_xsize<<" 0 rlineto\n";
    out<<"0 "<<_viewing_ysize<<" rlineto\n";
    out<<-_viewing_xsize<<" 0 rlineto\n";
    out<<"closepath\n";
    out<<"fill\n";
}

void pl_psfile::pageready()
{
    page_init();
    out<<"showpage\n"<<flush;
    _newpage = 1;
}

void pl_psfile::ps_init()
{
    out<<flush;
    out<<"%!PS-Adobe-\n";
    out<<"%%DocumentFonts:\n";
    out<<"%%BoundingBox: " << _x_offset << ' ' << _y_offset << ' ' <<
	_x_offset+_image_xsize<<' '<<_y_offset+_image_ysize<<'\n';
    out<<"%%EndComments\n";
    out<<"/colordict 256 dict def\n";
    for (int i=0; i<PL_COLORCOUNT; i++)
	_marked[i] = '\0';
    out<<"%%EndProlog\n";
}

void pl_psfile::page_init()
{
    if (! _newpage)
	return;
    _newpage = 0;
    if (_pagenumber==0)
	ps_init();
    _pagenumber++;
    out<<"%%Page: "<<_pagenumber<<' '<<_pagenumber<<'\n';
    out<<_x_offset<<' '<<_y_offset<<" translate\n";
    out<<0<<' '<<0<<" moveto\n";
    out<<0<<' '<<_image_ysize<<" lineto\n";
    out<<_image_xsize<<' '<<_image_ysize<<" lineto\n";
    out<<_image_xsize<<' '<<0<<" lineto\n";
    out<<"closepath\n";
    out<<"gsave\n";
    out<<"% Paint background\n";
    setcol(_backcol);
    out<<"fill\n";
    out<<"grestore\n";
    out<<"% To show outline box remove the % from the next line\n";
    out<<"%gsave 0 setgray stroke grestore\n";
    out<<"clip\nnewpath\n";
    float viewing_size = _viewing_xsize>_viewing_ysize ? _viewing_xsize : _viewing_ysize;
    float image_size = _image_xsize>_image_ysize ? _image_xsize : _image_ysize;
    out	<<"currentlinewidth "<<image_size/viewing_size<<" div setlinewidth\n";
    out	<<_image_xsize/_viewing_xsize<<' '
	<<_image_ysize/_viewing_ysize<<" scale\n";
    out	<<-_viewing_x<<' '<<-_viewing_y<<" translate\n";
}
